/*
 * GasTexConverter.h
 *
 *  Created on: Jan 1, 2014
 *      Author: Martin Zak
 */

#ifndef GAS_TEX_CONVERTER_H_
#define GAS_TEX_CONVERTER_H_

#include <ostream>
#include <common/multipleDispatch.hpp>
#include <map>
#include <utility>
#include <vector>
#include "automaton/Automaton.h"
#include "alphabet/Symbol.h"

class GasTexConverter : public std::SingleDispatchFirstStaticParam<void, std::ostream&, automaton::AutomatonBase> {
	static void printTransitionMap( const std::map<std::pair<std::string, std::string>, std::string> transitionMap, std::ostream& out);
	static std::string getStackSymbols(const std::vector<alphabet::Symbol>& stackSymbols);

	static void transitions(const automaton::EpsilonNFA& fsm, std::ostream& out);
	static void transitions(const automaton::MultiInitialStateNFA& fsm, std::ostream& out);
	static void transitions(const automaton::NFA& fsm, std::ostream& out);
	static void transitions(const automaton::DFA& fsm, std::ostream& out);
	static void transitions(const automaton::ExtendedNFA& fsm, std::ostream& out);
	static void transitions(const automaton::CompactNFA& fsm, std::ostream& out);
	static void transitions(const automaton::NFTA& fsm, std::ostream& out);
	static void transitions(const automaton::DFTA& fsm, std::ostream& out);
	static void transitions(const automaton::DPDA& pda, std::ostream& out);
	static void transitions(const automaton::SinglePopDPDA& tm, std::ostream& out);
	static void transitions(const automaton::InputDrivenDPDA& pda, std::ostream& out);
	static void transitions(const automaton::InputDrivenNPDA& pda, std::ostream& out);
	static void transitions(const automaton::VisiblyPushdownDPDA& tm, std::ostream& out);
	static void transitions(const automaton::VisiblyPushdownNPDA& tm, std::ostream& out);
	static void transitions(const automaton::RealTimeHeightDeterministicDPDA& tm, std::ostream& out);
	static void transitions(const automaton::RealTimeHeightDeterministicNPDA& tm, std::ostream& out);
	static void transitions(const automaton::NPDA& pda, std::ostream& out);
	static void transitions(const automaton::SinglePopNPDA& tm, std::ostream& out);
	static void transitions(const automaton::OneTapeDTM& tm, std::ostream& out);
public:
	static void convert(std::ostream& out, const automaton::Automaton& a);

	static void convert(std::ostream& out, const automaton::EpsilonNFA& a);
	static void convert(std::ostream& out, const automaton::MultiInitialStateNFA& a);
	static void convert(std::ostream& out, const automaton::NFA& a);
	static void convert(std::ostream& out, const automaton::DFA& a);
	static void convert(std::ostream& out, const automaton::ExtendedNFA& a);
	static void convert(std::ostream& out, const automaton::CompactNFA& a);
	static void convert(std::ostream& out, const automaton::NFTA& a);
	static void convert(std::ostream& out, const automaton::DFTA& a);
	static void convert(std::ostream& out, const automaton::DPDA& a);
	static void convert(std::ostream& out, const automaton::SinglePopDPDA& a);
	static void convert(std::ostream& out, const automaton::InputDrivenDPDA& a);
	static void convert(std::ostream& out, const automaton::InputDrivenNPDA& a);
	static void convert(std::ostream& out, const automaton::VisiblyPushdownDPDA& a);
	static void convert(std::ostream& out, const automaton::VisiblyPushdownNPDA& a);
	static void convert(std::ostream& out, const automaton::RealTimeHeightDeterministicDPDA& a);
	static void convert(std::ostream& out, const automaton::RealTimeHeightDeterministicNPDA& a);
	static void convert(std::ostream& out, const automaton::NPDA& a);
	static void convert(std::ostream& out, const automaton::SinglePopNPDA& a);
	static void convert(std::ostream& out, const automaton::OneTapeDTM& a);

	static GasTexConverter& getInstance() {
		static GasTexConverter res;
		return res;
	}
};

#endif /* GAS_TEX_CONVERTER_H_ */
