/*
 * TikZConverter.h
 *
 *  Created on: Dec 1, 2015
 *      Author: Jan Travnicek
 */

#ifndef TIKZ_CONVERTER_H_
#define TIKZ_CONVERTER_H_

#include <ostream>
#include <common/multipleDispatch.hpp>

#include <automaton/Automaton.h>
#include <automaton/AutomatonFeatures.h>

#include <map>
#include <utility>

class TikZConverter : public std::SingleDispatchFirstStaticParam < void, std::ostream &, automaton::AutomatonBase > {
	static void transitions ( const automaton::EpsilonNFA & fsm, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::MultiInitialStateNFA & fsm, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::NFA & fsm, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::DFA & fsm, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::ExtendedNFA & fsm, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::CompactNFA & fsm, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::NFTA & fsm, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::DFTA & fsm, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::DPDA & pda, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::SinglePopDPDA & tm, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::InputDrivenDPDA & pda, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::InputDrivenNPDA & pda, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::VisiblyPushdownDPDA & tm, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::VisiblyPushdownNPDA & tm, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::RealTimeHeightDeterministicDPDA & tm, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::RealTimeHeightDeterministicNPDA & tm, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::NPDA & pda, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::SinglePopNPDA & tm, const std::map < automaton::State, int > & states, std::ostream & out );
	static void transitions ( const automaton::OneTapeDTM & tm, const std::map < automaton::State, int > & states, std::ostream & out );

public:
	static void convert ( std::ostream & out, const automaton::Automaton & a );

	static void convert ( std::ostream & out, const automaton::EpsilonNFA & a );
	static void convert ( std::ostream & out, const automaton::MultiInitialStateNFA & a );
	static void convert ( std::ostream & out, const automaton::NFA & a );
	static void convert ( std::ostream & out, const automaton::DFA & a );
	static void convert ( std::ostream & out, const automaton::ExtendedNFA & a );
	static void convert ( std::ostream & out, const automaton::CompactNFA & a );
	static void convert ( std::ostream & out, const automaton::NFTA & a );
	static void convert ( std::ostream & out, const automaton::DFTA & a );
	static void convert ( std::ostream & out, const automaton::DPDA & a );
	static void convert ( std::ostream & out, const automaton::SinglePopDPDA & a );
	static void convert ( std::ostream & out, const automaton::InputDrivenDPDA & a );
	static void convert ( std::ostream & out, const automaton::InputDrivenNPDA & a );
	static void convert ( std::ostream & out, const automaton::VisiblyPushdownDPDA & a );
	static void convert ( std::ostream & out, const automaton::VisiblyPushdownNPDA & a );
	static void convert ( std::ostream & out, const automaton::RealTimeHeightDeterministicDPDA & a );
	static void convert ( std::ostream & out, const automaton::RealTimeHeightDeterministicNPDA & a );
	static void convert ( std::ostream & out, const automaton::NPDA & a );
	static void convert ( std::ostream & out, const automaton::SinglePopNPDA & a );
	static void convert ( std::ostream & out, const automaton::OneTapeDTM & a );

	static TikZConverter & getInstance ( ) {
		static TikZConverter res;

		return res;
	}

};

#endif /* TIKZ_CONVERTER_H_ */
