/*
 * adeterminize.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Tomas Pecka
 */

#include <tclap/CmdLine.h>
#include <common/GlobalData.h>
#include <measure>

#include "automaton/Automaton.h"
#include "factory/XmlDataFactory.hpp"
#include "exception/AlibException.h"
#include "automaton/determinize/Determinize.h"

int main(int argc, char** argv) {
	try {
		TCLAP::CmdLine cmd("Automaton determinize binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Automaton to determinize",		false,	"-",		"file");
		cmd.add( input );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		measurements::start("Overal", measurements::Type::OVERALL);
		measurements::start("Input read", measurements::Type::AUXILIARY);

		std::deque<sax::Token> tokens;
		if(input.isSet()) {
			if(input.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(tokens);
			} else {
				sax::SaxParseInterface::parseFile(input.getValue(), tokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(tokens);
		}

		automaton::Automaton automaton = alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens);

		measurements::end();
		measurements::start("Algorithm", measurements::Type::MAIN);

		automaton::Automaton res = automaton::determinize::Determinize::determinize(automaton);

		measurements::end();
		measurements::start("Output write", measurements::Type::AUXILIARY);

		alib::XmlDataFactory::toStdout(res);

		measurements::end();
		measurements::end();

		if(measure.getValue()) std::clog << measurements::results() << std::endl;

		return 0;
	} catch (const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
