/*
 * BadCharacterShiftTable.cpp
 *
 *  Created on: 5. 11. 2014
 *      Author: Radomir Polach, Tomas Pecka, Jan Travnicek
 */

#include "BadCharacterShiftTable.h"

#include <exception/AlibException.h>
#include <tree/ranked/PrefixRankedBarPattern.h>

namespace arbology {

namespace exact {

std::map < alphabet::RankedSymbol, size_t > BadCharacterShiftTable::bcs ( const tree::RankedTreeWrapper & pattern ) {
	return getInstance ( ).dispatch ( pattern.getData ( ) );
}

std::map < alphabet::RankedSymbol, size_t > BadCharacterShiftTable::bcs ( const tree::PrefixRankedBarPattern & pattern ) {
	const std::set < alphabet::RankedSymbol > & alphabet = pattern.getAlphabet ( );

	std::map < alphabet::RankedSymbol, size_t > bcs;

	 // initialisation of bcs table to the size of the pattern
	for ( const alphabet::RankedSymbol & symbol : alphabet ) {
		if ( ( symbol == pattern.getSubtreeWildcard ( ) ) || ( symbol.getSymbol ( ) == pattern.getVariablesBarSymbol ( ) ) ) continue;

		bcs.insert ( std::make_pair ( symbol, pattern.getContent ( ).size ( ) ) );
	}

	 // find the distance between the end of the pattern and the index
	 // of the last symbol representing the variable
	int lastSOffset = pattern.getContent ( ).size ( );

	for ( unsigned i = 0; i < pattern.getContent ( ).size ( ); i++ )
		if ( pattern.getContent ( )[i] == pattern.getSubtreeWildcard ( ) )
			lastSOffset = pattern.getContent ( ).size ( ) - i - 1;

	// limit the shift by occurrence of the last variable

	for ( const alphabet::RankedSymbol & symbol : alphabet ) {
		if ( ( symbol == pattern.getSubtreeWildcard ( ) ) || ( symbol.getSymbol ( ) == pattern.getVariablesBarSymbol ( ) ) ) continue;

		size_t tmp = lastSOffset;

		if ( symbol.getSymbol ( ) != pattern.getBarSymbol ( ) )
			 // size of the smallest subtree containing given terminal depend
			 // on the arity of the terminal
			tmp += symbol.getRank ( ).getData ( ) * 2;
		else if ( tmp >= 2 )
			 // bar symbols match the variable bar which is one symbol after
			 // the last variable, conditioned because of the case S S| where
			 // the -1 would cause shift by 0 -- illegal
			tmp -= 1;

		if ( bcs[symbol] > tmp )
			bcs[symbol] = tmp;
	}

	 // limit the shift by position of symbols within the pattern
	for ( unsigned i = 0; i < pattern.getContent ( ).size ( ) - 1; i++ ) // last symbol is not concerned
		if ( ( pattern.getContent ( )[i] != pattern.getSubtreeWildcard ( ) ) && ( pattern.getContent ( )[i].getSymbol ( ) != pattern.getVariablesBarSymbol ( ) ) ) {
			size_t tmp = pattern.getContent ( ).size ( ) - i - 1;

			if ( bcs[pattern.getContent ( )[i]] > tmp )
				bcs[pattern.getContent ( )[i]] = tmp;
		}

	return bcs;
}

auto BadCharacterShiftTablePrefixRankedBarPattern = BadCharacterShiftTable::RegistratorWrapper < std::map < alphabet::RankedSymbol, size_t >, tree::PrefixRankedBarPattern > ( BadCharacterShiftTable::getInstance ( ), BadCharacterShiftTable::bcs );

} /* namespace exact */

} /* namespace arbology */
