/*
 * ExactPatternMatchingAutomaton.h
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#ifndef _EXACT_PATTERN_MATCHING_AUTOMATON_H__
#define _EXACT_PATTERN_MATCHING_AUTOMATON_H__

#include <automaton/Automaton.h>
#include <automaton/AutomatonFeatures.h>
#include <tree/TreeFeatures.h>
#include <common/multipleDispatch.hpp>

namespace arbology {

namespace exact {

class ExactPatternMatchingAutomaton : public std::SingleDispatch < automaton::Automaton, tree::TreeBase > {
public:
	ExactPatternMatchingAutomaton ( ) { }

	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static automaton::Automaton construct ( const tree::Tree & pattern );

	static automaton::NPDA construct ( const tree::PrefixRankedPattern & pattern );
	static automaton::InputDrivenNPDA construct ( const tree::PrefixRankedTree & pattern );
	static automaton::VisiblyPushdownNPDA construct ( const tree::PrefixRankedBarPattern & pattern );
	static automaton::InputDrivenNPDA construct ( const tree::PrefixRankedBarTree & pattern );
	static automaton::NFTA construct ( const tree::RankedTree & pattern );
	static automaton::NFTA construct ( const tree::RankedPattern & pattern );

	static ExactPatternMatchingAutomaton & getInstance ( ) {
		static ExactPatternMatchingAutomaton res;

		return res;
	}

};

} /* namespace exact */

} /* namespace arbology */

#endif /* _EXACT_PATTERN_MATCHING_AUTOMATON_H__ */
