/*
 * NFACommon.cpp
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Vesely
 */

#include "NFACommon.h"
#include "label/LabelSetLabel.h"

#include <deque>
#include <algorithm>
#include <iostream>

namespace automaton {

namespace determinize {

// --------------------------------------------------------------------------------------------------------------------

automaton::State createDFAState ( std::set < automaton::State > nfaStates ) {
	std::set < label::Label > labelSet;

	for ( automaton::State state : std::make_moveable_set ( nfaStates ) )
		labelSet.insert ( std::move ( state.getName ( ) ) );

	return automaton::State ( label::Label ( label::LabelSetLabel ( std::move ( labelSet ) ) ) );
}

std::set < automaton::State > recreateNFAStates ( const automaton::State & dfaState ) {
	 // std::cout << dfaState << std::endl;
	std::set < automaton::State > states;

	for ( const auto & label : static_cast < const label::LabelSetLabel & > ( dfaState.getName ( ).getData ( ) ).getData ( ) )
		 // std::cout << label << std::endl;
		states.insert ( automaton::State ( label ) );

	return states;
}

// --------------------------------------------------------------------------------------------------------------------

dsubset_iterator::dsubset_iterator ( ) {
}

dsubset_iterator::dsubset_iterator ( Iterator it ) : current ( it ) {
}

dsubset_iterator::dsubset_iterator ( const dsubset_iterator & it ) : current ( it.current ) {
}

dsubset_iterator & dsubset_iterator::operator =( const dsubset_iterator & it ) {
	current = it.current;
	return * this;
}

dsubset_iterator::iterator_type dsubset_iterator::base ( ) const {
	return current;
}

dsubset_iterator::value_type dsubset_iterator::operator *( ) const {
	return automaton::State ( * current );
}

dsubset_iterator & dsubset_iterator::operator ++( ) {
	++current;
	return * this;
}

dsubset_iterator & dsubset_iterator::operator --( ) {
	--current;
	return * this;
}

dsubset_iterator dsubset_iterator::operator ++( int ) {
	dsubset_iterator temp = * this;

	++current;
	return temp;
}

dsubset_iterator dsubset_iterator::operator --( int ) {
	dsubset_iterator temp = * this;

	--current;
	return temp;
}

bool dsubset_iterator::operator ==( const dsubset_iterator & other ) {
	return this->current == other.current;
}

bool dsubset_iterator::operator !=( const dsubset_iterator & other ) {
	return !( * this == other );
}

dsubset_iterator make_dsubset_iterator ( std::set < label::Label >::iterator it ) {
	return dsubset_iterator ( it );
}

// --------------------------------------------------------------------------------------------------------------------

dsubset_iterator_aux::dsubset_iterator_aux ( const std::set < label::Label > & param ) : theSet ( param ) {
}

// --------------------------------------------------------------------------------------------------------------------

dsubset_iterator dsubset_iterator_aux::begin ( ) {
	return make_dsubset_iterator ( theSet.begin ( ) );
}

dsubset_iterator dsubset_iterator_aux::end ( ) {
	return make_dsubset_iterator ( theSet.end ( ) );
}

// --------------------------------------------------------------------------------------------------------------------

dsubset_iterator_aux make_dsubset_iterator ( const automaton::State & state ) {
	return dsubset_iterator_aux ( static_cast < const label::LabelSetLabel & > ( state.getName ( ).getData ( ) ).getData ( ) );
}

// --------------------------------------------------------------------------------------------------------------------

} /* namespace determinize */

} /* namespace automaton */
