/*
 * NFACommon.h
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Vesely
 */

#ifndef NFA_COMMON_H_
#define NFA_COMMON_H_

#include <automaton/common/State.h>
#include <set>

namespace automaton {

namespace determinize {

// --------------------------------------------------------------------------------------------------------------------

/**
 * Returns existing state from the resulting automaton, if there is one, or creates new one and adds it into
 *  the resulting deterministic automaton.
 *
 * @param originalStates set of states from nondeterministic fsm which represents state of deterministic fsm
 * @return state of deterministic fsm
 */
automaton::State createDFAState ( std::set < automaton::State > nfaStates );

/**
 * Finds states from nondeterministic fsm to which at least one state from given set of states have transition
 *  with given input.
 *
 * @param fromStates set of states from nondeterministic fsm
 * @param input symbol from input alphabet
 * @return set of states from nondeterministic fsm
 */
std::set < automaton::State > recreateNFAStates ( const automaton::State & dfaState );

// --------------------------------------------------------------------------------------------------------------------

class dsubset_iterator {
	typedef std::set < label::Label >::iterator Iterator;
	Iterator current;

public:
	typedef Iterator iterator_type;
	typedef typename std::iterator_traits < Iterator >::iterator_category iterator_category;
	typedef typename std::iterator_traits < Iterator >::difference_type difference_type;
	typedef automaton::State value_type;
	typedef value_type && reference;
	typedef Iterator pointer;

	explicit dsubset_iterator ( );

	explicit dsubset_iterator ( Iterator it );

	dsubset_iterator ( const dsubset_iterator & it );

	dsubset_iterator & operator =( const dsubset_iterator & it );

	iterator_type base ( ) const;

	value_type operator *( ) const;

	dsubset_iterator & operator ++( );

	dsubset_iterator & operator --( );

	dsubset_iterator operator ++( int );

	dsubset_iterator operator --( int );

	bool operator ==( const dsubset_iterator & other );

	bool operator !=( const dsubset_iterator & other );
};

// --------------------------------------------------------------------------------------------------------------------

dsubset_iterator make_dsubset_iterator ( std::set < label::Label >::iterator it );

// --------------------------------------------------------------------------------------------------------------------

class dsubset_iterator_aux {
	const std::set < label::Label > & theSet;

public:
	dsubset_iterator_aux ( const std::set < label::Label > & param );

	dsubset_iterator begin ( );

	dsubset_iterator end ( );
};

// --------------------------------------------------------------------------------------------------------------------

dsubset_iterator_aux make_dsubset_iterator ( const automaton::State & state );

// --------------------------------------------------------------------------------------------------------------------

} /* namespace determinize */

} /* namespace automaton */

#endif /* NFA_COMMON_H_ */
