/*
 * AutomataConcatenation.h
 *
 *  Created on: 20. 11. 2014
 *	  Author: Tomas Pecka
 */

#ifndef AUTOMATA_CONCATENATION_H_
#define AUTOMATA_CONCATENATION_H_

#include <common/multipleDispatch.hpp>
#include <automaton/Automaton.h>
#include <automaton/FSM/EpsilonNFA.h>

namespace automaton {

namespace transform {

/**
 * Concatenates two automata.
 *  - For finite automata A1, A2, we create automaton L accepting L(A1).L(A2) (Melichar, 2.78)
 */
class AutomataConcatenation : public std::PromotingDoubleDispatch<automaton::Automaton, automaton::AutomatonBase> {
public:
	static automaton::Automaton concatenation(const automaton::Automaton& first, const automaton::Automaton& second);

	static automaton::NFA concatenation(const automaton::DFA& first, const automaton::DFA& second);
	static automaton::NFA concatenation(const automaton::NFA& first, const automaton::NFA& second);

	static AutomataConcatenation& getInstance() {
		static AutomataConcatenation res;
		return res;
	}
};

} /* namespace transform */

} /* namespace automaton */

#endif /* AUTOMATA_CONCATENATION_H_ */
