/*
 * ToGrammarRightRG.cpp
 *
 *  Created on: 8. 3. 2014
 *      Author: Tomas Pecka
 */

#include "ToGrammarRightRG.h"

#include <exception/AlibException.h>

namespace grammar {

namespace convert {

grammar::RightRG ToGrammarRightRG::convert(const grammar::Grammar& grammar) {
	return getInstance().dispatch(grammar.getData());
}

grammar::RightRG ToGrammarRightRG::convert(const grammar::LeftRG& grammar) {
	// 1.
	alphabet::Symbol s = alphabet::createUniqueSymbol( grammar.getInitialSymbol( ), grammar.getNonterminalAlphabet(), grammar.getTerminalAlphabet() );

	grammar::RightRG rrg( s );

	for(const auto & nonterminalSymbol : grammar.getNonterminalAlphabet() ) {
		rrg.addNonterminalSymbol( nonterminalSymbol );
	}

	rrg.setTerminalAlphabet( grammar.getTerminalAlphabet( ) );
	rrg.setGeneratesEpsilon( grammar.getGeneratesEpsilon( ) );

	// 2
	for( const auto & rule : grammar.getRules( ) ) {
		const alphabet::Symbol& lhs = rule.first;

		for(const auto & ruleRHS : rule.second ) {
		        if( ruleRHS.is<std::pair<alphabet::Symbol, alphabet::Symbol>>( ) ) {
				const std::pair<alphabet::Symbol, alphabet::Symbol>& rhs = ruleRHS.get<std::pair<alphabet::Symbol, alphabet::Symbol>>();

				alphabet::Symbol leftSide = ( rhs.first );
				std::pair<alphabet::Symbol, alphabet::Symbol> rightSide = std::make_pair( rhs.second, lhs );
				rrg.addRule( leftSide, rightSide );

				if( lhs == grammar.getInitialSymbol( ) ) {
					alphabet::Symbol leftSide = rhs.first;
					alphabet::Symbol rightSide = rhs.second;
					rrg.addRule( leftSide, rightSide );
				}
			} else {
				const alphabet::Symbol& rhs = ruleRHS.get<alphabet::Symbol>();

				alphabet::Symbol leftSide = rrg.getInitialSymbol( );
				std::pair<alphabet::Symbol, alphabet::Symbol> rightSide = std::make_pair( rhs, lhs );
				rrg.addRule( leftSide, rightSide );

				if( lhs == grammar.getInitialSymbol( ) ) {
					alphabet::Symbol leftSide = rrg.getInitialSymbol( );
					alphabet::Symbol rightSide = rhs;
					rrg.addRule( leftSide, rightSide );
				}
			}
		}
	}
	return rrg;
}

auto ToGrammarRightRGLeftRG = ToGrammarRightRG::RegistratorWrapper<grammar::RightRG, grammar::LeftRG>(ToGrammarRightRG::getInstance(), ToGrammarRightRG::convert);

} /* namespace convert */

} /* namespace grammar */
