/*
 * ToRegExpAlgebraic.cpp
 *
 *  Created on: 4. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "ToRegExpAlgebraic.h"

#include "../../equations/LeftRegularEquationSolver.h"
#include "../../equations/RightRegularEquationSolver.h"

#include <exception/AlibException.h>
#include <regexp/unbounded/UnboundedRegExp.h>
#include <regexp/unbounded/UnboundedRegExpElements.h>

namespace grammar {

namespace convert {

regexp::RegExp ToRegExpAlgebraic::convert(const grammar::Grammar& grammar) {
	return getInstance().dispatch(grammar.getData());
}

regexp::RegExp ToRegExpAlgebraic::convert(const grammar::LeftRG& grammar) {
	equations::LeftRegularEquationSolver solver;

	for(const auto& symbol : grammar.getNonterminalAlphabet())
		solver.addSymbol(symbol);

	for(const auto & rule : grammar.getRules()) {
		const alphabet::Symbol& lhs = rule.first;

		for(const auto& ruleRHS : rule.second) {
			if(ruleRHS.is<alphabet::Symbol>()) {
				const alphabet::Symbol& rhs = ruleRHS.get<alphabet::Symbol>();
				solver.addEquation(lhs, regexp::UnboundedRegExpSymbol(rhs));
			} else {
				const std::pair<alphabet::Symbol, alphabet::Symbol>& rhs = ruleRHS.get<std::pair<alphabet::Symbol, alphabet::Symbol>>();
				solver.addEquation(lhs, rhs.first, regexp::UnboundedRegExpSymbol(rhs.second));
			}
		}
	}
	if(grammar.getGeneratesEpsilon())
		solver.addEquation(grammar.getInitialSymbol(), regexp::UnboundedRegExpEpsilon());

	return regexp::RegExp{solver.solve(grammar.getInitialSymbol())};
}

auto ToRegExpAlgebraicLeftRG = ToRegExpAlgebraic::RegistratorWrapper<regexp::RegExp, grammar::LeftRG>(ToRegExpAlgebraic::getInstance(), ToRegExpAlgebraic::convert);

regexp::RegExp ToRegExpAlgebraic::convert(const grammar::RightRG& grammar) {
	equations::RightRegularEquationSolver solver;

	for(const auto& symbol : grammar.getNonterminalAlphabet())
		solver.addSymbol(symbol);

	for(const auto & rule : grammar.getRules()) {
		const alphabet::Symbol& lhs = rule.first;

		for(const auto& ruleRHS : rule.second) {
			if(ruleRHS.is<alphabet::Symbol>()) {
				const alphabet::Symbol& rhs = ruleRHS.get<alphabet::Symbol>();
				solver.addEquation(lhs, regexp::UnboundedRegExpSymbol(rhs));
			} else {
				const std::pair<alphabet::Symbol, alphabet::Symbol>& rhs = ruleRHS.get<std::pair<alphabet::Symbol, alphabet::Symbol>>();
				solver.addEquation(lhs, rhs.second, regexp::UnboundedRegExpSymbol(rhs.first));
			}
		}
	}
	if(grammar.getGeneratesEpsilon())
		solver.addEquation(grammar.getInitialSymbol(), regexp::UnboundedRegExpEpsilon());

	return regexp::RegExp{solver.solve(grammar.getInitialSymbol())};
}

auto ToRegExpAlgebraicRightRG = ToRegExpAlgebraic::RegistratorWrapper<regexp::RegExp, grammar::RightRG>(ToRegExpAlgebraic::getInstance(), ToRegExpAlgebraic::convert);

} /* namespace convert */

} /* namespace grammar */
