/*
 * ExtractRightContext.cpp
 *
 *  Created on: 9. 6. 2015
 *	  Author: Jan Travnicek
 */

#include "ExtractRightContext.h"
#include "First.h"
#include "common/Substitute.h"
#include <grammar/ContextFree/CFG.h>

namespace grammar {

namespace parsing {

void ExtractRightContext::extractRightContext ( grammar::CFG & grammar, const alphabet::Symbol & terminal, const std::set < alphabet::Symbol > & nonterminals ) {
	grammar::CFG res ( grammar.getInitialSymbol ( ) );

	res.setNonterminalAlphabet ( grammar.getNonterminalAlphabet ( ) );
	res.setTerminalAlphabet ( grammar.getTerminalAlphabet ( ) );

	for ( const std::pair < alphabet::Symbol, std::set < std::vector < alphabet::Symbol > > > & rule : grammar.getRules ( ) ) {
		const alphabet::Symbol & lhs = rule.first;

		for ( const std::vector < alphabet::Symbol > & rhs : rule.second ) {
			bool substitued = false;
			if(rhs.size() > 0)
				for ( std::vector < alphabet::Symbol >::const_iterator iter = rhs.begin ( ); iter + 1 != rhs.end ( ); ++iter )
					if ( nonterminals.count ( * iter ) && grammar.getNonterminalAlphabet ( ).count ( * ( iter + 1 ) ) && First::first ( grammar, std::vector < alphabet::Symbol > ( iter + 1, rhs.end ( ) ) ).count ( terminal ) ) {
						Substitute::substitute ( grammar, res, lhs, rhs, iter + 1 );
						substitued = true;
						break;
					}
			if( ! substitued )
				res.addRule ( lhs, rhs );
		}

	}

	grammar = res;
}

} /* namespace parsing */

} /* namespace grammar */
