/*
 * First.h
 *
 *  Created on: 9. 6. 2015
 *	  Author: Tomas Pecka, Jan Travnicek
 */

#ifndef FIRST_H_
#define FIRST_H_

#include <common/multipleDispatch.hpp>
#include <grammar/Grammar.h>
#include <alphabet/Symbol.h>
#include <string/Epsilon.h>
#include <vector>
#include <set>
#include <variant>

namespace grammar {

namespace parsing {

typedef std::map < std::vector < alphabet::Symbol >, std::set < std::variant < alphabet::Symbol, string::Epsilon > > > FirstResult1;
typedef std::SingleDispatch < FirstResult1, grammar::GrammarBase > FirstBase1;

typedef std::set < std::variant < alphabet::Symbol, string::Epsilon > > FirstResult2;
typedef std::SingleDispatchLastStaticParam < FirstResult2, grammar::GrammarBase, const std::vector < alphabet::Symbol > & > FirstBase2;

class First : public FirstBase1, public FirstBase2 {
	static std::set < std::variant < alphabet::Symbol, string::Epsilon > > first ( const std::set < alphabet::Symbol > & terminals, const std::set < alphabet::Symbol > & nonterminals, const std::map < alphabet::Symbol, std::set < std::variant < alphabet::Symbol, string::Epsilon > > > & firstOfNonterminal, const std::vector < alphabet::Symbol > & rhs );

	static std::map < alphabet::Symbol, std::set < std::variant < alphabet::Symbol, string::Epsilon > > > first ( const std::set < alphabet::Symbol > & terminals, const std::set < alphabet::Symbol > & nonterminals, const std::map < alphabet::Symbol, std::set < std::vector < alphabet::Symbol > > > & rules );

	static First & getInstance ( ) {
		static First res;

		return res;
	}

public:
	template < class T >
	static FirstResult1 first ( const T & grammar );

	template < class T >
	static FirstResult2 first ( const T & grammar, const std::vector < alphabet::Symbol > & rhs );

	static FirstResult1 first ( const grammar::Grammar & grammar );

	static FirstResult2 first ( const grammar::Grammar & grammar, const std::vector < alphabet::Symbol > & rhs );

	static FirstBase1 & getInstance1 ( ) {
		return getInstance ( );
	}

	static FirstBase2 & getInstance2 ( ) {
		return getInstance ( );
	}

};

} /* namespace parsing */

} /* namespace grammar */

#endif /* FIRST_H_ */
