/*
 * Follow.h
 *
 *  Created on: 9. 6. 2015
 *	  Author: Tomas Pecka
 */

#ifndef FOLLOW_H_
#define FOLLOW_H_

#include <common/multipleDispatch.hpp>
#include <grammar/Grammar.h>
#include <alphabet/Symbol.h>
#include <string/Epsilon.h>
#include <vector>
#include <set>
#include <variant>

namespace grammar {

namespace parsing {

typedef std::map < alphabet::Symbol, std::set < std::variant < alphabet::Symbol, string::Epsilon > > > FollowResult1;
typedef std::SingleDispatch < FollowResult1, grammar::GrammarBase > FollowBase1;

typedef std::set < std::variant < alphabet::Symbol, string::Epsilon > > FollowResult2;
typedef std::SingleDispatchLastStaticParam < FollowResult2, grammar::GrammarBase, const alphabet::Symbol & > FollowBase2;

class Follow : public FollowBase1, public FollowBase2 {
	template < class T >
	static void follow ( const T & grammar, std::map < alphabet::Symbol, std::set < std::variant < alphabet::Symbol, string::Epsilon > > > & followSet );

	static Follow & getInstance ( ) {
		static Follow res;

		return res;
	}

public:
	static FollowResult1 follow ( const grammar::Grammar & grammar );

	static FollowResult2 follow ( const grammar::Grammar & grammar, const alphabet::Symbol & nt );

	template < class T >
	static FollowResult1 follow ( const T & grammar );

	template < class T >
	static FollowResult2 follow ( const T & grammar, const alphabet::Symbol & nt );

	static FollowBase1 & getInstance1 ( ) {
		return getInstance ( );
	}

	static FollowBase2 & getInstance2 ( ) {
		return getInstance ( );
	}

};

} /* namespace parsing */

} /* namespace grammar */

#endif /* FOLLOW_H_ */
