/*
 * LeftFactorize.cpp
 *
 *  Created on: 9. 6. 2015
 *	  Author: Jan Travnicek
 */

#include "LeftFactorize.h"

#include <grammar/ContextFree/CFG.h>

namespace grammar {

namespace parsing {

void LeftFactorize::leftFactorize ( grammar::CFG & grammar, const alphabet::Symbol & terminal, const alphabet::Symbol & nonterminal ) {
	grammar::CFG res ( grammar.getInitialSymbol ( ) );

	res.setNonterminalAlphabet ( grammar.getNonterminalAlphabet ( ) );
	res.setTerminalAlphabet ( grammar.getTerminalAlphabet ( ) );

	alphabet::Symbol primed = alphabet::createUniqueSymbol ( nonterminal, grammar.getTerminalAlphabet ( ), grammar.getNonterminalAlphabet ( ) );
	res.addNonterminalSymbol ( primed );

	for ( const std::pair < alphabet::Symbol, std::set < std::vector < alphabet::Symbol > > > & rule : grammar.getRules ( ) ) {
		const alphabet::Symbol & lhs = rule.first;

		for ( const std::vector < alphabet::Symbol > & rhs : rule.second ) {
			if ( ( lhs == nonterminal ) && ( rhs.size ( ) > 0 ) && ( rhs[0] == terminal ) ) {
				res.addRule ( lhs, std::vector < alphabet::Symbol > { terminal, primed } );
				res.addRule ( primed, std::vector < alphabet::Symbol > ( rhs.begin ( ) + 1, rhs.end ( ) ) );
			} else {
				res.addRule ( lhs, rhs );
			}
		}
	}

	grammar = res;
}

} /* namespace parsing */

} /* namespace grammar */
