/*
 * NonterminalUnitRuleCycle.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef NONTERMINAL_UNIT_RULE_CYCLE_H_
#define NONTERMINAL_UNIT_RULE_CYCLE_H_

#include <common/multipleDispatch.hpp>

#include <grammar/Grammar.h>
#include <alphabet/Symbol.h>

namespace grammar {

namespace properties {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class NonterminalUnitRuleCycle : public std::SingleDispatchLastStaticParam<std::set<alphabet::Symbol>, grammar::GrammarBase, const alphabet::Symbol&> {
public:
	static std::set<alphabet::Symbol> getNonterminalUnitRuleCycle( const grammar::Grammar & grammar, const alphabet::Symbol& nonterminal );

	/**
	 * Retrieves set N = {B : A->^* B} for given grammar and nonterminal
	 *
	 * Source: Melichar, algorithm 2.6, step 1
	 *
	 * @param grammar grammar
	 * @param nonterminal nonterminal
	 * @return set of nonterminals for which we can be derived from giveUnitRuleCyclen nonterminals in finite number of steps
	 */
	template<class T>
	static std::set<alphabet::Symbol> getNonterminalUnitRuleCycle(const T& grammar, const alphabet::Symbol& nonterminal);

	static NonterminalUnitRuleCycle& getInstance() {
		static NonterminalUnitRuleCycle res;
		return res;
	}
};

} /* namespace properties */

} /* namespace grammar */

#endif /* NONTERMINAL_UNIT_RULE_CYCLE_H_ */
