#ifndef GRAPH_HOPCROFT_TARJAN_H_
#define GRAPH_HOPCROFT_TARJAN_H_

#include <common/multipleDispatch.hpp>

#include <graph/Graph.h>
#include <graph/directed/DirectedGraph.h>
#include <graph/undirected/UndirectedGraph.h>

namespace graph
{

namespace embedding
{

// Computes combinatorial embedding of bidirected biconnected simple planar graph
class HopcroftTarjan : public std::SingleDispatch<std::unordered_map<Node, std::vector<Node>>, graph::GraphBase>
{
public:
	typedef std::unordered_map<Node, std::vector<Node>> Result;

	static Result hopcrofttarjan(const Graph &graph);

	static Result hopcrofttarjan(const DirectedGraph &graph);
	static Result hopcrofttarjan(const UndirectedGraph &graph);

	static HopcroftTarjan &getInstance() {
		static HopcroftTarjan res;
		return res;
	}
};

} // namespace embedding

} // namespace graph

#endif // GRAPH_HOPCROFT_TARJAN_H_
