#include "RandomGraphFactory.h"

#include <random>
#include <algorithm>
#include <unordered_map>
#include <iostream>

#ifndef NDEBUG
#define GEN_GRAPH(x) std::cout << x << std::endl;
#else
#define GEN_GRAPH(x)
#endif

namespace graph
{

namespace generate
{

struct FuncOut
{
	UndirectedGraph graph;
	std::vector<Node> targetNodes; // 3 nodes
	std::vector<Node> sourceNodes; // 3 nodes
};

static int NODE_LABEL;

// G1
static FuncOut gen5_1()
{
	GEN_GRAPH("gen5_1");

	Node n1(NODE_LABEL++);
	Node n2(NODE_LABEL++);
	Node n3(NODE_LABEL++);
	Node n4(NODE_LABEL++);
	Node n5(NODE_LABEL++);

	UndirectedGraph g;
	g.addEdge(UndirectedEdge(n1, n2));
	g.addEdge(UndirectedEdge(n1, n3));
	g.addEdge(UndirectedEdge(n1, n4));
	g.addEdge(UndirectedEdge(n2, n3));
	g.addEdge(UndirectedEdge(n3, n4));
	g.addEdge(UndirectedEdge(n5, n4));
	g.addEdge(UndirectedEdge(n5, n3));
	g.addEdge(UndirectedEdge(n5, n2));
	return {g, {n2, n5, n4}, {n2, n1, n4}};
}

static FuncOut gen6_1()
{
	GEN_GRAPH("gen6_1");

	Node n1(NODE_LABEL++);
	Node n2(NODE_LABEL++);
	Node n3(NODE_LABEL++);
	Node n4(NODE_LABEL++);
	Node n5(NODE_LABEL++);
	Node n6(NODE_LABEL++);

	UndirectedGraph g;
	g.addEdge(UndirectedEdge(n1, n2));
	g.addEdge(UndirectedEdge(n1, n3));
	g.addEdge(UndirectedEdge(n1, n4));
	g.addEdge(UndirectedEdge(n2, n3));
	g.addEdge(UndirectedEdge(n3, n4));
	g.addEdge(UndirectedEdge(n5, n4));
	g.addEdge(UndirectedEdge(n5, n3));
	g.addEdge(UndirectedEdge(n5, n2));
	g.addEdge(UndirectedEdge(n6, n5));
	g.addEdge(UndirectedEdge(n6, n4));
	g.addEdge(UndirectedEdge(n6, n1));
	return {g, {n2, n5, n6}, {n2, n1, n6}};
}

static FuncOut gen7_1()
{
	GEN_GRAPH("gen7_1");

	Node n1(NODE_LABEL++);
	Node n2(NODE_LABEL++);
	Node n3(NODE_LABEL++);
	Node n4(NODE_LABEL++);
	Node n5(NODE_LABEL++);
	Node n6(NODE_LABEL++);
	Node n7(NODE_LABEL++);

	UndirectedGraph g;
	g.addEdge(UndirectedEdge(n1, n2));
	g.addEdge(UndirectedEdge(n1, n3));
	g.addEdge(UndirectedEdge(n1, n4));
	g.addEdge(UndirectedEdge(n2, n3));
	g.addEdge(UndirectedEdge(n3, n4));
	g.addEdge(UndirectedEdge(n5, n4));
	g.addEdge(UndirectedEdge(n5, n3));
	g.addEdge(UndirectedEdge(n5, n2));
	g.addEdge(UndirectedEdge(n6, n5));
	g.addEdge(UndirectedEdge(n6, n4));
	g.addEdge(UndirectedEdge(n6, n1));
	g.addEdge(UndirectedEdge(n7, n2));
	g.addEdge(UndirectedEdge(n7, n1));
	g.addEdge(UndirectedEdge(n7, n6));
	return {g, {n2, n5, n6}, {n2, n7, n6}};
}

static FuncOut gen8_1()
{
	GEN_GRAPH("gen8_1");

	Node n1(NODE_LABEL++);
	Node n2(NODE_LABEL++);
	Node n3(NODE_LABEL++);
	Node n4(NODE_LABEL++);
	Node n5(NODE_LABEL++);
	Node n6(NODE_LABEL++);
	Node n7(NODE_LABEL++);
	Node n8(NODE_LABEL++);

	UndirectedGraph g;
	g.addEdge(UndirectedEdge(n1, n2));
	g.addEdge(UndirectedEdge(n1, n3));
	g.addEdge(UndirectedEdge(n1, n4));
	g.addEdge(UndirectedEdge(n2, n3));
	g.addEdge(UndirectedEdge(n3, n4));
	g.addEdge(UndirectedEdge(n5, n4));
	g.addEdge(UndirectedEdge(n5, n3));
	g.addEdge(UndirectedEdge(n5, n2));
	g.addEdge(UndirectedEdge(n6, n5));
	g.addEdge(UndirectedEdge(n6, n4));
	g.addEdge(UndirectedEdge(n6, n1));
	g.addEdge(UndirectedEdge(n7, n2));
	g.addEdge(UndirectedEdge(n7, n1));
	g.addEdge(UndirectedEdge(n7, n6));
	g.addEdge(UndirectedEdge(n8, n5));
	g.addEdge(UndirectedEdge(n8, n2));
	g.addEdge(UndirectedEdge(n8, n7));
	return {g, {n8, n5, n6}, {n8, n7, n6}};
}

// G2
static FuncOut gen5_2()
{
	GEN_GRAPH("gen5_2");

	Node n1(NODE_LABEL++);
	Node n2(NODE_LABEL++);
	Node n3(NODE_LABEL++);
	Node n4(NODE_LABEL++);
	Node n5(NODE_LABEL++);

	UndirectedGraph g;
	g.addEdge(UndirectedEdge(n1, n5));
	g.addEdge(UndirectedEdge(n1, n3));
	g.addEdge(UndirectedEdge(n1, n4));
	g.addEdge(UndirectedEdge(n1, n2));
	g.addEdge(UndirectedEdge(n2, n4));
	g.addEdge(UndirectedEdge(n3, n4));
	g.addEdge(UndirectedEdge(n3, n5));
	return {g, {n5, n1, n2}, {n5, n3, n4}};
}

static FuncOut gen6_2()
{
	GEN_GRAPH("gen6_2");

	Node n1(NODE_LABEL++);
	Node n2(NODE_LABEL++);
	Node n3(NODE_LABEL++);
	Node n4(NODE_LABEL++);
	Node n5(NODE_LABEL++);
	Node n6(NODE_LABEL++);

	UndirectedGraph g;
	g.addEdge(UndirectedEdge(n1, n5));
	g.addEdge(UndirectedEdge(n1, n3));
	g.addEdge(UndirectedEdge(n1, n4));
	g.addEdge(UndirectedEdge(n1, n2));
	g.addEdge(UndirectedEdge(n2, n4));
	g.addEdge(UndirectedEdge(n3, n4));
	g.addEdge(UndirectedEdge(n3, n5));
	g.addEdge(UndirectedEdge(n6, n5));
	g.addEdge(UndirectedEdge(n6, n3));
	g.addEdge(UndirectedEdge(n6, n4));
	return {g, {n5, n1, n2}, {n5, n6, n4}};
}

static FuncOut gen7_2()
{
	GEN_GRAPH("gen7_2");

	Node n1(NODE_LABEL++);
	Node n2(NODE_LABEL++);
	Node n3(NODE_LABEL++);
	Node n4(NODE_LABEL++);
	Node n5(NODE_LABEL++);
	Node n6(NODE_LABEL++);
	Node n7(NODE_LABEL++);

	UndirectedGraph g;
	g.addEdge(UndirectedEdge(n1, n5));
	g.addEdge(UndirectedEdge(n1, n3));
	g.addEdge(UndirectedEdge(n1, n4));
	g.addEdge(UndirectedEdge(n1, n2));
	g.addEdge(UndirectedEdge(n2, n4));
	g.addEdge(UndirectedEdge(n3, n4));
	g.addEdge(UndirectedEdge(n3, n5));
	g.addEdge(UndirectedEdge(n6, n5));
	g.addEdge(UndirectedEdge(n6, n3));
	g.addEdge(UndirectedEdge(n6, n4));
	g.addEdge(UndirectedEdge(n7, n2));
	g.addEdge(UndirectedEdge(n7, n4));
	g.addEdge(UndirectedEdge(n7, n6));
	return {g, {n5, n1, n2}, {n5, n6, n7}};
}

static FuncOut gen8_2()
{
	GEN_GRAPH("gen8_2");

	Node n1(NODE_LABEL++);
	Node n2(NODE_LABEL++);
	Node n3(NODE_LABEL++);
	Node n4(NODE_LABEL++);
	Node n5(NODE_LABEL++);
	Node n6(NODE_LABEL++);
	Node n7(NODE_LABEL++);
	Node n8(NODE_LABEL++);

	UndirectedGraph g;
	g.addEdge(UndirectedEdge(n1, n5));
	g.addEdge(UndirectedEdge(n1, n3));
	g.addEdge(UndirectedEdge(n1, n4));
	g.addEdge(UndirectedEdge(n1, n2));
	g.addEdge(UndirectedEdge(n2, n4));
	g.addEdge(UndirectedEdge(n3, n4));
	g.addEdge(UndirectedEdge(n3, n5));
	g.addEdge(UndirectedEdge(n6, n5));
	g.addEdge(UndirectedEdge(n6, n3));
	g.addEdge(UndirectedEdge(n6, n4));
	g.addEdge(UndirectedEdge(n7, n2));
	g.addEdge(UndirectedEdge(n7, n4));
	g.addEdge(UndirectedEdge(n7, n6));
	g.addEdge(UndirectedEdge(n8, n5));
	g.addEdge(UndirectedEdge(n8, n1));
	g.addEdge(UndirectedEdge(n8, n7));
	return {g, {n5, n8, n7}, {n5, n6, n7}};
}

// G3
static FuncOut gen5_3()
{
	GEN_GRAPH("gen5_3");

	Node n1(NODE_LABEL++);
	Node n2(NODE_LABEL++);
	Node n3(NODE_LABEL++);
	Node n4(NODE_LABEL++);
	Node n5(NODE_LABEL++);

	UndirectedGraph g;
	g.addEdge(UndirectedEdge(n3, n4));
	g.addEdge(UndirectedEdge(n3, n1));
	g.addEdge(UndirectedEdge(n3, n2));
	g.addEdge(UndirectedEdge(n3, n5));
	g.addEdge(UndirectedEdge(n4, n1));
	g.addEdge(UndirectedEdge(n1, n2));
	g.addEdge(UndirectedEdge(n2, n5));
	return {g, {n1, n4, n3}, {n1, n2, n5}};
}

static FuncOut gen6_3()
{
	GEN_GRAPH("gen6_3");

	Node n1(NODE_LABEL++);
	Node n2(NODE_LABEL++);
	Node n3(NODE_LABEL++);
	Node n4(NODE_LABEL++);
	Node n5(NODE_LABEL++);
	Node n6(NODE_LABEL++);

	UndirectedGraph g;
	g.addEdge(UndirectedEdge(n3, n4));
	g.addEdge(UndirectedEdge(n3, n1));
	g.addEdge(UndirectedEdge(n3, n2));
	g.addEdge(UndirectedEdge(n3, n5));
	g.addEdge(UndirectedEdge(n4, n1));
	g.addEdge(UndirectedEdge(n1, n2));
	g.addEdge(UndirectedEdge(n2, n5));
	g.addEdge(UndirectedEdge(n6, n4));
	g.addEdge(UndirectedEdge(n6, n3));
	g.addEdge(UndirectedEdge(n6, n5));
	return {g, {n1, n4, n6}, {n2, n5, n6}};
}

static FuncOut gen7_3()
{
	GEN_GRAPH("gen7_3");

	Node n1(NODE_LABEL++);
	Node n2(NODE_LABEL++);
	Node n3(NODE_LABEL++);
	Node n4(NODE_LABEL++);
	Node n5(NODE_LABEL++);
	Node n6(NODE_LABEL++);
	Node n7(NODE_LABEL++);

	UndirectedGraph g;
	g.addEdge(UndirectedEdge(n3, n4));
	g.addEdge(UndirectedEdge(n3, n1));
	g.addEdge(UndirectedEdge(n3, n2));
	g.addEdge(UndirectedEdge(n3, n5));
	g.addEdge(UndirectedEdge(n4, n1));
	g.addEdge(UndirectedEdge(n1, n2));
	g.addEdge(UndirectedEdge(n2, n5));
	g.addEdge(UndirectedEdge(n6, n4));
	g.addEdge(UndirectedEdge(n6, n3));
	g.addEdge(UndirectedEdge(n6, n5));
	g.addEdge(UndirectedEdge(n7, n1));
	g.addEdge(UndirectedEdge(n7, n2));
	g.addEdge(UndirectedEdge(n7, n6));
	return {g, {n1, n4, n6}, {n1, n7, n6}};
}

static void mergeGraphs(FuncOut &target, const FuncOut &source)
{
	if (target.targetNodes.empty()) {
		target = source;
		return;
	}

	for (const UndirectedEdge &e : source.graph.getEdges())
		target.graph.addEdge(e);

	target.graph.addEdge(UndirectedEdge(target.targetNodes.at(0), source.sourceNodes.at(0)));
	target.graph.addEdge(UndirectedEdge(target.targetNodes.at(1), source.sourceNodes.at(1)));
	target.graph.addEdge(UndirectedEdge(target.targetNodes.at(2), source.sourceNodes.at(2)));
	target.targetNodes = source.targetNodes;
}

UndirectedGraph RandomGraphFactory::generateUndirectedTriConnectedPlanarGraph(int size)
{
	NODE_LABEL = 0;

	std::unordered_map<size_t, std::vector<std::function<FuncOut()>>> funcs;
	funcs[5] = {gen5_1, gen5_2, gen5_3};
	funcs[6] = {gen6_1, gen6_2, gen6_3};
	funcs[7] = {gen7_1, gen7_2, gen7_3};
	funcs[8] = {gen8_1, gen8_2};
	size_t lastFuncMod = 9;

	FuncOut out;
	int nodes = 0;
	while (nodes < size) {
		size_t pos = std::random_devices::semirandom() % lastFuncMod;
		if (funcs.find(pos) == funcs.end())
			continue;
		mergeGraphs(out, funcs.at(pos).at(std::random_devices::semirandom() % funcs.at(pos).size())());
		nodes += pos;
	}
	return out.graph;
}

UndirectedGraph RandomGraphFactory::generateUndirectedIsomorphicGraph(const UndirectedGraph &graph)
{
	UndirectedGraph out;
	std::unordered_map<Node, Node> mapping;

	for (const Node &n : graph.getNodes()) {
		Node node(std::string(n.getName()) + "i");
		mapping.insert({n, node});
		out.addNode(node);
	}

	for (const UndirectedEdge &e : graph.getEdges()) {
		UndirectedEdge edge(mapping.at(e.getFirstNode()), mapping.at(e.getSecondNode()), e.getName());
		out.addEdge(edge);
	}

	const std::set<UndirectedEdge> &edges = out.getEdges();

	for (size_t i = 0; i < edges.size(); ++i) {
		size_t pos = std::random_devices::semirandom() % edges.size();
		UndirectedEdge e = *std::next(edges.begin(), pos);
		out.removeEdge(e);
		if (std::random_devices::semirandom() % 2)
			e = UndirectedEdge(e.getSecondNode(), e.getFirstNode(), e.getName());
		out.addEdge(e);
	}

	return out;
}

} // namespace generate

} // namespace graph
