#ifndef GRAPH_HOPCROFT_IMPL_H_
#define GRAPH_HOPCROFT_IMPL_H_

#include <list>
#include <vector>
#include <unordered_map>

#include "HopcroftGraph.h"

namespace graph
{

namespace isomorphism
{

namespace hopcroft
{

class HopcroftImpl
{
public:
	explicit HopcroftImpl(Graph *g1, Graph *g2);

	bool testIsomorphism();

//private:

	// Subalgorithms
	int newInteger();
	void saveInteger();
	void restoreInteger();
	std::vector<int> vectorPartitioning(const std::vector<std::vector<int>> &tuples);
	bool circleIsomorphism(std::vector<int> a, std::vector<int> b);
	std::vector<int> circlePartitioning(const std::vector<std::vector<int>> &circles);

	// Reductions
	bool doReductions();

	// A
	bool reductionA();
	bool removeLoops();
	bool removeCorollas();
	bool removeKnots();
	bool removeDegreeOneVertices();
	bool removeSpokeCenters();
	bool removeStars();
	bool removeDumbells();

	// B
	bool reductionB();
	bool replaceClumps();
	bool replaceSkeins();

	// C
	bool reductionC();
	bool removeIsolatedLowDegreeVertices();
	bool removeIsolatedLowDegree(int degree);
	bool removeNonregularGraphsWithNoIsolatedLowDegreeVertices();
	bool removeIsolatedDegree(int degree);
	bool removeFourDegreeException();
	bool removeLowDegreeVerticesWithSolitaryFaces();
	bool removeVertexWithSolitaryFace(int sides);
	bool removeTheLastGeneralCase();
	bool removeAnotherSpecialDegreeFourCase();

	// Final
	bool testByDefinition();

	bool isRegularDegreeTwoGraph();
	bool regularDegreeTwoGraphIsomorphism();

	bool isomorphism_r(std::vector<Vertex*> vertices1, std::vector<Vertex*> vertices2, std::unordered_map<Vertex*, Vertex*> &mapping);
	bool isomorphism_mapping(std::unordered_map<Vertex*, Vertex*> &mapping);

	void dumpGraphs();
	void dumpLabels();

	Graph *m_g1;
	Graph *m_g2;
	bool m_fail;
	int m_integer;
	int m_savedInteger;
};

} // namespace hopcroft

} // namespace isomorphism

} // namespace graph

#endif // GRAPH_HOPCROFT_IMPL_H_
