#ifndef GRAPH_FLOYD_WARSHALL_FORD_H_
#define GRAPH_FLOYD_WARSHALL_FORD_H_

#include <common/multipleDispatch.hpp>
#include <unordered_map>

#include <graph/Graph.h>
#include <graph/directed/DirectedGraph.h>
#include <graph/undirected/UndirectedGraph.h>

namespace graph {

namespace shortestpath {

// Floyd-Warshall only works on graphs without negative-weight cycle
// However it will detect the negative-weight cycle and throw exception
//
// note: negative-weight undirected edge = negative-weight cycle
// note: std::numeric_limits<int>::max() is used as infinity

class FloydWarshall : public std::SingleDispatch<std::unordered_map<Node, std::unordered_map<Node, int>>, graph::GraphBase> {
public:
	typedef std::unordered_map<Node, std::unordered_map<Node, int>> Result;

	static Result floydwarshall(const Graph &graph);

	static Result floydwarshall(const DirectedGraph &graph);
	static Result floydwarshall(const UndirectedGraph &graph);

	static FloydWarshall& getInstance() {
		static FloydWarshall res;
		return res;
	}
};

} // namespace shortestpath

} // namespace graph

#endif // GRAPH_FLOYD_WARSHALL_FORD_H_
