#include "TopologicalSort.h"
#include "../traverse/Dfs.h"

#include <exception/AlibException.h>

namespace graph {

namespace sort {

static TopologicalSort::Result topsort_impl(const DirectedGraph &graph) {
	TopologicalSort::Result s;
	std::unordered_map<Node, bool> visited;

	for (const Node &n : graph.getNodes()) {
		traverse::Dfs::dfs(graph, n, [&s, &visited](const Node &n, const Node&, int, int) {
			if (visited.find(n) != visited.end()) {
				return false;
			}
			visited.insert({n, true});
			s.push_front(n);
			return true;
		});
	}

	return s;
}

TopologicalSort::Result TopologicalSort::topologicalsort(const Graph &graph) {
	return getInstance().dispatch(graph.getData());
}

TopologicalSort::Result TopologicalSort::topologicalsort(const DirectedGraph &graph) {
	return topsort_impl(graph);
}

auto TopologicalSortDirectedGraph = TopologicalSort::RegistratorWrapper<TopologicalSort::Result, graph::DirectedGraph>(TopologicalSort::getInstance(), TopologicalSort::topologicalsort);

} // namespace sort

} // namespace graph
