#include "Dfs.h"

#include <stack>
#include <unordered_map>

namespace graph {

namespace traverse {

struct Data {
	Node start;
	Dfs::Function func;
	Dfs::FunctionExt func2;
};

template <typename T>
static void dfs_impl(const T &graph, const Node &start, Dfs::Function func) {
	std::unordered_map<Node, bool> visited;
	std::stack<Node> s;

	s.push(start);
	visited[start] = true;

	while (!s.empty()) {
		Node n = s.top(); s.pop();

		if (!func(n)) {
			return;
		}

		for (const Node &e : graph.neighbors(n)) {
			if (visited.find(e) == visited.end()) {
				visited[e] = true;
				s.push(e);
			}
		}
	}
}

template <typename T>
static bool dfs2_impl(const T &graph, const Node &n, const Node &p, std::unordered_map<Node, bool> &visited, int &time, Dfs::FunctionExt func) {
	int opened = ++time;
	visited[n] = true;

	for (const Node &e : graph.neighbors(n)) {
		if (visited.find(e) == visited.end()) {
			if (!dfs2_impl(graph, e, n, visited, time, func)) {
				return false;
			}
		}
	}

	return func(n, p, opened, ++time);
}

void Dfs::dfs(const Graph &graph, const Node &start, Dfs::Function func) {
	getInstance().dispatch(graph.getData(), start, func);
}

void Dfs::dfs(const Graph &graph, const Node &start, Dfs::FunctionExt func) {
	getInstance().dispatch(graph.getData(), start, func);
}

void Dfs::dfs(const DirectedGraph &graph, const Node &start, Dfs::Function func) {
	dfs_impl(graph, start, func);
}

auto DfsDirectedGraph1 = Dfs::SingleDispatch2LastStaticParams<void, graph::GraphBase, const Node&, Dfs::Function>::RegistratorWrapper<void, graph::DirectedGraph>(Dfs::getInstance(), Dfs::dfs);

void Dfs::dfs(const DirectedGraph &graph, const Node &start, Dfs::FunctionExt func) {
	int time = 0;
	std::unordered_map<Node, bool> visited;
	dfs2_impl(graph, start, start, visited, time, func);
}

auto DfsDirectedGraph2 = Dfs::SingleDispatch2LastStaticParams<void, graph::GraphBase, const Node&, Dfs::FunctionExt>::RegistratorWrapper<void, graph::DirectedGraph>(Dfs::getInstance(), Dfs::dfs);

void Dfs::dfs(const UndirectedGraph &graph, const Node &start, Dfs::Function func) {
	dfs_impl(graph, start, func);
}

auto DfsUndirectedGraph1 = Dfs::SingleDispatch2LastStaticParams<void, graph::GraphBase, const Node&, Dfs::Function>::RegistratorWrapper<void, graph::UndirectedGraph>(Dfs::getInstance(), Dfs::dfs);

void Dfs::dfs(const UndirectedGraph &graph, const Node &start, Dfs::FunctionExt func) {
	int time = 0;
	std::unordered_map<Node, bool> visited;
	dfs2_impl(graph, start, start, visited, time, func);
}

auto DfsUndirectedGraph2 = Dfs::SingleDispatch2LastStaticParams<void, graph::GraphBase, const Node&, Dfs::FunctionExt>::RegistratorWrapper<void, graph::UndirectedGraph>(Dfs::getInstance(), Dfs::dfs);

} // namespace traverse

} // namespace graph
