/*
 * ExactFactorAutomaton.cpp
 *
 *  Created on: 7. 4. 2015
 *      Author: Jan Travnicek
 */

#include "ExactFactorAutomaton.h"
#include <exception/AlibException.h>
#include <string/LinearString.h>
#include <string/Epsilon.h>

#include <deque>

namespace stringology {

namespace exact {

automaton::Automaton ExactFactorAutomaton::construct(const string::String& text) {
	return getInstance().dispatch(text.getData());
}

automaton::EpsilonNFA ExactFactorAutomaton::construct(const string::LinearString& text) {
	automaton::EpsilonNFA res(automaton::State(0));
	res.addFinalState(automaton::State(0));
	res.setInputAlphabet(text.getAlphabet());
	int i = 1;
	for(const alphabet::Symbol& symbol : text.getContent()) {
		res.addState(automaton::State(i));
		res.addFinalState(automaton::State(i));
		res.addTransition(automaton::State(i-1), symbol, automaton::State(i));
		res.addTransition(automaton::State(0), automaton::State(i));
		i++;
	}
	return res;
}

auto ExactFactorAutomatonLinearString = ExactFactorAutomaton::RegistratorWrapper<automaton::EpsilonNFA, string::LinearString>(ExactFactorAutomaton::getInstance(), ExactFactorAutomaton::construct);

} /* namespace exact */

} /* namespace stringology */
