/*
 * ExactMultiNondeterministicSubsequenceAutomaton.cpp
 *
 *  Created on: 7. 4. 2015
 *      Author: Jan Travnicek
 */

#include "ExactMultiNondeterministicSubsequenceAutomaton.h"
#include <exception/AlibException.h>
#include <string/LinearString.h>
#include <string/Epsilon.h>
#include "../../common/ContainerConverter.hpp"

namespace stringology {

namespace exact {

automaton::Automaton ExactMultiNondeterministicSubsequenceAutomaton::construct(const std::set<string::String>& texts) {
	return automaton::Automaton(ExactMultiNondeterministicSubsequenceAutomaton::construct(common::ContainerConverter<std::set<string::LinearString>, std::set<string::String>, string::LinearString>::convert(texts)));
}

automaton::EpsilonNFA ExactMultiNondeterministicSubsequenceAutomaton::construct(const std::set<string::LinearString>& texts) {
	automaton::EpsilonNFA res(automaton::State(0));
	res.addFinalState(automaton::State(0));

	int j = 0;
	for(const string::LinearString& text : texts) {
		res.addInputSymbols(text.getAlphabet());

		res.addState(automaton::State(0, j));
		res.addFinalState(automaton::State(0, j));
		res.addTransition(automaton::State(0), automaton::State(0, j));

		int i = 1;
		for(const alphabet::Symbol& symbol : text.getContent()) {
			res.addState(automaton::State(i, j));
			res.addFinalState(automaton::State(i, j));

			res.addTransition(automaton::State(i-1, j), symbol, automaton::State(i, j));
			res.addTransition(automaton::State(i-1, j), automaton::State(i, j));
			i++;
		}
		j++;
	}

	return res;
}

} /* namespace exact */

} /* namespace stringology */
