/*
 * ReversedBadCharacterShiftTable.cpp
 *
 *  Created on: 5. 11. 2014
 *      Author: Jan Travnicek
 */

#include "ReversedBadCharacterShiftTable.h"

#include <exception/AlibException.h>
#include <string/LinearString.h>
#include <alphabet/Symbol.h>

namespace stringology {

namespace exact {

std::map < alphabet::Symbol, size_t > ReversedBadCharacterShiftTable::bcs ( const string::String & pattern ) {
	return getInstance ( ).dispatch ( pattern.getData ( ) );
}

std::map < alphabet::Symbol, size_t > ReversedBadCharacterShiftTable::bcs ( const string::LinearString & pattern ) {
	const std::set < alphabet::Symbol > & alphabet = pattern.getAlphabet ( );
	std::map < alphabet::Symbol, size_t > bcs;

	 /* Initialization of BCS to the length of the needle. */
	for ( const auto & symbol : alphabet )
		bcs.insert ( std::make_pair ( symbol, pattern.getContent ( ).size ( ) ) );

	 /* Filling out BCS, ignoring first character. */
	for ( size_t i = pattern.getContent ( ).size ( ) - 1; i > 0; i-- )
		bcs[pattern.getContent ( ).at ( i )] = i;

	return bcs;
}

auto ReversedBadCharacterShiftTableLinearString = ReversedBadCharacterShiftTable::RegistratorWrapper < std::map < alphabet::Symbol, size_t >, string::LinearString > ( ReversedBadCharacterShiftTable::getInstance ( ), ReversedBadCharacterShiftTable::bcs );

} /* namespace exact */

} /* namespace stringology */
