/*
 * ReversedBoyerMooreHorspool.h
 *
 *  Created on: 5. 11. 2014
 *      Author: Jan Travnicek
 */

#ifndef _STRINGOLOGY_REVERSED_BOYER_MOORE_HORSPOOL_H_
#define _STRINGOLOGY_REVERSED_BOYER_MOORE_HORSPOOL_H_

#include <set>
#include <common/multipleDispatch.hpp>
#include <string/String.h>
#include <string/StringFeatures.h>

namespace stringology {

namespace exact {

/**
 * Implementation of BMH for MI(E+\eps)-EVY course 2014 (Reversed)
 * To get rid of zeros in BCS table we ignore last haystack character
 */
class ReversedBoyerMooreHorspool : public std::DoubleDispatch < std::set < unsigned >, string::StringBase, string::StringBase > {
public:
	/**
	 * Search for pattern in linear string.
	 * @return set set of occurences
	 */
	static std::set < unsigned > match ( const string::String & subject, const string::String & pattern );

	static std::set < unsigned > match ( const string::LinearString & subject, const string::LinearString & pattern );

	static ReversedBoyerMooreHorspool & getInstance ( ) {
		static ReversedBoyerMooreHorspool res;

		return res;
	}

};

} /* namespace exact */

} /* namespace stringology */

#endif /* _STRINGOLOGY_REVERSED_BOYER_MOORE_HORSPOOL_H_ */
