#include "GrammarGenerateUpToLength.h"

#include "grammar/generate/GenerateUpToLength.h"

#include "grammar/ContextFree/EpsilonFreeCFG.h"
#include "string/LinearString.h"

#include <factory/StringDataFactory.hpp>

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GrammarGenerateUpToLength, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION( GrammarGenerateUpToLength );

void GrammarGenerateUpToLength::setUp() {
}

void GrammarGenerateUpToLength::tearDown() {
}

void GrammarGenerateUpToLength::testGenerate1() {
	alphabet::Symbol S = alphabet::symbolFrom("S");
	alphabet::Symbol A = alphabet::symbolFrom("A");
	alphabet::Symbol B = alphabet::symbolFrom("B");
	alphabet::Symbol C = alphabet::symbolFrom("C");
	alphabet::Symbol D = alphabet::symbolFrom("D");

	alphabet::Symbol a = alphabet::symbolFrom('a');
	alphabet::Symbol b = alphabet::symbolFrom('b');

	grammar::EpsilonFreeCFG grammar1(S);
	grammar1.setNonterminalAlphabet({S, A, B, C, D});
	grammar1.setTerminalAlphabet({a, b});
	grammar1.setGeneratesEpsilon(false);

	std::set<string::LinearString> strings;

	CPPUNIT_ASSERT(strings == grammar::generate::GenerateUpToLength::generate(grammar1, 5));
}

void GrammarGenerateUpToLength::testGenerate2() {
	alphabet::Symbol S = alphabet::symbolFrom("S");
	alphabet::Symbol A = alphabet::symbolFrom("A");
	alphabet::Symbol B = alphabet::symbolFrom("B");
	alphabet::Symbol C = alphabet::symbolFrom("C");
	alphabet::Symbol D = alphabet::symbolFrom("D");

	alphabet::Symbol a = alphabet::symbolFrom('a');
	alphabet::Symbol b = alphabet::symbolFrom('b');
	alphabet::Symbol c = alphabet::symbolFrom('c');
	alphabet::Symbol d = alphabet::symbolFrom('d');

	grammar::EpsilonFreeCFG grammar1(S);
	grammar1.setNonterminalAlphabet({S, A, B, C, D});
	grammar1.setTerminalAlphabet({a, b, c, d});
	grammar1.addRule({S}, {A, D});
	grammar1.addRule({S}, {A});
	grammar1.addRule({S}, {D});
	grammar1.addRule({A}, {a, A, d});
	grammar1.addRule({A}, {a, d});
	grammar1.addRule({A}, {B, C});
	grammar1.addRule({A}, {B});
	grammar1.addRule({A}, {C});
	grammar1.addRule({B}, {b, B, c});
	grammar1.addRule({B}, {b, c});
	grammar1.addRule({C}, {c, C});
	grammar1.addRule({C}, {c});
	grammar1.addRule({D}, {d, D});
	grammar1.addRule({D}, {d});
	grammar1.setGeneratesEpsilon(true);

	std::set<string::LinearString> strings;
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{c}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{c, c}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{c, c, c}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{d, d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{d, d, d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{a, d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{a, d, d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{b, c}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{b, c, c}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{c, d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{c, c, d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{c, d, d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{a, c, d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{b, c, d}));

	for(const string::LinearString& str : grammar::generate::GenerateUpToLength::generate(grammar1, 3)) {
		std::cout << alib::StringDataFactory::toString(string::String(str)) << std::endl;
	}

	CPPUNIT_ASSERT(strings == grammar::generate::GenerateUpToLength::generate(grammar1, 3));
}

void GrammarGenerateUpToLength::testGenerate3() {
	alphabet::Symbol S = alphabet::symbolFrom("S");
	alphabet::Symbol A = alphabet::symbolFrom("A");
	alphabet::Symbol B = alphabet::symbolFrom("B");
	alphabet::Symbol C = alphabet::symbolFrom("C");
	alphabet::Symbol D = alphabet::symbolFrom("D");

	alphabet::Symbol a = alphabet::symbolFrom('a');
	alphabet::Symbol b = alphabet::symbolFrom('b');
	alphabet::Symbol c = alphabet::symbolFrom('c');
	alphabet::Symbol d = alphabet::symbolFrom('d');

	grammar::EpsilonFreeCFG grammar1(S);
	grammar1.setNonterminalAlphabet({S, A, B, C, D});
	grammar1.setTerminalAlphabet({a, b, c, d});
	grammar1.addRule({S}, {A, D});
	grammar1.addRule({S}, {D});
	grammar1.addRule({A}, {a, A, d});
	grammar1.addRule({A}, {a, d});
	grammar1.addRule({A}, {B, C});
	grammar1.addRule({A}, {B});
	grammar1.addRule({A}, {C});
	grammar1.addRule({B}, {b, B, c});
	grammar1.addRule({B}, {b, c});
	grammar1.addRule({C}, {c, C});
	grammar1.addRule({C}, {c});
	grammar1.addRule({D}, {d, D});
	grammar1.addRule({D}, {d});

	std::set<string::LinearString> strings;
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{d, d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{d, d, d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{a, d, d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{c, d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{c, c, d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{c, d, d}));
	strings.insert(string::LinearString(std::vector<alphabet::Symbol>{b, c, d}));

	for(const string::LinearString& str : grammar::generate::GenerateUpToLength::generate(grammar1, 3)) {
		std::cout << alib::StringDataFactory::toString(string::String(str)) << std::endl;
	}

	CPPUNIT_ASSERT(strings == grammar::generate::GenerateUpToLength::generate(grammar1, 3));
}
