#include "FirstTest.h"

#include "grammar/parsing/First.h"
#include "string/Epsilon.h"
#include "grammar/ContextFree/CFG.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( FirstTest, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( FirstTest );

void FirstTest::setUp ( ) {
}

void FirstTest::tearDown ( ) {
}

void FirstTest::testFirst ( ) {
	{
		alphabet::Symbol nE = alphabet::symbolFrom ( 'E' );
		alphabet::Symbol nT = alphabet::symbolFrom ( 'T' );
		alphabet::Symbol nF = alphabet::symbolFrom ( 'F' );

		alphabet::Symbol tP = alphabet::symbolFrom ( '+' );
		alphabet::Symbol tS = alphabet::symbolFrom ( '*' );
		alphabet::Symbol tL = alphabet::symbolFrom ( '(' );
		alphabet::Symbol tR = alphabet::symbolFrom ( ')' );
		alphabet::Symbol tA = alphabet::symbolFrom ( 'a' );

		grammar::CFG grammar ( nE );
		grammar.setTerminalAlphabet ( std::set < alphabet::Symbol > { tP, tS, tL, tR, tA } );
		grammar.setNonterminalAlphabet ( std::set < alphabet::Symbol > { nE, nT, nF } );

		std::vector < alphabet::Symbol > rhsE1 ( { nE, tP, nT } );
		std::vector < alphabet::Symbol > rhsE2 ( { nT } );
		std::vector < alphabet::Symbol > rhsT1 ( { nT, tS, nF } );
		std::vector < alphabet::Symbol > rhsT2 ( { nF } );
		std::vector < alphabet::Symbol > rhsF1 ( { tA } );
		std::vector < alphabet::Symbol > rhsF2 ( { tL, nE, tR } );

		grammar.addRule ( nE, rhsE1 );
		grammar.addRule ( nE, rhsE2 );
		grammar.addRule ( nT, rhsT1 );
		grammar.addRule ( nT, rhsT2 );
		grammar.addRule ( nF, rhsF1 );
		grammar.addRule ( nF, rhsF2 );

		 // --------------------------------------------------
		std::map < std::vector < alphabet::Symbol >, std::set < std::variant < alphabet::Symbol, string::Epsilon > > > first;

		first[rhsE1] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tA, tL
		};
		first[rhsE2] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tA, tL
		};
		first[rhsT1] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tA, tL
		};
		first[rhsT2] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tA, tL
		};
		first[rhsF1] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tA
		};
		first[rhsF2] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tL
		};

		// --------------------------------------------------

		std::map < std::vector < alphabet::Symbol >, std::set < std::variant < alphabet::Symbol, string::Epsilon > > > firstAlgo;

		for ( const auto & rule : grammar.getRawRules ( ) )
			for ( const auto & rhs : rule.second )
				firstAlgo[rhs] = grammar::parsing::First::first ( grammar, rhs );

		CPPUNIT_ASSERT ( first == firstAlgo );
	}

	{
		alphabet::Symbol nS = alphabet::symbolFrom ( 'S' );
		alphabet::Symbol nA = alphabet::symbolFrom ( 'A' );
		alphabet::Symbol nB = alphabet::symbolFrom ( 'B' );
		alphabet::Symbol nC = alphabet::symbolFrom ( 'C' );
		alphabet::Symbol nD = alphabet::symbolFrom ( 'D' );
		alphabet::Symbol nE = alphabet::symbolFrom ( 'E' );
		alphabet::Symbol nF = alphabet::symbolFrom ( 'F' );

		alphabet::Symbol tA = alphabet::symbolFrom ( 'a' );
		alphabet::Symbol tB = alphabet::symbolFrom ( 'b' );
		alphabet::Symbol tC = alphabet::symbolFrom ( 'c' );
		alphabet::Symbol tD = alphabet::symbolFrom ( 'd' );
		alphabet::Symbol tE = alphabet::symbolFrom ( 'e' );

		grammar::CFG grammar ( nS );
		grammar.setTerminalAlphabet ( std::set < alphabet::Symbol > { tA, tB, tC, tD, tE } );
		grammar.setNonterminalAlphabet ( std::set < alphabet::Symbol > { nS, nA, nB, nC, nD, nE, nF } );

		std::vector < alphabet::Symbol > rhsS1 ( { nB, tD, nS } );
		std::vector < alphabet::Symbol > rhsS2 ( { tD, tD, nC } );
		std::vector < alphabet::Symbol > rhsS3 ( { tC, nA } );
		std::vector < alphabet::Symbol > rhsA1 ( { tA, tE, nE } );
		std::vector < alphabet::Symbol > rhsA2 ( { tB, tB, nE } );
		std::vector < alphabet::Symbol > rhsB1 ( { tA, nF } );
		std::vector < alphabet::Symbol > rhsB2 ( { tB, tB, nD } );
		std::vector < alphabet::Symbol > rhsC1 ( { tA, nB, tD } );
		std::vector < alphabet::Symbol > rhsC2 ( { tE, nA } );
		std::vector < alphabet::Symbol > rhsD1 ( { tC, tA, nF } );
		std::vector < alphabet::Symbol > rhsE1 ( { tC, tA, tE, nE } );
		std::vector < alphabet::Symbol > rhsE2 ( { } );
		std::vector < alphabet::Symbol > rhsF1 ( { tE, nD } );
		std::vector < alphabet::Symbol > rhsF2 ( { } );

		grammar.addRule ( nS, rhsS1 );
		grammar.addRule ( nS, rhsS2 );
		grammar.addRule ( nS, rhsS3 );
		grammar.addRule ( nA, rhsA1 );
		grammar.addRule ( nA, rhsA2 );
		grammar.addRule ( nB, rhsB1 );
		grammar.addRule ( nB, rhsB2 );
		grammar.addRule ( nC, rhsC1 );
		grammar.addRule ( nC, rhsC2 );
		grammar.addRule ( nD, rhsD1 );
		grammar.addRule ( nE, rhsE1 );
		grammar.addRule ( nE, rhsE2 );
		grammar.addRule ( nF, rhsF1 );
		grammar.addRule ( nF, rhsF2 );

		 // --------------------------------------------------
		std::map < std::vector < alphabet::Symbol >, std::set < std::variant < alphabet::Symbol, string::Epsilon > > > first;

		first[rhsS1] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tA, tB
		};
		first[rhsS2] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tD
		};
		first[rhsS3] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tC
		};
		first[rhsA1] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tA
		};
		first[rhsA2] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tB
		};
		first[rhsB1] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tA
		};
		first[rhsB2] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tB
		};
		first[rhsC1] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tA
		};
		first[rhsC2] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tE
		};
		first[rhsD1] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tC
		};
		first[rhsE1] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tC
		};
		first[rhsE2] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			string::Epsilon::EPSILON
		};
		first[rhsF1] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			tE
		};
		first[rhsF2] = std::set < std::variant < alphabet::Symbol, string::Epsilon > > {
			string::Epsilon::EPSILON
		};

		// --------------------------------------------------

		std::map < std::vector < alphabet::Symbol >, std::set < std::variant < alphabet::Symbol, string::Epsilon > > > firstAlgo;

		for ( const auto & rule : grammar.getRawRules ( ) )
			for ( const auto & rhs : rule.second )
				firstAlgo[rhs] = grammar::parsing::First::first ( grammar, rhs );

		CPPUNIT_ASSERT ( first == firstAlgo );
	}
}

void FirstTest::testFirst2 ( ) {
	alphabet::Symbol A = alphabet::symbolFrom ( 'A' );
	alphabet::Symbol c = alphabet::symbolFrom ( 'c' );
	alphabet::Symbol d = alphabet::symbolFrom ( 'd' );

	grammar::CFG grammar ( A );

	grammar.setTerminalAlphabet ( { c, d } );
	grammar.setNonterminalAlphabet ( { { A } } );
	grammar.setInitialSymbol ( A );

	grammar.addRule ( A, std::vector < alphabet::Symbol > { A, c } );
	grammar.addRule ( A, std::vector < alphabet::Symbol > { d } );

	std::map < std::vector < alphabet::Symbol >, std::set < std::variant < alphabet::Symbol, string::Epsilon > > > res = { { { d }, { d } }, { { A, c }, { d } } };
	CPPUNIT_ASSERT ( res == grammar::parsing::First::first ( grammar ) );
}

void FirstTest::testFirst3 ( ) {
	alphabet::Symbol S = alphabet::symbolFrom ( 'S' );
	alphabet::Symbol A = alphabet::symbolFrom ( 'A' );
	alphabet::Symbol B = alphabet::symbolFrom ( 'B' );
	alphabet::Symbol a = alphabet::symbolFrom ( 'a' );
	alphabet::Symbol b = alphabet::symbolFrom ( 'b' );
	alphabet::Symbol c = alphabet::symbolFrom ( 'c' );
	alphabet::Symbol d = alphabet::symbolFrom ( 'd' );
	alphabet::Symbol f = alphabet::symbolFrom ( 'f' );

	grammar::CFG grammar ( S );

	grammar.setTerminalAlphabet ( { a, b, c, d, f } );
	grammar.setNonterminalAlphabet ( { { S, A, B } } );
	grammar.setInitialSymbol ( S );

	grammar.addRule ( S, std::vector < alphabet::Symbol > { A, a } );
	grammar.addRule ( S, std::vector < alphabet::Symbol > { b, S } );
	grammar.addRule ( A, std::vector < alphabet::Symbol > { c, A, d } );
	grammar.addRule ( A, std::vector < alphabet::Symbol > { B } );
	grammar.addRule ( B, std::vector < alphabet::Symbol > { f, S } );
	grammar.addRule ( B, std::vector < alphabet::Symbol > { } );

	std::map < std::vector < alphabet::Symbol >, std::set < std::variant < alphabet::Symbol, string::Epsilon > > > res =
	{
		{ { A, a }, { c, f, a } }, { { b, S }, { b } }, { { c, A, d }, { c } }, { { B }, { f, string::Epsilon::EPSILON } }, { { f, S }, { f } }, { { }, { string::Epsilon::EPSILON } }
	};
	CPPUNIT_ASSERT ( res == grammar::parsing::First::first ( grammar ) );
}
