#include "HandleFirstFollowConflict.h"

#include "grammar/ContextFree/CFG.h"
#include "alphabet/SymbolPairSymbol.h"
#include "grammar/parsing/HandleFirstFollowConflict.h"
#include "factory/StringDataFactory.hpp"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( HandleFirstFollowConflict, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( HandleFirstFollowConflict );

void HandleFirstFollowConflict::setUp ( ) {
}

void HandleFirstFollowConflict::tearDown ( ) {
}

void HandleFirstFollowConflict::testHandleFirstFollowConflict ( ) {
	alphabet::Symbol A = alphabet::symbolFrom ( "A" );
	alphabet::Symbol B = alphabet::symbolFrom ( "B" );
	alphabet::Symbol C = alphabet::symbolFrom ( "C" );

	alphabet::Symbol a = alphabet::symbolFrom ( 'a' );
	alphabet::Symbol b = alphabet::symbolFrom ( 'b' );
	alphabet::Symbol c = alphabet::symbolFrom ( 'c' );
	alphabet::Symbol d = alphabet::symbolFrom ( 'd' );

	alphabet::Symbol Ba = alphabet::createUniqueSymbol ( alphabet::Symbol ( alphabet::SymbolPairSymbol ( std::make_pair ( B, a ) ) ), { }, { } );

	grammar::CFG grammar ( A );

	grammar.setTerminalAlphabet ( { a, b, c, d } );
	grammar.setNonterminalAlphabet ( { A, B, C } );
	grammar.setInitialSymbol ( A );

	grammar.addRule ( A, std::vector < alphabet::Symbol > { a, B, C } );
	grammar.addRule ( A, std::vector < alphabet::Symbol > { c, C } );
	grammar.addRule ( B, std::vector < alphabet::Symbol > { a, B, a, d } );
	grammar.addRule ( B, std::vector < alphabet::Symbol > { } );
	grammar.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	grammar.addRule ( C, std::vector < alphabet::Symbol > { b, B } );

	grammar::CFG res = grammar;
	grammar::parsing::HandleFirstFollowConflict::handleFirstFollowConflict ( res, a, B, { { a, B, C }, { } } );

	grammar::CFG comp ( A );

	comp.setTerminalAlphabet ( { a, b, c, d } );
	comp.setNonterminalAlphabet ( { A, B, C } );
	comp.setInitialSymbol ( A );

	comp.addRule ( A, std::vector < alphabet::Symbol > { a, B, a, C } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { a, B, b, B } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { c, C } );
	comp.addRule ( B, std::vector < alphabet::Symbol > { a, B, a, d } );
	comp.addRule ( B, std::vector < alphabet::Symbol > { } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { b, B } );

//	std::cout << "res " << alib::StringDataFactory::toString ( grammar::Grammar ( res ) ) << std::endl << "comp " << alib::StringDataFactory::toString ( grammar::Grammar ( comp ) ) << std::endl;

	CPPUNIT_ASSERT ( res == comp );

	grammar::parsing::HandleFirstFollowConflict::handleFirstFollowConflict ( res, a, B, { { a, B, C }, { } } );

	grammar::CFG comp2 ( A );

	comp2.setTerminalAlphabet ( { a, b, c, d } );
	comp2.setNonterminalAlphabet ( { A, B, Ba, C } );
	comp2.setInitialSymbol ( A );

	comp2.addRule ( A, std::vector < alphabet::Symbol > { a, Ba, C } );
	comp2.addRule ( A, std::vector < alphabet::Symbol > { a, B, b, B } );
	comp2.addRule ( A, std::vector < alphabet::Symbol > { c, C } );
	comp2.addRule ( B, std::vector < alphabet::Symbol > { a, Ba, d } );
	comp2.addRule ( B, std::vector < alphabet::Symbol > { } );
	comp2.addRule ( Ba, std::vector < alphabet::Symbol > { a, Ba, d, a } );
	comp2.addRule ( Ba, std::vector < alphabet::Symbol > { a } );
	comp2.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	comp2.addRule ( C, std::vector < alphabet::Symbol > { b, B } );

//	std::cout << "res2 " << alib::StringDataFactory::toString ( grammar::Grammar ( res ) ) << std::endl << "comp2 " << alib::StringDataFactory::toString ( grammar::Grammar ( comp2 ) ) << std::endl;

	CPPUNIT_ASSERT ( res == comp2 );
}

void HandleFirstFollowConflict::testHandleFirstFollowConflict2 ( ) {
	alphabet::Symbol A = alphabet::symbolFrom ( "A" );
	alphabet::Symbol B = alphabet::symbolFrom ( "B" );
	alphabet::Symbol C = alphabet::symbolFrom ( "C" );
	alphabet::Symbol X = alphabet::symbolFrom ( "X" );

	alphabet::Symbol a = alphabet::symbolFrom ( 'a' );
	alphabet::Symbol b = alphabet::symbolFrom ( 'b' );
	alphabet::Symbol c = alphabet::symbolFrom ( 'c' );
	alphabet::Symbol d = alphabet::symbolFrom ( 'd' );

	alphabet::Symbol Ba = alphabet::createUniqueSymbol ( alphabet::Symbol ( alphabet::SymbolPairSymbol ( std::make_pair ( B, a ) ) ), { }, { } );
	alphabet::Symbol Xa = alphabet::createUniqueSymbol ( alphabet::Symbol ( alphabet::SymbolPairSymbol ( std::make_pair ( X, a ) ) ), { }, { } );

	grammar::CFG grammar ( A );

	grammar.setTerminalAlphabet ( { a, b, c, d } );
	grammar.setNonterminalAlphabet ( { A, B, C, X } );
	grammar.setInitialSymbol ( A );

	grammar.addRule ( A, std::vector < alphabet::Symbol > { a, X, C, a } );
	grammar.addRule ( A, std::vector < alphabet::Symbol > { c, C } );
	grammar.addRule ( X, { c, B });
	grammar.addRule ( B, std::vector < alphabet::Symbol > { a, B, a, d } );
	grammar.addRule ( B, std::vector < alphabet::Symbol > { } );
	grammar.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	grammar.addRule ( C, std::vector < alphabet::Symbol > { b, B, c } );

	grammar::CFG res = grammar;
	grammar::parsing::HandleFirstFollowConflict::handleFirstFollowConflict ( res, a, B, { { a, B, C }, { } } );

	grammar::CFG comp ( A );

	comp.setTerminalAlphabet ( { a, b, c, d } );
	comp.setNonterminalAlphabet ( { A, B, C, X } );
	comp.setInitialSymbol ( A );

	comp.addRule ( A, std::vector < alphabet::Symbol > { a, X, a, C, a } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { a, X, b, B, c, a } );
	comp.addRule ( A, std::vector < alphabet::Symbol > { c, C } );
	comp.addRule ( X, { c, B } );
	comp.addRule ( B, std::vector < alphabet::Symbol > { a, B, a, d } );
	comp.addRule ( B, std::vector < alphabet::Symbol > { } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	comp.addRule ( C, std::vector < alphabet::Symbol > { b, B, c } );

//	std::cout << "res " << alib::StringDataFactory::toString ( grammar::Grammar ( res ) ) << std::endl << "comp " << alib::StringDataFactory::toString ( grammar::Grammar ( comp ) ) << std::endl;

	CPPUNIT_ASSERT ( res == comp );

	grammar::parsing::HandleFirstFollowConflict::handleFirstFollowConflict ( res, a, B, { { a, B, C }, { } } );

	grammar::CFG comp2 ( A );

	comp2.setTerminalAlphabet ( { a, b, c, d } );
	comp2.setNonterminalAlphabet ( { A, B, Ba, C, X, Xa } );
	comp2.setInitialSymbol ( A );

	comp2.addRule ( A, std::vector < alphabet::Symbol > { a, Xa, C, a } );
	comp2.addRule ( A, std::vector < alphabet::Symbol > { a, X, b, B, c, a } );
	comp2.addRule ( A, std::vector < alphabet::Symbol > { c, C } );
	comp2.addRule ( X, { c, B } );
	comp2.addRule ( Xa, { c, Ba } );
	comp2.addRule ( B, std::vector < alphabet::Symbol > { a, Ba, d } );
	comp2.addRule ( B, std::vector < alphabet::Symbol > { } );
	comp2.addRule ( Ba, std::vector < alphabet::Symbol > { a, Ba, d, a } );
	comp2.addRule ( Ba, std::vector < alphabet::Symbol > { a } );
	comp2.addRule ( C, std::vector < alphabet::Symbol > { a, C } );
	comp2.addRule ( C, std::vector < alphabet::Symbol > { b, B, c } );

//	std::cout << "res2 " << alib::StringDataFactory::toString ( grammar::Grammar ( res ) ) << std::endl << "comp2 " << alib::StringDataFactory::toString ( grammar::Grammar ( comp2 ) ) << std::endl;

	CPPUNIT_ASSERT ( res == comp2 );
}
