#include "GrammarToGNFTest.h"

#include "grammar/simplify/ToGNF.h"
#include "grammar/generate/GenerateUpToLength.h"

#include "grammar/ContextFree/GNF.h"
#include "grammar/ContextFree/EpsilonFreeCFG.h"

#include <factory/StringDataFactory.hpp>

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GrammarToGNFTest, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION( GrammarToGNFTest );

void GrammarToGNFTest::setUp() {
}

void GrammarToGNFTest::tearDown() {
}

void GrammarToGNFTest::testRemoveToGNFRules1() {
	alphabet::Symbol S = alphabet::symbolFrom("S");
	alphabet::Symbol A = alphabet::symbolFrom("A");
	alphabet::Symbol B = alphabet::symbolFrom("B");
	alphabet::Symbol C = alphabet::symbolFrom("C");
	alphabet::Symbol D = alphabet::symbolFrom("D");

	alphabet::Symbol a = alphabet::symbolFrom("a");
	alphabet::Symbol b = alphabet::symbolFrom("b");

	grammar::EpsilonFreeCFG grammar1(S);
	grammar1.setNonterminalAlphabet({S, A, B, C, D});
	grammar1.setTerminalAlphabet({a, b});

	alphabet::Symbol aprimed = createUniqueSymbol(a, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());
	alphabet::Symbol bprimed = createUniqueSymbol(b, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());

	grammar::GNF grammar2 = grammar::simplify::ToGNF::convert(grammar1);

	grammar::GNF grammar3(S);
	grammar3.setNonterminalAlphabet({S, A, B, C, D, aprimed, bprimed});
	grammar3.setTerminalAlphabet({a, b});
	grammar3.addRule(aprimed, std::make_pair(a, std::vector<alphabet::Symbol>{}));
	grammar3.addRule(bprimed, std::make_pair(b, std::vector<alphabet::Symbol>{}));

	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar2)) << std::endl;
	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar3)) << std::endl;

	CPPUNIT_ASSERT(grammar2 == grammar3);
}

void GrammarToGNFTest::testRemoveToGNFRules2() {
	alphabet::Symbol S = alphabet::symbolFrom("S");
	alphabet::Symbol A = alphabet::symbolFrom("A");
	alphabet::Symbol B = alphabet::symbolFrom("B");
	alphabet::Symbol C = alphabet::symbolFrom("C");
	alphabet::Symbol D = alphabet::symbolFrom("D");

	alphabet::Symbol a = alphabet::symbolFrom("a");
	alphabet::Symbol b = alphabet::symbolFrom("b");
	alphabet::Symbol c = alphabet::symbolFrom("c");

	grammar::EpsilonFreeCFG grammar1(S);
	grammar1.setNonterminalAlphabet({S, A, B, C, D});
	grammar1.setTerminalAlphabet({a, b, c});
	grammar1.addRule(S, {A});
	grammar1.addRule(A, {A, a});
	grammar1.addRule(A, {A, b});
	grammar1.addRule(A, {c});

	alphabet::Symbol Aprimed = createUniqueSymbol(A, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());
	alphabet::Symbol aprimed = createUniqueSymbol(a, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());
	alphabet::Symbol bprimed = createUniqueSymbol(b, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());
	alphabet::Symbol cprimed = createUniqueSymbol(c, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());

	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar1)) << std::endl;

	grammar::GNF grammar2 = grammar::simplify::ToGNF::convert(grammar1);

	grammar::GNF grammar3(S);
	grammar3.setNonterminalAlphabet({S, A, Aprimed, B, C, D, aprimed, bprimed, cprimed});
	grammar3.setTerminalAlphabet({a, b, c});
	grammar3.addRule(S, std::make_pair(c, std::vector<alphabet::Symbol>{Aprimed}));
	grammar3.addRule(S, std::make_pair(c, std::vector<alphabet::Symbol>{}));
	grammar3.addRule(A, std::make_pair(c, std::vector<alphabet::Symbol>{Aprimed}));
	grammar3.addRule(A, std::make_pair(c, std::vector<alphabet::Symbol>{}));
	grammar3.addRule(Aprimed, std::make_pair(a, std::vector<alphabet::Symbol>{Aprimed}));
	grammar3.addRule(Aprimed, std::make_pair(a, std::vector<alphabet::Symbol>{}));
	grammar3.addRule(Aprimed, std::make_pair(b, std::vector<alphabet::Symbol>{Aprimed}));
	grammar3.addRule(Aprimed, std::make_pair(b, std::vector<alphabet::Symbol>{}));
	grammar3.addRule(aprimed, std::make_pair(a, std::vector<alphabet::Symbol>{}));
	grammar3.addRule(bprimed, std::make_pair(b, std::vector<alphabet::Symbol>{}));
	grammar3.addRule(cprimed, std::make_pair(c, std::vector<alphabet::Symbol>{}));

	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar2)) << std::endl;
	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar3)) << std::endl;

	CPPUNIT_ASSERT(grammar2 == grammar3);
}

void GrammarToGNFTest::testRemoveToGNFRules3() {
	alphabet::Symbol S = alphabet::symbolFrom("S");
	alphabet::Symbol A = alphabet::symbolFrom("A");
	alphabet::Symbol B = alphabet::symbolFrom("B");
	alphabet::Symbol C = alphabet::symbolFrom("C");

	alphabet::Symbol a = alphabet::symbolFrom("a");
	alphabet::Symbol b = alphabet::symbolFrom("b");
	alphabet::Symbol c = alphabet::symbolFrom("c");

	grammar::EpsilonFreeCFG grammar1(S);
	grammar1.setNonterminalAlphabet({S, A, B, C});
	grammar1.setTerminalAlphabet({a, b, c});
	grammar1.addRule(S, {A});
	grammar1.addRule(A, {B, a});
	grammar1.addRule(B, {A, b});
	grammar1.addRule(A, {c});

	alphabet::Symbol Bprimed = createUniqueSymbol(B, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());
	alphabet::Symbol aprimed = createUniqueSymbol(a, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());
	alphabet::Symbol bprimed = createUniqueSymbol(b, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());
	alphabet::Symbol cprimed = createUniqueSymbol(c, grammar1.getTerminalAlphabet(), grammar1.getNonterminalAlphabet());

	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar1)) << std::endl;

	grammar::GNF grammar2 = grammar::simplify::ToGNF::convert(grammar1);

	grammar::GNF grammar3(S);
	grammar3.setNonterminalAlphabet({S, A, B, Bprimed, C, aprimed, bprimed, cprimed});
	grammar3.setTerminalAlphabet({a, b, c});
	grammar3.addRule(S, std::make_pair(c, std::vector<alphabet::Symbol>{}));
	grammar3.addRule(S, std::make_pair(c, std::vector<alphabet::Symbol>{bprimed, Bprimed, aprimed}));
	grammar3.addRule(S, std::make_pair(c, std::vector<alphabet::Symbol>{bprimed, aprimed}));
	grammar3.addRule(A, std::make_pair(c, std::vector<alphabet::Symbol>{bprimed, aprimed}));
	grammar3.addRule(A, std::make_pair(c, std::vector<alphabet::Symbol>{bprimed, Bprimed, aprimed}));
	grammar3.addRule(A, std::make_pair(c, std::vector<alphabet::Symbol>{}));
	grammar3.addRule(B, std::make_pair(c, std::vector<alphabet::Symbol>{bprimed}));
	grammar3.addRule(B, std::make_pair(c, std::vector<alphabet::Symbol>{bprimed, Bprimed}));
	grammar3.addRule(Bprimed, std::make_pair(a, std::vector<alphabet::Symbol>{bprimed, Bprimed}));
	grammar3.addRule(Bprimed, std::make_pair(a, std::vector<alphabet::Symbol>{bprimed}));
	grammar3.addRule(aprimed, std::make_pair(a, std::vector<alphabet::Symbol>{}));
	grammar3.addRule(bprimed, std::make_pair(b, std::vector<alphabet::Symbol>{}));
	grammar3.addRule(cprimed, std::make_pair(c, std::vector<alphabet::Symbol>{}));

	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar2)) << std::endl;
	std::cout << alib::StringDataFactory::toString<grammar::Grammar>(grammar::Grammar(grammar3)) << std::endl;

	CPPUNIT_ASSERT(grammar2 == grammar3);
}

void GrammarToGNFTest::testRemoveToGNFRules4() {
	alphabet::Symbol A = alphabet::symbolFrom("A");
	alphabet::Symbol B = alphabet::symbolFrom("B");
	alphabet::Symbol C = alphabet::symbolFrom("C");

	alphabet::Symbol a = alphabet::symbolFrom("a");
	alphabet::Symbol b = alphabet::symbolFrom("b");

	grammar::EpsilonFreeCFG grammar1(A);
	grammar1.setNonterminalAlphabet({A, B, C});
	grammar1.setTerminalAlphabet({a, b});
	grammar1.addRule(A, {B, C});
	grammar1.addRule(A, {a});
	grammar1.addRule(B, {C, A});
	grammar1.addRule(B, {A, b});
	grammar1.addRule(C, {A, B});
	grammar1.addRule(C, {C, C});
	grammar1.addRule(C, {a});

	grammar::GNF grammar2 = grammar::simplify::ToGNF::convert(grammar1);

	CPPUNIT_ASSERT(grammar::generate::GenerateUpToLength::generate(grammar1, 7) == grammar::generate::GenerateUpToLength::generate(grammar2, 7));
}
