/*
 * RankedSymbol.cpp
 *
 *  Created on: Apr 10, 2013
 *      Author: Stepan Plachy
 */

#include "RankedSymbol.h"
#include "../sax/FromXMLParserHelper.h"
#include "Symbol.h"
#include "../object/Object.h"
#include "../XmlApi.hpp"

namespace alphabet {

RankedSymbol::RankedSymbol(int number, int rank) : symbol(alphabet::symbolFrom(number)), rank(primitive::Unsigned(rank)) {

}

RankedSymbol::RankedSymbol(char character, int rank) : symbol(alphabet::symbolFrom(character)), rank(primitive::Unsigned(rank)) {

}

RankedSymbol::RankedSymbol(std::string symbol, int rank) : symbol(alphabet::symbolFrom(std::move(symbol))), rank(primitive::Unsigned(rank)) {

}

RankedSymbol::RankedSymbol(alphabet::Symbol symbol, primitive::Unsigned rank) : symbol(std::move(symbol)), rank(std::move(rank)) {

}

RankedSymbol::RankedSymbol(alphabet::Symbol symbol, int rank) : symbol(std::move(symbol)), rank(primitive::Unsigned(rank)) {

}

SymbolBase* RankedSymbol::clone() const {
	return new RankedSymbol(*this);
}

SymbolBase* RankedSymbol::plunder() && {
	return new RankedSymbol(std::move(*this));
}

const alphabet::Symbol& RankedSymbol::getSymbol() const {
	return symbol;
}

const primitive::Unsigned& RankedSymbol::getRank() const {
	return rank;
}

int RankedSymbol::compare(const RankedSymbol& other) const {
	int res = this->symbol.getData().compare(other.symbol.getData());
	if(res == 0) res = this->rank.compare(other.rank);
	return res;
}

void RankedSymbol::operator>>(std::ostream& out) const {
	out << "(RankedSymbol " << symbol << " #" << rank << ")";
}

RankedSymbol::operator std::string () const {
	return (std::string) symbol + "_" + (std::string) rank;
}

const std::string RankedSymbol::XML_TAG_NAME = "RankedSymbol";

RankedSymbol RankedSymbol::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, RankedSymbol::XML_TAG_NAME);
	alphabet::Symbol data = alib::xmlApi<alphabet::Symbol>::parse(input);
	primitive::Unsigned rank = alib::xmlApi<primitive::Unsigned>::parse(input);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, RankedSymbol::XML_TAG_NAME);
	return RankedSymbol(std::move(data), std::move(rank));
}

void RankedSymbol::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(RankedSymbol::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	alib::xmlApi<primitive::Unsigned>::compose(out, rank);
	out.emplace_back(RankedSymbol::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

SymbolBase* RankedSymbol::next() const {
	return new RankedSymbol(symbol.next(), rank);
}

SymbolBase* RankedSymbol::inc() && {
	symbol.inc();
	return NULL;
}

} /* namespace alphabet */

namespace alib {

auto rankedSymbolParserRegister = xmlApi<alphabet::Symbol>::ParserRegister<alphabet::RankedSymbol>();
auto rankedSymbolParserRegister2 = xmlApi<alib::Object>::ParserRegister<alphabet::RankedSymbol>();

} /* namespace alib */
