/*
 * AutomatonFromXMLParser.cpp
 *
 *  Created on: Oct 12, 2013
 *      Author: Jan Travnicek
 */

#include "AutomatonFromXMLParser.h"
#include "../../sax/ParserException.h"
#include "../../XmlApi.hpp"
#include "../../alphabet/RankedSymbol.h"
#include "Shift.h"
#include "../../string/StringClasses.h"
#include "../../regexp/RegExp.h"

namespace automaton {

std::set < State > AutomatonFromXMLParser::parseStates ( std::deque < sax::Token >::iterator & input ) {
	std::set < State > states;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "states" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		states.insert ( State::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "states" );
	return states;
}

std::set < alphabet::Symbol > AutomatonFromXMLParser::parseCallInputAlphabet ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::Symbol > inputSymbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "callInputAlphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		inputSymbols.insert ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "callInputAlphabet" );
	return inputSymbols;
}

std::set < alphabet::Symbol > AutomatonFromXMLParser::parseReturnInputAlphabet ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::Symbol > inputSymbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "returnInputAlphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		inputSymbols.insert ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "returnInputAlphabet" );
	return inputSymbols;
}

std::set < alphabet::Symbol > AutomatonFromXMLParser::parseLocalInputAlphabet ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::Symbol > inputSymbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "localInputAlphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		inputSymbols.insert ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "localInputAlphabet" );
	return inputSymbols;
}

std::set < alphabet::Symbol > AutomatonFromXMLParser::parseInputAlphabet ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::Symbol > inputSymbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "inputAlphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		inputSymbols.insert ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "inputAlphabet" );
	return inputSymbols;
}

std::set < alphabet::RankedSymbol > AutomatonFromXMLParser::parseRankedInputAlphabet ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::RankedSymbol > inputSymbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "rankedInputAlphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		inputSymbols.insert ( alib::xmlApi < alphabet::RankedSymbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "rankedInputAlphabet" );
	return inputSymbols;
}

State AutomatonFromXMLParser::parseInitialState ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "initialState" );
	State state ( alib::xmlApi < label::Label >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "initialState" );

	return state;
}

std::set < State > AutomatonFromXMLParser::parseInitialStates ( std::deque < sax::Token >::iterator & input ) {
	std::set < State > initialStates;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "initialStates" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		initialStates.insert ( State::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "initialStates" );
	return initialStates;
}

std::set < State > AutomatonFromXMLParser::parseFinalStates ( std::deque < sax::Token >::iterator & input ) {
	std::set < State > finalStates;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "finalStates" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		finalStates.insert ( State::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "finalStates" );
	return finalStates;
}

std::set < alphabet::Symbol > AutomatonFromXMLParser::parseStackAlphabet ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::Symbol > stackSymbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "stackAlphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		stackSymbols.insert ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "stackAlphabet" );
	return stackSymbols;
}

std::set < alphabet::Symbol > AutomatonFromXMLParser::parseInitialStackSymbols ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::Symbol > initialSymbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "initialStackSymbols" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		initialSymbols.insert ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "initialStackSymbols" );
	return initialSymbols;
}

alphabet::Symbol AutomatonFromXMLParser::parseInitialStackSymbol ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "initialStackSymbol" );
	alphabet::Symbol initialSymbol ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "initialStackSymbol" );

	return initialSymbol;
}

std::set < alphabet::Symbol > AutomatonFromXMLParser::parseTapeAlphabet ( std::deque < sax::Token >::iterator & input ) {
	std::set < alphabet::Symbol > tapeSymbols;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "tapeAlphabet" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		tapeSymbols.insert ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "tapeAlphabet" );
	return tapeSymbols;
}

alphabet::Symbol AutomatonFromXMLParser::parseBlankSymbol ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "blankSymbol" );
	alphabet::Symbol blank ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "blankSymbol" );

	return blank;
}

alphabet::Symbol AutomatonFromXMLParser::parseBottomOfTheStackSymbol ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "bottomOfTheStackSymbol" );
	alphabet::Symbol blank ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "bottomOfTheStackSymbol" );

	return blank;
}

std::map < alphabet::Symbol, std::pair < std::vector < alphabet::Symbol >, std::vector < alphabet::Symbol > > > AutomatonFromXMLParser::parseInputToPushdownStoreOperation ( std::deque < sax::Token >::iterator & input ) {
	std::map < alphabet::Symbol, std::pair < std::vector < alphabet::Symbol >, std::vector < alphabet::Symbol > > > operations;

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "inputToPushdownStoreOperations" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) ) {
		sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "operation" );

		alphabet::Symbol inputSymbol ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

		std::vector < alphabet::Symbol > pop  = parseTransitionPop ( input );
		std::vector < alphabet::Symbol > push = parseTransitionPush ( input );

		operations.insert ( std::make_pair ( std::move ( inputSymbol ), std::make_pair ( std::move ( pop ), std::move ( push ) ) ) );

		sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "operation" );
	}

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "inputToPushdownStoreOperations" );
	return operations;
}

State AutomatonFromXMLParser::parseTransitionTo ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "to" );
	State state ( alib::xmlApi < label::Label >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "to" );

	return state;
}

State AutomatonFromXMLParser::parseTransitionFrom ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "from" );
	State state ( alib::xmlApi < label::Label >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "from" );

	return state;
}

std::vector < State > AutomatonFromXMLParser::parseTransitionFromMultiple ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "from" );
	std::vector < State > states;

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		states.push_back ( State::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "from" );
	return states;
}

Shift AutomatonFromXMLParser::parseTransitionShift ( std::deque < sax::Token >::iterator & input ) {
	Shift shift;

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "shift" );

	if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::CHARACTER, "left" ) )
		shift = Shift::LEFT;
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::CHARACTER, "right" ) )
		shift = Shift::RIGHT;
	else if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::CHARACTER, "none" ) )
		shift = Shift::NONE;
	else
		throw sax::ParserException ( sax::Token ( "", sax::Token::TokenType::CHARACTER ), * input );

	++input;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "shift" );

	return shift;
}

std::vector < alphabet::Symbol > AutomatonFromXMLParser::parseTransitionPop ( std::deque < sax::Token >::iterator & input ) {
	std::vector < alphabet::Symbol > pops;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "pop" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		pops.push_back ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "pop" );
	return pops;
}

alphabet::Symbol AutomatonFromXMLParser::parseTransitionSinglePop ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "pop" );
	alphabet::Symbol pop = alib::xmlApi < alphabet::Symbol >::parse ( input );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "pop" );

	return pop;
}

std::vector < alphabet::Symbol > AutomatonFromXMLParser::parseTransitionPush ( std::deque < sax::Token >::iterator & input ) {
	std::vector < alphabet::Symbol > pushes;
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "push" );

	while ( sax::FromXMLParserHelper::isTokenType ( input, sax::Token::TokenType::START_ELEMENT ) )
		pushes.push_back ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "push" );
	return pushes;
}

alphabet::Symbol AutomatonFromXMLParser::parseTransitionSinglePush ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "push" );
	alphabet::Symbol push = alib::xmlApi < alphabet::Symbol >::parse ( input );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "push" );

	return push;
}

alphabet::Symbol AutomatonFromXMLParser::parseTransitionInputSymbol ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "input" );
	alphabet::Symbol result ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "input" );

	return result;
}

alphabet::RankedSymbol AutomatonFromXMLParser::parseTransitionInputRankedSymbol ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "input" );
	alphabet::RankedSymbol result ( alib::xmlApi < alphabet::RankedSymbol >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "input" );

	return result;
}

alphabet::Symbol AutomatonFromXMLParser::parseTransitionOutputSymbol ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "output" );
	alphabet::Symbol result ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "output" );

	return result;
}

std::variant < string::Epsilon, alphabet::Symbol > AutomatonFromXMLParser::parseTransitionInputEpsilonSymbol ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "input" );

	if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "epsilon" ) ) {
		std::variant < string::Epsilon, alphabet::Symbol > result ( string::Epsilon::EPSILON );
		++input;
		sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "epsilon" );
		sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "input" );
		return result;
	} else {
		std::variant < string::Epsilon, alphabet::Symbol > result ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );
		sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "input" );
		return result;
	}
}

std::variant < string::Epsilon, alphabet::Symbol > AutomatonFromXMLParser::parseTransitionOutputEpsilonSymbol ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "output" );

	if ( sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, "epsilon" ) ) {
		std::variant < string::Epsilon, alphabet::Symbol > result ( string::Epsilon::EPSILON );
		++input;
		sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "epsilon" );
		sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "output" );
		return result;
	} else {
		std::variant < string::Epsilon, alphabet::Symbol > result ( alib::xmlApi < alphabet::Symbol >::parse ( input ) );
		sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "output" );
		return result;
	}
}

string::LinearString AutomatonFromXMLParser::parseTransitionInputString ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "input" );
	string::LinearString result ( alib::xmlApi < string::LinearString >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "input" );

	return result;
}

regexp::RegExp AutomatonFromXMLParser::parseTransitionInputRegexp ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, "input" );
	regexp::RegExp result ( alib::xmlApi < regexp::RegExp >::parse ( input ) );
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, "input" );

	return result;
}

} /* namespace automaton */
