/*
 * BlankSymbolInputTapeAlphabet.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "BlankSymbolInputTapeAlphabet.h"

#include <algorithm>
#include "../AutomatonException.h"

namespace automaton {

BlankSymbolInputTapeAlphabet::BlankSymbolInputTapeAlphabet(alphabet::Symbol blank) : blankSymbol(blank) {
	tapeAlphabet.insert(std::move(blank));
}

const std::set<alphabet::Symbol>& BlankSymbolInputTapeAlphabet::getInputAlphabet() const {
	return inputAlphabet;
}

bool BlankSymbolInputTapeAlphabet::addInputSymbol(alphabet::Symbol symbol) {
	if (!tapeAlphabet.count(symbol)) {
		throw AutomatonException("Input symbol \"" + (std::string) symbol + "\" is not in tape alphabet.");
	}

	if (symbol == blankSymbol) {
		throw AutomatonException("Input symbol \"" + (std::string) symbol + "\" cannot be blank symbol.");
	}

	return inputAlphabet.insert(std::move(symbol)).second;
}

void BlankSymbolInputTapeAlphabet::setInputAlphabet(std::set<alphabet::Symbol> newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(inputAlphabet.begin(), inputAlphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	if(!std::includes(tapeAlphabet.begin(), tapeAlphabet.end(), newSymbols.begin(), newSymbols.end())) {
		throw AutomatonException("Input symbol ? is not in tape alphabet.");
	}

	if(newSymbols.count(blankSymbol)) {
		throw AutomatonException("Input symbol ? cannot be blank symbol.");
	}

	for(const alphabet::Symbol& removedSymbol : removed) {
		removeInputSymbol(removedSymbol);
	}

	inputAlphabet = std::move(newSymbols);
}

bool BlankSymbolInputTapeAlphabet::addTapeSymbol(alphabet::Symbol symbol) {
	return tapeAlphabet.insert(std::move(symbol)).second;
}

void BlankSymbolInputTapeAlphabet::setTapeAlphabet(std::set<alphabet::Symbol> newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(tapeAlphabet.begin(), tapeAlphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	for(const alphabet::Symbol& removedSymbol : removed) {
		removeTapeSymbol(removedSymbol);
	}

	tapeAlphabet = std::move(newSymbols);
}

const std::set<alphabet::Symbol>& BlankSymbolInputTapeAlphabet::getTapeAlphabet() const {
	return tapeAlphabet;
}

void BlankSymbolInputTapeAlphabet::setBlankSymbol(alphabet::Symbol symbol) {
	if (inputAlphabet.count(symbol))
		throw AutomatonException("Blank symbol \"" + (std::string) symbol + "\" is in input alphabet.");

	if (!tapeAlphabet.count(symbol))
		throw AutomatonException("Blank symbol \"" + (std::string) symbol + "\" is not in tape alphabet.");

	blankSymbol = std::move(symbol);

}

const alphabet::Symbol& BlankSymbolInputTapeAlphabet::getBlankSymbol() const {
	return blankSymbol;
}

} /* namespace automaton */

