/*
 * MultiInitialStates.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "MultiInitialStates.h"

#include <set>
#include <algorithm>
#include "../AutomatonException.h"

namespace automaton {

bool MultiInitialStates::addInitialState(State state) {
	if (!states.count(state)) {
		throw AutomatonException("State " + (std::string) state.getName() + " cannot be set as initial state. It is not present in the automaton.");
	}

	return initialStates.insert(std::move(state)).second;
}

void MultiInitialStates::setInitialStates(std::set<State> newStates) {
	std::set<State> removed;
	std::set_difference(initialStates.begin(), initialStates.end(), newStates.begin(), newStates.end(), std::inserter(removed, removed.end()));

	for(const State& removedState : removed) {
		removeInitialState(removedState);
	}

	if(!std::includes(states.begin(), states.end(), newStates.begin(), newStates.end())) {
		throw AutomatonException("State ? cannot be set as initial state. It is not present in the automaton.");
	}

	initialStates = std::move(newStates);
}

bool MultiInitialStates::removeInitialState(const State& state) {
	return initialStates.erase(state);
}

const std::set<State>& MultiInitialStates::getInitialStates() const {
	return initialStates;
}

} /* namespace automaton */

