/*
 * States.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Martin Zak
 */

#include "States.h"
#include <set>
#include "../AutomatonException.h"
#include <algorithm>

namespace automaton {

bool States::addState(State state) {
	return states.insert(std::move(state)).second;
}

void States::setStates(std::set<State> newStates) {
	std::set<State> removed;
	std::set_difference(states.begin(), states.end(), newStates.begin(), newStates.end(), std::inserter(removed, removed.end()));

	for(const State& removedState : removed) {
		removeState(removedState);
	}

	states = std::move(newStates);
}

const std::set<State>& States::getStates() const {
	return states;
}

bool States::addFinalState(State state) {
	if (!states.count(state)) {
		throw AutomatonException("State " + (std::string) state.getName() + " cannot be set as final state. It is not present in the automaton.");
	}

	return finalStates.insert(std::move(state)).second;
}

void States::setFinalStates(std::set<State> newStates) {
	std::set<State> removed;
	std::set_difference(finalStates.begin(), finalStates.end(), newStates.begin(), newStates.end(), std::inserter(removed, removed.end()));

	for(const State& removedState : removed) {
		removeFinalState(removedState);
	}

	if(!std::includes(states.begin(), states.end(), newStates.begin(), newStates.end())) {
		throw AutomatonException("State ? cannot be set as final state. It is not present in the automaton.");
	}

	finalStates = std::move(newStates);
}

bool States::removeFinalState(const State& state) {
	return finalStates.erase(state);
}

const std::set<State>& States::getFinalStates() const {
	return finalStates;
}

} /* namespace automaton */


