/*
 * Map.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#include "ObjectsMap.h"

#include <sstream>

#include "Container.h"
#include "../object/Object.h"
#include "../XmlApi.hpp"

namespace container {

ObjectsMap::ObjectsMap(std::map<alib::Object, alib::Object> map) : std::map<alib::Object, alib::Object>(std::move(map)) {

}

ObjectsMap::ObjectsMap() : std::map<alib::Object, alib::Object>() {

}

ContainerBase* ObjectsMap::clone() const {
	return new ObjectsMap(*this);
}

ContainerBase* ObjectsMap::plunder() && {
	return new ObjectsMap(std::move(*this));
}

int ObjectsMap::compare(const ObjectsMap& other) const {
	std::compare<std::map<alib::Object, alib::Object>> comp;
	return comp(static_cast<const std::map<alib::Object, alib::Object>>(*this), static_cast<const std::map<alib::Object, alib::Object>>(other));
}

void ObjectsMap::operator>>(std::ostream& os) const {
	os << "(ObjectsMap " << static_cast<const std::map<alib::Object, alib::Object>>(*this) << ")";
}

ObjectsMap::operator std::string() const {
	std::stringstream ss;
	ss << *this;
	return std::move(ss).str();
}

const std::string ObjectsMap::XML_TAG_NAME = "Map";

ObjectsMap ObjectsMap::parse(std::deque<sax::Token>::iterator& input) {
	return ObjectsMap(parseRaw<alib::Object, alib::Object>(input));
}

void ObjectsMap::compose(std::deque<sax::Token>& out) const {
	compose(out, *this);
}

} /* namespace container */

namespace alib {

auto objectsMapParserRegister = xmlApi<container::Container>::ParserRegister<container::ObjectsMap>();
auto objectsMapParserRegister2 = xmlApi<alib::Object>::ParserRegister<container::ObjectsMap>();

} /* namespace alib */
