/*
 * Pair.h
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef OBJECTS_PAIR_H_
#define OBJECTS_PAIR_H_

#include <pair>
#include <string>

#include "../object/Object.h"
#include "ContainerBase.h"

#include "../XmlApi.hpp"

namespace container {

/**
 * Basic container from which are derived all other containers.
 * Contains reason why the container occured.
 */
class ObjectsPair : public std::pair < alib::Object, alib::Object >, public ContainerBase {
public:
	explicit ObjectsPair ( alib::Object first, alib::Object second );
	explicit ObjectsPair ( std::pair < alib::Object, alib::Object > );

	virtual ContainerBase * clone ( ) const;

	virtual ContainerBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const ObjectsPair & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static ObjectsPair parse ( std::deque < sax::Token >::iterator & input );
	template < class T, class R >
	static std::pair < T, R > parseRaw ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
	template < class T, class R >
	static void compose ( std::deque < sax::Token > & out, const std::pair < T, R > & input );
};

template < class T, class R >
std::pair < T, R > ObjectsPair::parseRaw ( std::deque < sax::Token >::iterator & input ) {
	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::START_ELEMENT, ObjectsPair::XML_TAG_NAME );

	T first	 = alib::xmlApi < T >::parse ( input );
	R second = alib::xmlApi < R >::parse ( input );

	sax::FromXMLParserHelper::popToken ( input, sax::Token::TokenType::END_ELEMENT, ObjectsPair::XML_TAG_NAME );

	return std::make_pair ( std::move ( first ), std::move ( second ) );
}

template < class T, class R >
void ObjectsPair::compose ( std::deque < sax::Token > & out, const std::pair < T, R > & input ) {
	out.emplace_back ( ObjectsPair::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT );

	alib::xmlApi < T >::compose ( out, input.first );
	alib::xmlApi < R >::compose ( out, input.second );

	out.emplace_back ( ObjectsPair::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT );
}

} /* namespace container */

namespace std {

template < >
struct compare < container::ObjectsPair > {
	int operator ()( const container::ObjectsPair & first, const container::ObjectsPair & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

namespace alib {

template < typename T, typename R >
struct xmlApi < std::pair < T, R > > {
	static std::pair < T, R > parse ( std::deque < sax::Token >::iterator & input );
	static bool first ( const std::deque < sax::Token >::const_iterator & input );
	static void compose ( std::deque < sax::Token > & output, const std::pair < T, R > & data );
};

template < typename T, typename R >
std::pair < T, R > xmlApi < std::pair < T, R > >::parse ( std::deque < sax::Token >::iterator & input ) {
	return container::ObjectsPair::parseRaw < T, R > ( input );
}

template < typename T, typename R >
bool xmlApi < std::pair < T, R > >::first ( const std::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, container::ObjectsPair::XML_TAG_NAME );
}

template < typename T, typename R >
void xmlApi < std::pair < T, R > >::compose ( std::deque < sax::Token > & output, const std::pair < T, R > & input ) {
	return container::ObjectsPair::compose < T, R > ( output, input );
}

} /* namespace alib */

#endif /* OBJECTS_PAIR_H_ */
