/*
 * Set.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#include "ObjectsSet.h"

#include "Container.h"
#include "../object/Object.h"
#include <sstream>

namespace container {

ObjectsSet::ObjectsSet(std::set<alib::Object> set) : std::set<alib::Object>(std::move(set)) {

}

ObjectsSet::ObjectsSet() : std::set<alib::Object>() {

}

ContainerBase* ObjectsSet::clone() const {
	return new ObjectsSet(*this);
}

ContainerBase* ObjectsSet::plunder() && {
	return new ObjectsSet(std::move(*this));
}

int ObjectsSet::compare(const ObjectsSet& other) const {
	std::compare<std::set<alib::Object>> comp;
	return comp(static_cast<const std::set<alib::Object>>(*this), static_cast<const std::set<alib::Object>>(other));
}

void ObjectsSet::operator>>(std::ostream& os) const {
	os << "(ObjectsSet " << static_cast<const std::set<alib::Object>>(*this) << ")";
}

ObjectsSet::operator std::string() const {
	std::stringstream ss;
	ss << *this;
	return std::move(ss).str();
}

const std::string ObjectsSet::XML_TAG_NAME = "Set";

ObjectsSet ObjectsSet::parse(std::deque<sax::Token>::iterator& input) {
	return ObjectsSet(parseRaw<alib::Object>(input));
}

void ObjectsSet::compose(std::deque<sax::Token>& out) const {
	compose(out, *this);
}

} /* namespace container */

namespace alib {

auto objectsSetParserRegister = xmlApi<container::Container>::ParserRegister<container::ObjectsSet>();
auto objectsSetParserRegister2 = xmlApi<alib::Object>::ParserRegister<container::ObjectsSet>();

} /* namespace alib */
