/*
 * AlibException.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Martin Zak
 */

#include "AlibException.h"

#include <cstdlib>
#include <iostream>
#include <stdexcept>
#include <sstream>

#include <execinfo.h>
#ifdef DEBUG
	#include "../debug/simpleStacktrace.h"
#endif

#include "../sax/FromXMLParserHelper.h"
#include "../object/Object.h"
#include "../XmlApi.hpp"

namespace exception {

AlibException::AlibException ( ) {
	#ifdef DEBUG
		this->backtrace = std::simpleStacktrace();
	#else
		this->backtrace = "";
	#endif

	this->whatMessage += this->backtrace;
}

AlibException::AlibException ( std::string cause ) : AlibException { } {
	this->whatMessage += this->cause;

	this->cause = std::move(cause);
}

AlibException::AlibException ( std::string cause, std::string backtrace ) {
	this->whatMessage += this->backtrace;
	this->whatMessage += this->cause;

	this->backtrace = std::move(backtrace);
	this->cause = std::move(cause);
}

AlibException::~AlibException ( ) noexcept {

}

alib::ObjectBase* AlibException::clone() const {
	return new AlibException(*this);
}

alib::ObjectBase* AlibException::plunder() && {
	return new AlibException(std::move(*this));
}

const char * AlibException::what ( ) const noexcept {
	return whatMessage.c_str ( );
}

const std::string & AlibException::getCause ( ) const {
	return cause;
}

const std::string & AlibException::getBacktrace ( ) const {
	return backtrace;
}

int AlibException::compare(const AlibException& other) const {
	return this->whatMessage.compare(other.whatMessage);
}

void AlibException::operator>>(std::ostream& os) const {
	os << this->whatMessage;
}

AlibException::operator std::string() const {
	return this->whatMessage;
}

const std::string AlibException::XML_TAG_NAME = "AlibException";

AlibException AlibException::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, AlibException::XML_TAG_NAME);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "cause");
	std::string cause = "";
	if (input->getType() == sax::Token::TokenType::CHARACTER) {
		cause = std::move(*input).moveData();
		++input;
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "cause");
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "backtrace");
	std::string backtrace = "";
	if (input->getType() == sax::Token::TokenType::CHARACTER) {
		backtrace = std::move(*input).moveData();
		++input;
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "backtrace");
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, AlibException::XML_TAG_NAME);
	return AlibException(std::move(cause), std::move(backtrace));
}

void AlibException::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(AlibException::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	out.emplace_back("cause", sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(cause, sax::Token::TokenType::CHARACTER);
	out.emplace_back("cause", sax::Token::TokenType::END_ELEMENT);
	out.emplace_back("backtrace", sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(backtrace, sax::Token::TokenType::CHARACTER);
	out.emplace_back("backtrace", sax::Token::TokenType::END_ELEMENT);
	out.emplace_back(AlibException::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

} /* namespace exception */

namespace alib {

auto alibExceptionParserRegister2 = xmlApi<alib::Object>::ParserRegister<exception::AlibException>();

} /* namespace alib */
