/*
 * CFG.h
 *
 *  Created on: Nov 17, 2013
 *      Author: Jan Travnicek
 */

#ifndef CFG_H_
#define CFG_H_

#include "../GrammarBase.h"
#include "EpsilonFreeCFG.h"
#include <map>
#include <vector>
#include "../common/TerminalNonterminalAlphabetInitialSymbol.h"

namespace grammar {

/**
 * Context free grammar. Type 2 in Chomsky hierarchy. Produces context free language.
 */
class CFG : public GrammarBase, public TerminalNonterminalAlphabetInitialSymbol {
	std::map < alphabet::Symbol, std::set < std::vector < alphabet::Symbol > > > rules;

public:
	explicit CFG ( alphabet::Symbol initialSymbol );

	explicit CFG ( std::set < alphabet::Symbol > nonTerminalSymbols, std::set < alphabet::Symbol > terminalSymbols, alphabet::Symbol initialSymbol );

	explicit CFG ( const EpsilonFreeCFG & other );

	virtual GrammarBase * clone ( ) const;

	virtual GrammarBase * plunder ( ) &&;

	bool addRule ( alphabet::Symbol leftHandSide, std::vector < alphabet::Symbol > rightHandSide );

	const std::map < alphabet::Symbol, std::set < std::vector < alphabet::Symbol > > > & getRules ( ) const;

	bool removeRule ( const alphabet::Symbol & leftHandSide, const std::vector < alphabet::Symbol > & rightHandSide );

	bool addRawRule ( alphabet::Symbol leftHandSide, std::vector < alphabet::Symbol > rightHandSide );

	std::map < alphabet::Symbol, std::set < std::vector < alphabet::Symbol > > > getRawRules ( ) const;

	bool removeRawRule ( const alphabet::Symbol & leftHandSide, const std::vector < alphabet::Symbol > & rightHandSide );

	bool removeTerminalSymbol ( const alphabet::Symbol & symbol );

	bool removeNonterminalSymbol ( const alphabet::Symbol & symbol );

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const CFG & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static CFG parse ( std::deque < sax::Token >::iterator & input );
	static void parseRule ( std::deque < sax::Token >::iterator & input, CFG & grammar );

	void compose ( std::deque < sax::Token > & out ) const;
	void composeRules ( std::deque < sax::Token > & out ) const;
};

} /* namespace grammar */

namespace std {

template < >
struct compare < grammar::CFG > {
	int operator ()( const grammar::CFG & first, const grammar::CFG & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* CFG_H_ */
