/*
 * CSG.h
 *
 *  Created on: Nov 17, 2013
 *      Author: Jan Travnicek
 */

#ifndef CSG_H_
#define CSG_H_

#include "../GrammarBase.h"
#include <map>
#include <vector>
#include "../common/TerminalNonterminalAlphabetInitialSymbol.h"

namespace grammar {

/**
 * Epsilon free context free grammar. Type 2 in Chomsky hierarchy. Produces context free languages.
 */
class CSG : public GrammarBase, public TerminalNonterminalAlphabetInitialSymbol {
	std::map < std::tuple < std::vector < alphabet::Symbol >, alphabet::Symbol, std::vector < alphabet::Symbol > >, std::set < std::vector < alphabet::Symbol > > > rules;
	bool generatesEpsilon;

public:
	explicit CSG ( alphabet::Symbol initialSymbol );

	explicit CSG ( std::set < alphabet::Symbol > nonTerminalSymbols, std::set < alphabet::Symbol > terminalSymbols, alphabet::Symbol initialSymbol );

	virtual GrammarBase * clone ( ) const;

	virtual GrammarBase * plunder ( ) &&;

	bool addRule ( std::vector < alphabet::Symbol > lContext, alphabet::Symbol leftHandSide, std::vector < alphabet::Symbol > rContext, std::vector < alphabet::Symbol > rightHandSide );

	const std::map < std::tuple < std::vector < alphabet::Symbol >, alphabet::Symbol, std::vector < alphabet::Symbol > >, std::set < std::vector < alphabet::Symbol > > > & getRules ( ) const;

	bool removeRule ( const std::vector < alphabet::Symbol > & lContext, const alphabet::Symbol & leftHandSide, const std::vector < alphabet::Symbol > & rContext, const std::vector < alphabet::Symbol > & rightHandSide );

	bool removeTerminalSymbol ( const alphabet::Symbol & symbol );

	bool removeNonterminalSymbol ( const alphabet::Symbol & symbol );

	void setGeneratesEpsilon ( bool genEps );
	bool getGeneratesEpsilon ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const CSG & other ) const;

	virtual void operator >>( std::ostream & os ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static CSG parse ( std::deque < sax::Token >::iterator & input );
	static void parseRule ( std::deque < sax::Token >::iterator & input, CSG & grammar );

	void compose ( std::deque < sax::Token > & out ) const;
	void composeRules ( std::deque < sax::Token > & out ) const;
};

} /* namespace grammar */

namespace std {

template < >
struct compare < grammar::CSG > {
	int operator ()( const grammar::CSG & first, const grammar::CSG & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* CSG_H_ */
