/*
 * GraphFeatures.h
 *
 *  Created on: Jun 19, 2014
 *      Author: David Rosca
 */

#ifndef ADJACENCY_LIST_DIRECTED_GRAPH_H_
#define ADJACENCY_LIST_DIRECTED_GRAPH_H_

#include <list>
#include <unordered_map>

#include "IDirectedGraph.h"

namespace graph {

/*
 *  AdjacencyList representation
 *
 *  unordered_map:
 *    [ n1 ] -> list [ n2, n3, n4 ]
 *    [ n2 ] -> list [ n1, n1 ]      < multi-edge
 *    [ n3 ] -> list [ n4 ]
 *    [ n4 ] -> list [ n1, n2 ]
 *
 *  edges:
 *    n1 -> n2, n1 -> n3, n1 -> n4
 *    n2 -> n1, n2 -> n1
 *    ...
 */

class AdjacencyListDirectedGraph : public IDirectedGraph
{
public:
	std::set<Node> allNodes() const override;
	std::set<DirectedEdge> allEdges() const override;

	bool hasNode(const Node &node) const override;
	bool hasEdge(const DirectedEdge &edge) const override;

	bool hasEdge(const Node &from, const Node &to) const override;
	std::set<DirectedEdge> findEdges(const Node &from, const Node &to) const override;

	std::set<Node> neighbors(const Node &node) const override;
	std::set<DirectedEdge> neighborEdges(const Node &node) const override;

	bool addNode(const Node &node) override;
	bool removeNode(const Node &node) override;

	bool addEdge(const DirectedEdge &edge) override;
	bool removeEdge(const DirectedEdge &edge) override;

	void copy(IDirectedGraph *other) override;
	int compare(IDirectedGraph *other) const override;

private:
	std::set<DirectedEdge> edges;
	std::unordered_map<Node, std::list<Node>> adj;
};

} // namespace graph

#endif // ADJACENCY_LIST_DIRECTED_GRAPH_H_
