/*
 * GraphFeatures.h
 *
 *  Created on: Jun 19, 2014
 *      Author: David Rosca
 */

#ifndef DIRECTED_EDGE_H_
#define DIRECTED_EDGE_H_

#include "../common/Macros.h"
#include "../common/Node.h"

namespace graph {

class DirectedEdge {
public:
	explicit DirectedEdge(Node from, Node to);
	explicit DirectedEdge(Node from, Node to, label::Label name);
	explicit DirectedEdge(Node from, Node to, int number);
	explicit DirectedEdge(Node from, Node to, char character);
	explicit DirectedEdge(Node from, Node to, std::string name);

	DirectedEdge *clone() const;
	DirectedEdge *plunder() &&;

	const Node &getFromNode() const;
	const Node &getToNode() const;
	const label::Label &getName() const;

	bool operator < (const DirectedEdge& other) const;
	bool operator > (const DirectedEdge& other) const;
	bool operator == (const DirectedEdge& other) const;
	bool operator != (const DirectedEdge& other) const;
	int compare(const DirectedEdge &other) const;

	explicit operator std::string() const;

	friend std::ostream &operator<<(std::ostream &out, const DirectedEdge &node);

private:
	void dump(std::ostream &os) const;

	Node from;
	Node to;
	label::Label name;
};

} // namespace graph

GRAPH_DEFINE_STD_HASH(DirectedEdge)
GRAPH_DEFINE_STD_COMPARE(DirectedEdge)

#endif // DIRECTED_EDGE_H_
