/*
 * GraphFeatures.h
 *
 *  Created on: Jun 19, 2014
 *      Author: David Rosca
 */

#ifndef DIRECTED_GRAPH_H_
#define DIRECTED_GRAPH_H_

#include <unordered_map>
#include <vector>
#include <set>

#include "../GraphBase.h"
#include "../GraphRepresentation.h"
#include "../common/Macros.h"
#include "../common/Node.h"
#include "DirectedEdge.h"

namespace graph {

class IDirectedGraph;

class DirectedGraph : public GraphBase {
public:
	explicit DirectedGraph ( REPRESENTATION representation = REPRESENTATION::ADJACENCY_LIST );
	~DirectedGraph ( ) noexcept;

	DirectedGraph ( const DirectedGraph & other );
	DirectedGraph ( DirectedGraph && other ) noexcept;
	DirectedGraph & operator =( const DirectedGraph & other );
	DirectedGraph & operator =( DirectedGraph && other ) noexcept;

	GraphBase * clone ( ) const override;
	GraphBase * plunder ( ) && override;

	REPRESENTATION getRepresentation ( ) const;

	std::set < Node > getNodes ( ) const;
	std::set < DirectedEdge > getEdges ( ) const;

	bool hasNode ( const Node & node ) const;
	bool hasEdge ( const DirectedEdge & edge ) const;

	bool hasEdge ( const Node & from, const Node & to ) const;
	std::set < DirectedEdge > findEdges ( const Node & from, const Node & to ) const;

	std::set < Node > neighbors ( const Node & node ) const;
	std::set < DirectedEdge > neighborEdges ( const Node & node ) const;

	bool addNode ( const Node & node );
	bool addNode ( const Node & node, int value );
	bool removeNode ( const Node & node );

	bool addEdge ( const DirectedEdge & edge );
	bool addEdge ( const DirectedEdge & edge, int value );
	bool removeEdge ( const DirectedEdge & edge );

	int getNodeValue ( const Node & node ) const;
	void setNodeValue ( const Node & node, int value );

	int getEdgeValue ( const DirectedEdge & edge ) const;
	void setEdgeValue ( const DirectedEdge & edge, int value );

	std::unordered_map < Node, std::vector < Node > > getEmbedding ( ) const;
	void setEmbedding ( const std::unordered_map < Node, std::vector< Node > > embedding );

	int compare ( const ObjectBase & other ) const override {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	int compare ( const DirectedGraph & other ) const;

	void operator >>( std::ostream & out ) const override;

	explicit operator std::string ( ) const override;

	const static std::string XML_TAG_NAME;

	static DirectedGraph parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const override;

private:
	void init ( );

	REPRESENTATION representation;
	IDirectedGraph * impl;

	std::unordered_map < Node, int > nodeValues;
	std::unordered_map < DirectedEdge, int > edgeValues;
	std::unordered_map < Node, std::vector < Node > > embedding;

	friend class GraphToXMLComposer;
	friend class GraphToStringComposer;
};

} // namespace graph

GRAPH_DEFINE_STD_COMPARE ( DirectedGraph )

#endif // DIRECTED_GRAPH_H_
