/*
 * GraphFeatures.h
 *
 *  Created on: Jun 19, 2014
 *      Author: David Rosca
 */

#ifndef UNDIRECTED_EDGE_H_
#define UNDIRECTED_EDGE_H_

#include "../common/Macros.h"
#include "../common/Node.h"

namespace graph {

class UndirectedEdge {
public:
	explicit UndirectedEdge(Node first, Node second);
	explicit UndirectedEdge(Node first, Node second, label::Label name);
	explicit UndirectedEdge(Node first, Node second, int number);
	explicit UndirectedEdge(Node first, Node second, char character);
	explicit UndirectedEdge(Node first, Node second, std::string name);

	UndirectedEdge *clone() const;
	UndirectedEdge *plunder() &&;

	const Node &getFirstNode() const;
	const Node &getSecondNode() const;
	const label::Label &getName() const;

	bool operator < (const UndirectedEdge& other) const;
	bool operator > (const UndirectedEdge& other) const;
	bool operator == (const UndirectedEdge& other) const;
	bool operator != (const UndirectedEdge& other) const;
	int compare(const UndirectedEdge &other) const;

	explicit operator std::string() const;

	friend std::ostream &operator<<(std::ostream &out, const UndirectedEdge &node);

private:
	void dump(std::ostream &os) const;

	Node first;
	Node second;
	label::Label name;
};

} // namespace graph

GRAPH_DEFINE_STD_HASH(UndirectedEdge)
GRAPH_DEFINE_STD_COMPARE(UndirectedEdge)

#endif // UNDIRECTED_EDGE_H_
