/*
 * SuffixTrieNodeTerminatingSymbol.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Stepan Plachy
 */

#ifndef SUFFIX_TRIE_NODE_TERMINATING_SYMBOL_H_
#define SUFFIX_TRIE_NODE_TERMINATING_SYMBOL_H_

#include "../../alphabet/Symbol.h"
#include "../../primitive/Unsigned.h"
#include <map>
#include <set>
#include "common/TerminatingSymbolAlphabet.h"

namespace indexes {

/**
 * Represents a node in the ranked tree. Contains name of the symbol.
 */
class SuffixTrieNodeTerminatingSymbol {
protected:
	std::map < alphabet::Symbol, SuffixTrieNodeTerminatingSymbol * > children;

	SuffixTrieNodeTerminatingSymbol * parent;

	/**
	 * Parent tree contanining this instance of RankedTree
	 */
	const TerminatingSymbolAlphabet * parentTree;

	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::testSymbol() const
	 */
	bool testSymbol ( const alphabet::Symbol & symbol ) const;

	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::attachTree()
	 */
	bool attachTree ( const TerminatingSymbolAlphabet * tree );

	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::computeMinimalAlphabet()
	 */
	void computeMinimalAlphabet ( std::set < alphabet::Symbol > & alphabet ) const;

public:
	explicit SuffixTrieNodeTerminatingSymbol ( std::map < alphabet::Symbol, SuffixTrieNodeTerminatingSymbol * > children );

	SuffixTrieNodeTerminatingSymbol ( const SuffixTrieNodeTerminatingSymbol & other );
	SuffixTrieNodeTerminatingSymbol ( SuffixTrieNodeTerminatingSymbol && other ) noexcept;
	SuffixTrieNodeTerminatingSymbol & operator =( const SuffixTrieNodeTerminatingSymbol & other );
	SuffixTrieNodeTerminatingSymbol & operator =( SuffixTrieNodeTerminatingSymbol && other ) noexcept;
	~SuffixTrieNodeTerminatingSymbol ( ) noexcept;

	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::clone() const
	 */
	SuffixTrieNodeTerminatingSymbol * clone ( ) const;

	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::plunder() const
	 */
	SuffixTrieNodeTerminatingSymbol * plunder ( ) &&;

	/**
	 * @return children
	 */
	const std::map < const alphabet::Symbol, const SuffixTrieNodeTerminatingSymbol * > & getChildren ( ) const;

	/**
	 * @return children
	 */
	const std::map < alphabet::Symbol, SuffixTrieNodeTerminatingSymbol * > & getChildren ( );

	SuffixTrieNodeTerminatingSymbol & getChild ( const alphabet::Symbol & symbol );

	const SuffixTrieNodeTerminatingSymbol & getChild ( const alphabet::Symbol & symbol ) const;

	bool hasChild ( const alphabet::Symbol & symbol ) const;

	SuffixTrieNodeTerminatingSymbol & addChild ( alphabet::Symbol symbol, SuffixTrieNodeTerminatingSymbol node );

	SuffixTrieNodeTerminatingSymbol * getParent ( );

	const SuffixTrieNodeTerminatingSymbol * getParent ( ) const;

	void swap ( SuffixTrieNodeTerminatingSymbol & other );

	int compare ( const SuffixTrieNodeTerminatingSymbol & ) const;

	/**
	 * @copydoc SuffixTrieNodeTerminatingSymbol::operator>>() const
	 */
	void operator >>( std::ostream & out ) const;

	friend std::ostream & operator <<( std::ostream &, const SuffixTrieNodeTerminatingSymbol & node );

	explicit operator std::string ( ) const;

	friend class SuffixTrieTerminatingSymbol;
};

} /* namespace indexes */

#endif /* SUFFIX_TRIE_NODE_TERMINATING_SYMBOL_H_ */
