/*
 * UniqueLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "UniqueLabel.h"
#include <sstream>
#include "../sax/FromXMLParserHelper.h"
#include "Label.h"
#include "../object/Object.h"
#include "../XmlApi.hpp"

namespace label {

UniqueLabel::UniqueLabel(Label label, primitive::Integer id) : label(std::move(label)), id(std::move(id)) {

}

LabelBase* UniqueLabel::clone() const {
	return new UniqueLabel(*this);
}

LabelBase* UniqueLabel::plunder() && {
	return new UniqueLabel(std::move(*this));
}

const Label& UniqueLabel::getLabel() const {
	return label;
}

const primitive::Integer& UniqueLabel::getId() const {
	return id;
}

primitive::Integer& UniqueLabel::getId() {
	return id;
}

int UniqueLabel::compare(const UniqueLabel& other) const {
	int res = label.getData().compare(other.label.getData());
	if(res == 0) res = id.compare(other.id);
	return res;
}

void UniqueLabel::operator>>(std::ostream& out) const {
	out << "(UniqueLabel " << label << ", " << id << ")";
}

UniqueLabel::operator std::string() const {
	std::stringstream ss;
	ss << "{";
	ss << label;
	ss << ", ";
	ss << id;
	ss << "}";
	return std::move(ss).str();
}

const std::string UniqueLabel::XML_TAG_NAME = "UniqueLabel";

UniqueLabel UniqueLabel::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, UniqueLabel::XML_TAG_NAME);

	Label firstLabel = alib::xmlApi<Label>::parse(input);
	primitive::Integer secondLabel = alib::xmlApi<primitive::Integer>::parse(input);

	UniqueLabel data(std::move(firstLabel), std::move(secondLabel));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, UniqueLabel::XML_TAG_NAME);
	return data;
}

void UniqueLabel::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(UniqueLabel::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<label::Label>::compose(out, label);
	alib::xmlApi<primitive::Integer>::compose(out, id);
	out.emplace_back(UniqueLabel::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

LabelBase* UniqueLabel::next() const {
	return new UniqueLabel(label, id.next());
}

LabelBase* UniqueLabel::inc() && {
	id.inc();
	return NULL;
}

} /* namespace label */

namespace alib {

auto uniqueLabelParserRegister = xmlApi<label::Label>::ParserRegister<label::UniqueLabel>();
auto uniqueLabelParserRegister2 = xmlApi<alib::Object>::ParserRegister<label::UniqueLabel>();

} /* namespace alib */
