/*
 * Character.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "Character.h"
#include "../sax/FromXMLParserHelper.h"
#include "Primitive.h"
#include "../object/Object.h"

namespace primitive {

Character::Character(char data) : data(data) {

}

PrimitiveBase* Character::clone() const {
	return new Character(*this);
}

PrimitiveBase* Character::plunder() && {
	return new Character(std::move(*this));
}

char Character::getData() const {
	return data;
}

void Character::setData(char data) {
	this->data = data;
}

int Character::compare(const Character& other) const {
	return data - other.data;
}

void Character::operator>>(std::ostream& out) const {
	out << "(Character " << data << ")";
}

Character::operator std::string() const {
	return std::string(1, data);
}

const std::string Character::XML_TAG_NAME = "Character";

Character Character::parse(std::deque<sax::Token>::iterator& input) {
	return Character(parseRaw(input));
}

char Character::parseRaw(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, Character::XML_TAG_NAME);
	char data = sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER)[0];
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, Character::XML_TAG_NAME);
	return data;
}

void Character::compose(std::deque<sax::Token>& out) const {
	compose(out, this->getData());
}

void Character::compose(std::deque<sax::Token>& out, char primitive) {
	out.emplace_back(Character::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(std::string(1, primitive), sax::Token::TokenType::CHARACTER);
	out.emplace_back(Character::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

Character Character::next() const {
	return Character(data + 1);
}

void Character::inc() {
	data++;
}

} /* namespace primitive */

namespace alib {

char xmlApi<char>::parse(std::deque<sax::Token>::iterator& input) {
	return primitive::Character::parseRaw(input);
}

bool xmlApi<char>::first(const std::deque<sax::Token>::const_iterator& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, primitive::Character::XML_TAG_NAME);
}

void xmlApi<char>::compose(std::deque<sax::Token>& output, char data) {
	primitive::Character::compose(output, data);
}

auto characterParserRegister = xmlApi<primitive::Primitive>::ParserRegister<primitive::Character>();
auto characterParserRegister2 = xmlApi<alib::Object>::ParserRegister<primitive::Character>();

} /* namespace alib */

