/*
 * SaxComposeInterface.h
 *
 *  Created on: 8.8.2012
 *      Author: Martin Zak
 */

#ifndef SAX_COMPOSE_INTERFACE_H_
#define SAX_COMPOSE_INTERFACE_H_

#include <libxml/xmlwriter.h>
#include <deque>
#include "Token.h"

namespace sax {

/**
 * This class performs parsing of file or string containing XML. Contains callback
 * methods for libxml SAX parser.
 */
class SaxComposeInterface {
	static void xmlSAXUserPrint(xmlTextWriterPtr writer, const std::deque<Token>& in);
public:
	/**
	 * Parses the string containing XML.
	 * @param xmlIn input XML
	 * @param out list of tokens that are returned
	 * @throws AlibException when an error occurs (e.g. XML is not valid)
	 */
	static void printMemory(std::string& xmlOut, const std::deque<Token>& in);

	/**
	 * Parses the file containing XML.
	 * @param filename input XML
	 * @param out list of tokens that are returned
	 * @throws AlibException when an error occurs (e.g. file doesn't exist, XML is not valid)
	 */
	static void printFile(const std::string& filename, const std::deque<Token>& in);
	
	/**
	 * Parses the XML from stdin.
	 * @param out list of tokens that are returned
	 */
	static void printStdout(const std::deque<Token>& in);
	
	/**
	 * Parses the XML from stream.
	 * @param in input XML
	 * @param out list of tokens that are returned
	 */
	static void printStream(std::ostream& out, const std::deque<Token>& in);
};

} /* namespace sax */

#endif /* SAX_COMPOSE_INTERFACE_H_ */

