/*
 * SaxParseInterface.cpp
 *
 *  Created on: 8.8.2012
 *      Author: Martin Zak
 */

#include "SaxParseInterface.h"

#include <string>
#include <cstring>
#include <cstdlib>
#include <iostream>
#include <algorithm>
#include "../exception/AlibException.h"

namespace sax {

void SaxParseInterface::parseMemory(const std::string& xmlIn, std::deque<Token>& out) {
	xmlParserInputBufferPtr buf = xmlParserInputBufferCreateMem (xmlIn.c_str(), xmlIn.length(), XML_CHAR_ENCODING_NONE);
	xmlTextReaderPtr reader = xmlNewTextReader(buf, "");

	int result = SaxParseInterface::xmlSAXUserParse(reader, out);

	xmlFreeTextReader(reader);
	xmlFreeParserInputBuffer(buf);
	xmlCleanupCharEncodingHandlers();

	if (result != 0) {
		throw exception::AlibException("Cannot parse the XML file " + xmlIn);
	}
}

void SaxParseInterface::parseFile(const std::string& filename, std::deque<Token>& out) {
	xmlTextReaderPtr reader = xmlNewTextReaderFilename(filename.c_str());

	int result = SaxParseInterface::xmlSAXUserParse(reader, out);

	xmlFreeTextReader(reader);

	if (result != 0) {
		throw exception::AlibException("Cannot parse the XML file " + filename);
	}
}

void SaxParseInterface::parseStdin(std::deque<Token>& out) {
	SaxParseInterface::parseFile("-", out);
}

void SaxParseInterface::parseStream(std::istream& in, std::deque<Token>& out) {
	std::string input(std::istreambuf_iterator<char>(in), (std::istreambuf_iterator<char>()));
	SaxParseInterface::parseMemory(input, out);
}

int SaxParseInterface::xmlSAXUserParse(xmlTextReaderPtr reader, std::deque<Token>& out) {
	int ret = xmlTextReaderRead(reader);
	// measurements::start("Sax Parser", measurements::Type::INIT);
	while (ret == 1) {
		xmlChar* name = xmlTextReaderName(reader);
		xmlChar* value;
		bool empty;

		switch(xmlTextReaderNodeType(reader)) {
			case 1: // START_ELEMENT
				out.emplace_back((const char*) name, Token::TokenType::START_ELEMENT);
				empty = xmlTextReaderIsEmptyElement(reader);
				while(xmlTextReaderMoveToNextAttribute(reader)) {
					xmlChar* attrName = xmlTextReaderName(reader);
					xmlChar* attrValue = xmlTextReaderValue(reader);

					out.emplace_back((const char*) attrName, Token::TokenType::START_ATTRIBUTE);
					out.emplace_back((const char*) attrValue, Token::TokenType::CHARACTER);
					out.emplace_back((const char*) attrName, Token::TokenType::END_ATTRIBUTE);

					xmlFree(attrName);
					xmlFree(attrValue);
				}
				if(empty) out.emplace_back((const char*) name, Token::TokenType::END_ELEMENT);
				break;
			case 3: //CHARACTER
				value = xmlTextReaderValue(reader);
				if(! std::all_of(value, value + strlen((const char*) value), isspace)) out.emplace_back((const char*) value, Token::TokenType::CHARACTER);
				xmlFree(value);
				break;
			case 15: //END_EMENENT
				out.emplace_back((const char*) name, Token::TokenType::END_ELEMENT);
				break;
		}

		xmlFree(name);

		ret = xmlTextReaderRead(reader);
	}
	// measurements::end();
	return ret;
}

} /* namespace sax */
