/*
 * CyclicString.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef CYCLIC_STRING_H_
#define CYCLIC_STRING_H_

#include "StringBase.h"
#include <set>
#include <vector>

#include "../alphabet/Symbol.h"
#include "common/StringAlphabet.h"

namespace string {

class Epsilon;

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of CyclicStringElement.
 */
class CyclicString : public StringBase, public StringAlphabet {
	std::vector < alphabet::Symbol > m_Data;

public:
	explicit CyclicString ( );
	explicit CyclicString ( std::set < alphabet::Symbol > alphabet, std::vector < alphabet::Symbol > data );
	explicit CyclicString ( std::vector < alphabet::Symbol > data );
	explicit CyclicString ( const std::string & string );
	explicit CyclicString ( const char * string );
	explicit CyclicString ( const Epsilon & epsilon );

	virtual StringBase * clone ( ) const;
	virtual StringBase * plunder ( ) &&;

	virtual const std::set < alphabet::Symbol > & getAlphabet ( ) const {
		return StringAlphabet::getAlphabet ( );
	}

	void extendAlphabet ( const std::set < alphabet::Symbol > & symbols ) {
		addSymbolsToAlphabet ( symbols );
	}

	virtual bool removeSymbolFromAlphabet ( const alphabet::Symbol & symbol );

	/**
	 * @return List of symbols forming string (const version).
	 */
	const std::vector < alphabet::Symbol > & getContent ( ) const;

	void setContent ( std::vector < alphabet::Symbol > data );

	/**
	 * @return true if string is an empty word (vector length is 0)
	 */
	bool isEmpty ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const CyclicString & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static CyclicString parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
};

} /* namespace string */

namespace std {

template < >
struct compare < ::string::CyclicString > {
	int operator ()( const::string::CyclicString & first, const::string::CyclicString & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* CYCLIC_STRING_H_ */
