/*
 * LinearStringTerminatingSymbol.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef LINEAR_STRING_TERMINATING_SYMBOL_H_
#define LINEAR_STRING_TERMINATING_SYMBOL_H_

#include "StringBase.h"
#include <iostream>
#include <set>
#include <vector>

#include "../alphabet/Symbol.h"
#include "common/TerminatingSymbolStringAlphabet.h"

namespace string {

class LinearString;

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of LinearStringElement.
 */
class LinearStringTerminatingSymbol : public StringBase, public TerminatingSymbolStringAlphabet {
	std::vector < alphabet::Symbol > m_Data;

public:
	explicit LinearStringTerminatingSymbol ( alphabet::Symbol terminatingSymbol );
	explicit LinearStringTerminatingSymbol ( std::set < alphabet::Symbol > alphabet, alphabet::Symbol terminatingSymbol, std::vector < alphabet::Symbol > data );
	explicit LinearStringTerminatingSymbol ( alphabet::Symbol terminatingSymbol, std::vector < alphabet::Symbol > data );
	explicit LinearStringTerminatingSymbol ( alphabet::Symbol terminatingSymbol, const LinearString & string );

	virtual StringBase * clone ( ) const;
	virtual StringBase * plunder ( ) &&;

	virtual const std::set < alphabet::Symbol > & getAlphabet ( ) const {
		return TerminatingSymbolStringAlphabet::getAlphabet ( );
	}

	void extendAlphabet ( const std::set < alphabet::Symbol > & symbols ) {
		addSymbolsToAlphabet ( symbols );
	}

	virtual bool removeSymbolFromAlphabet ( const alphabet::Symbol & symbol );

	/**
	 * @param element to append
	 */
	void appendSymbol ( alphabet::Symbol symbol );

	/**
	 * @return List of symbols forming string (const version).
	 */
	const std::vector < alphabet::Symbol > & getContent ( ) const;

	void setContent ( std::vector < alphabet::Symbol > data );

	/**
	 * @return true if string is an empty word (vector length is 0)
	 */
	bool isEmpty ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const LinearStringTerminatingSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static LinearStringTerminatingSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
};

} /* namespace string */

namespace std {

template < >
struct compare < ::string::LinearStringTerminatingSymbol > {
	int operator ()( const::string::LinearStringTerminatingSymbol & first, const::string::LinearStringTerminatingSymbol & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* LINEAR_STRING_TERMINATING_SYMBOL_H_ */
