/*
 * StringAlphabet.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "StringAlphabet.h"

#include <algorithm>

namespace string {

const std::set<alphabet::Symbol>& StringAlphabet::getAlphabet() const {
	return alphabet;
}

bool StringAlphabet::addSymbolToAlphabet(alphabet::Symbol symbol) {
	return alphabet.insert(std::move(symbol)).second;
}

void StringAlphabet::addSymbolsToAlphabet(const std::set<alphabet::Symbol>& symbols) {
	for(const alphabet::Symbol& addedSymbol : symbols) {
		addSymbolToAlphabet(addedSymbol);
	}
}

void StringAlphabet::setAlphabet(std::set<alphabet::Symbol> newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(alphabet.begin(), alphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	for(const alphabet::Symbol& removedSymbol : removed) {
		removeSymbolFromAlphabet(removedSymbol);
	}

	alphabet = std::move(newSymbols);
}

} /* namespace string */

