/*
 * RankedAlphabet.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Stepan Plachy
 */

#ifndef RANKED_ALPHABET_H_
#define RANKED_ALPHABET_H_

#include <set>
#include "../../alphabet/RankedSymbol.h"

namespace tree {

/**
 * Abstract base class for all strings. Contains common elements of strings.
 */
class RankedAlphabet {
protected:
	std::set<alphabet::RankedSymbol> alphabet;

public:
	/**
	 * Adds input symbol to input alphabet.
	 * @param symbol Symbol to add
	 */
	bool addSymbolToAlphabet(alphabet::RankedSymbol symbol);

	/**
	 * Adds input symbols to input alphabet.
	 * @param symbols Symbol to add
	 */
	void addSymbolsToAlphabet(const std::set<alphabet::RankedSymbol>& symbols);

	/**
	 * Sets input symbols of the tree.
	 * @param symbols Symbols to set
	 */
	void setAlphabet(std::set<alphabet::RankedSymbol> symbols);

	/**
	 * Removes input symbol from the input alphabet.
	 * @param symbol Symbol to remove
	 * @throws AutomatonException when symbol is not present in input alphabet
	 * or when symbol is part of the transition
	 */
	virtual bool removeSymbolFromAlphabet(const alphabet::RankedSymbol& symbol) = 0;

	/**
	 * @return the input alphabet
	 */
	const std::set<alphabet::RankedSymbol>& getAlphabet() const;

};

} /* namespace tree */

#endif /* RANKED_ALPHABET_H_ */

