/*
 * TreeToXMLComposer.cpp
 *
 *  Created on: Nov 16, 2014
 *      Author: Stepan Plachy
 */

#include "TreeToXMLComposer.h"
#include "../../XmlApi.hpp"

namespace tree {

void TreeToXMLComposer::composeBar ( std::deque < sax::Token > & out, const alphabet::Symbol & symbol ) {
	out.emplace_back ( sax::Token ( "bar", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < alphabet::Symbol >::compose ( out, symbol );
	out.emplace_back ( sax::Token ( "bar", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeVariablesBar ( std::deque < sax::Token > & out, const alphabet::Symbol & symbol ) {
	out.emplace_back ( sax::Token ( "variablesBar", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < alphabet::Symbol >::compose ( out, symbol );
	out.emplace_back ( sax::Token ( "variablesBar", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeSubtreeWildcard ( std::deque < sax::Token > & out, const alphabet::Symbol & symbol ) {
	out.emplace_back ( sax::Token ( "subtreeWildcard", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < alphabet::Symbol >::compose ( out, symbol );
	out.emplace_back ( sax::Token ( "subtreeWildcard", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeSubtreeWildcard ( std::deque < sax::Token > & out, const alphabet::RankedSymbol & symbol ) {
	out.emplace_back ( sax::Token ( "subtreeWildcard", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < alphabet::RankedSymbol >::compose ( out, symbol );
	out.emplace_back ( sax::Token ( "subtreeWildcard", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeAlphabet ( std::deque < sax::Token > & out, const std::set < alphabet::RankedSymbol > & symbols ) {
	out.emplace_back ( sax::Token ( "rankedAlphabet", sax::Token::TokenType::START_ELEMENT ) );

	for ( const auto & symbol : symbols )
		alib::xmlApi < alphabet::RankedSymbol >::compose ( out, symbol );

	out.emplace_back ( sax::Token ( "rankedAlphabet", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeAlphabet ( std::deque < sax::Token > & out, const std::set < alphabet::Symbol > & symbols ) {
	out.emplace_back ( sax::Token ( "alphabet", sax::Token::TokenType::START_ELEMENT ) );

	for ( const auto & symbol : symbols )
		alib::xmlApi < alphabet::Symbol >::compose ( out, symbol );

	out.emplace_back ( sax::Token ( "alphabet", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeNode ( std::deque < sax::Token > & out, const RankedNode & node ) {
	out.emplace_back ( sax::Token ( "rankedNode", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < alphabet::RankedSymbol >::compose ( out, node.getSymbol ( ) );

	for ( const auto & child : node.getChildren ( ) )
		composeNode ( out, * child );

	out.emplace_back ( sax::Token ( "rankedNode", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeNode ( std::deque < sax::Token > & out, const UnrankedNode & node ) {
	out.emplace_back ( sax::Token ( "unrankedNode", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < alphabet::Symbol >::compose ( out, node.getSymbol ( ) );

	for ( const auto & child : node.getChildren ( ) )
		composeNode ( out, * child );

	out.emplace_back ( sax::Token ( "unrankedNode", sax::Token::TokenType::END_ELEMENT ) );
}

void TreeToXMLComposer::composeContent ( std::deque < sax::Token > & out, const std::vector < alphabet::Symbol > & content ) {
	out.emplace_back ( "content", sax::Token::TokenType::START_ELEMENT );

	for ( const auto & symbol : content )
		alib::xmlApi < alphabet::Symbol >::compose ( out, symbol );

	out.emplace_back ( "content", sax::Token::TokenType::END_ELEMENT );
}

void TreeToXMLComposer::composeContent ( std::deque < sax::Token > & out, const std::vector < alphabet::RankedSymbol > & content ) {
	out.emplace_back ( "content", sax::Token::TokenType::START_ELEMENT );

	for ( const auto & symbol : content )
		alib::xmlApi < alphabet::RankedSymbol >::compose ( out, symbol );

	out.emplace_back ( "content", sax::Token::TokenType::END_ELEMENT );
}

} /* namespace tree */
