/*
 * TreeToXMLComposer.h
 *
 *  Created on: Nov 16, 2014
 *      Author: Stepan Plachy
 */

#ifndef TREE_TO_XML_COMPOSER_H_
#define TREE_TO_XML_COMPOSER_H_

#include <deque>
#include <set>
#include <vector>
#include "../../alphabet/SymbolFeatures.h"
#include "../../sax/Token.h"
#include "../ranked/RankedNode.h"
#include "../unranked/UnrankedNode.h"

namespace tree {

/**
 * This class contains methods to print XML representation of tree to the output stream.
 */
class TreeToXMLComposer {
public:
	static void composeAlphabet(std::deque<sax::Token>& out, const std::set<alphabet::RankedSymbol>& symbols);
	static void composeAlphabet(std::deque<sax::Token>& out, const std::set<alphabet::Symbol>& symbols);

	static void composeBar(std::deque<sax::Token>& out, const alphabet::Symbol& symbol);
	static void composeVariablesBar(std::deque<sax::Token>& out, const alphabet::Symbol& symbol);

	static void composeSubtreeWildcard(std::deque<sax::Token>& out, const alphabet::Symbol& symbol);
	static void composeSubtreeWildcard(std::deque<sax::Token>& out, const alphabet::RankedSymbol& symbol);

	static void composeNode(std::deque<sax::Token>& out, const RankedNode& node);
	static void composeNode(std::deque<sax::Token>& out, const UnrankedNode& node);

	static void composeContent(std::deque<sax::Token>& out, const std::vector<alphabet::Symbol>& content);
	static void composeContent(std::deque<sax::Token>& out, const std::vector<alphabet::RankedSymbol>& content);
};

} /* namespace tree */

#endif /* TREE_TO_XML_COMPOSER_H_ */

