/*
 * UnrankedAlphabet.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Stepan Plachy
 */

#include "UnrankedAlphabet.h"

#include <algorithm>

namespace tree {

const std::set<alphabet::Symbol>& UnrankedAlphabet::getAlphabet() const {
	return alphabet;
}

bool UnrankedAlphabet::addSymbolToAlphabet(alphabet::Symbol symbol) {
	return alphabet.insert(std::move(symbol)).second;
}

void UnrankedAlphabet::addSymbolsToAlphabet(const std::set<alphabet::Symbol>& symbols) {
	for(const alphabet::Symbol& addedSymbol : symbols) {
		addSymbolToAlphabet(addedSymbol);
	}
}

void UnrankedAlphabet::setAlphabet(std::set<alphabet::Symbol> newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(alphabet.begin(), alphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	for(const alphabet::Symbol& removedSymbol : removed) {
		removeSymbolFromAlphabet(removedSymbol);
	}

	alphabet = std::move(newSymbols);
}

} /* namespace tree */

