/*
 * PrefixRankedBarTree.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef PREFIX_RANKED_BAR_TREE_H_
#define PREFIX_RANKED_BAR_TREE_H_

#include <set>
#include <vector>

#include "../../alphabet/RankedSymbol.h"
#include "../RankedTreeBase.h"
#include "../common/RankedAlphabet.h"
#include "RankedNode.h"

namespace tree {

class RankedTree;

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of LinearStringElement.
 */
class PrefixRankedBarTree : public RankedTreeBase, public RankedAlphabet {
	std::vector < alphabet::RankedSymbol > m_Data;

	alphabet::Symbol bar;

	void toPrefixRankedBar ( const RankedNode & node );

public:
	explicit PrefixRankedBarTree ( alphabet::Symbol bar, std::set < alphabet::RankedSymbol > alphabet, std::vector < alphabet::RankedSymbol > data );
	explicit PrefixRankedBarTree ( alphabet::Symbol bar, std::vector < alphabet::RankedSymbol > data );
	explicit PrefixRankedBarTree ( alphabet::Symbol bar, const RankedTree & tree );
	explicit PrefixRankedBarTree ( const RankedTree & tree );

	virtual RankedTreeBase * clone ( ) const;
	virtual RankedTreeBase * plunder ( ) &&;

	virtual bool removeSymbolFromAlphabet ( const alphabet::RankedSymbol & symbol );

	/**
	 * @return List of symbols forming tree (const version).
	 */
	const std::vector < alphabet::RankedSymbol > & getContent ( ) const;

	const std::set < alphabet::RankedSymbol > & getAlphabet ( ) const {
		return RankedAlphabet::getAlphabet ( );
	}

	void extendAlphabet ( const std::set < alphabet::RankedSymbol > & symbols ) {
		addSymbolsToAlphabet ( symbols );
	}

	const alphabet::Symbol & getBarSymbol ( ) const;

	void setContent ( std::vector < alphabet::RankedSymbol > data );

	void arityChecksum ( const std::vector < alphabet::RankedSymbol > & data );

	/**
	 * @return true if tree is an empty word (vector length is 0)
	 */
	bool isEmpty ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const PrefixRankedBarTree & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static PrefixRankedBarTree parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
};

} /* namespace tree */

namespace std {

template < >
struct compare < ::tree::PrefixRankedBarTree > {
	int operator ()( const::tree::PrefixRankedBarTree & first, const::tree::PrefixRankedBarTree & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* PREFIX_RANKED_BAR_TREE_H_ */
