/*
 * PrefixBarTree.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef PREFIX_BAR_TREE_H_
#define PREFIX_BAR_TREE_H_

#include <set>
#include <vector>

#include "../../alphabet/Symbol.h"
#include "../UnrankedTreeBase.h"
#include "../common/UnrankedAlphabet.h"
#include "UnrankedNode.h"

namespace tree {

class UnrankedTree;

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of LinearStringElement.
 */
class PrefixBarTree : public UnrankedTreeBase, public UnrankedAlphabet {
	std::vector < alphabet::Symbol > m_Data;

	alphabet::Symbol bar;

	void toPrefixBar ( const UnrankedNode & node );

public:
	explicit PrefixBarTree ( alphabet::Symbol bar, std::set < alphabet::Symbol > alphabet, std::vector < alphabet::Symbol > data );
	explicit PrefixBarTree ( alphabet::Symbol bar, std::vector < alphabet::Symbol > data );
	explicit PrefixBarTree ( alphabet::Symbol bar, const UnrankedTree & tree );
	explicit PrefixBarTree ( const UnrankedTree & tree );

	virtual UnrankedTreeBase * clone ( ) const;
	virtual UnrankedTreeBase * plunder ( ) &&;

	virtual bool removeSymbolFromAlphabet ( const alphabet::Symbol & symbol );

	/**
	 * @return List of symbols forming tree (const version).
	 */
	const std::vector < alphabet::Symbol > & getContent ( ) const;

	const std::set < alphabet::Symbol > & getAlphabet ( ) const {
		return UnrankedAlphabet::getAlphabet ( );
	}

	void extendAlphabet ( const std::set < alphabet::Symbol > & symbols ) {
		addSymbolsToAlphabet ( symbols );
	}

	const alphabet::Symbol & getBarSymbol ( ) const;

	void setContent ( std::vector < alphabet::Symbol > data );

	void arityChecksum ( const std::vector < alphabet::Symbol > & data );

	/**
	 * @return true if tree is an empty word (vector length is 0)
	 */
	bool isEmpty ( ) const;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const PrefixBarTree & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	const static std::string XML_TAG_NAME;

	static PrefixBarTree parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
};

} /* namespace tree */

namespace std {

template < >
struct compare < ::tree::PrefixBarTree > {
	int operator ()( const::tree::PrefixBarTree & first, const::tree::PrefixBarTree & second ) const {
		return first.compare ( second );
	}

};

} /* namespace std */

#endif /* PREFIX_BAR_TREE_H_ */
