/*
 * Trim.cpp
 *
 *  Created on: 23. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "Trim.h"
#include "UselessStatesRemover.h"
#include "UnreachableStatesRemover.h"

#include <exception/AlibException.h>
#include <automaton/FSM/ExtendedNFA.h>
#include <automaton/FSM/CompactNFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include "automaton/common/State.h"
#include "automaton/Automaton.h"

namespace automaton {

namespace simplify {

namespace efficient {

template<class T>
T Trim::trim( const T & fsm ) {
	return UnreachableStatesRemover::remove(UselessStatesRemover::remove(fsm));
}

auto TrimDFA = Trim::RegistratorWrapper<automaton::DFA, automaton::DFA>(Trim::getInstance(), Trim::trim);
auto TrimNFA = Trim::RegistratorWrapper<automaton::NFA, automaton::NFA>(Trim::getInstance(), Trim::trim);
auto TrimMultiInitialStateNFA = Trim::RegistratorWrapper<automaton::MultiInitialStateNFA, automaton::MultiInitialStateNFA>(Trim::getInstance(), Trim::trim);
auto TrimEpsilonNFA = Trim::RegistratorWrapper<automaton::EpsilonNFA, automaton::EpsilonNFA>(Trim::getInstance(), Trim::trim);
auto TrimCompactNFA = Trim::RegistratorWrapper<automaton::CompactNFA, automaton::CompactNFA>(Trim::getInstance(), Trim::trim);
auto TrimExtendedNFA = Trim::RegistratorWrapper<automaton::ExtendedNFA, automaton::ExtendedNFA>(Trim::getInstance(), Trim::trim);

automaton::Automaton Trim::trim(const automaton::Automaton& automaton) {
	return getInstance().dispatch(automaton.getData());
}

} /* namespace efficient */

} /* namespace simplify */

} /* namespace automaton */

