/*
 * Author: Radovan Cerveny
 */

#include "MeasurementFrames.hpp"

namespace measurements {

void CounterDataFrame::init ( unsigned frame_idx, measurements::stealth_vector < MeasurementFrame > & frames ) {
	MeasurementFrame & current_frame = frames[frame_idx];
	MeasurementFrame & parent_frame	 = frames[current_frame.parent_idx];

	if ( parent_frame.type != measurements::Type::ROOT )
		current_frame.counter.counters = parent_frame.counter.counters;
}

void CounterDataFrame::update ( unsigned frame_idx, measurements::stealth_vector < MeasurementFrame > & frames ) {
	MeasurementFrame & current_frame = frames[frame_idx];
	MeasurementFrame & parent_frame	 = frames[current_frame.parent_idx];
}

void CounterDataFrame::hint ( unsigned frame_idx, measurements::stealth_vector < MeasurementFrame > & frames, CounterHint hint ) {

	CounterHint::value_type delta;

	switch ( hint.type ) {
	case CounterHint::Type::ADD:
		delta = hint.value;
		break;

	case CounterHint::Type::INC:
		delta = 1;
		break;

	case CounterHint::Type::SUB:
		delta = -hint.value;
		break;

	case CounterHint::Type::DEC:
		delta = -1;
		break;
	}

	frames[frame_idx].counter.real_counters[hint.name] += delta;

	unsigned cur_idx = frame_idx;

	for ( ; ; ) {
		MeasurementFrame & current_frame = frames[cur_idx];

		if ( current_frame.type == measurements::Type::ROOT ) break;

		current_frame.counter.counters[hint.name] += delta;

		cur_idx = current_frame.parent_idx;
	}
}

void CounterDataFrame::xml_tokens ( std::deque < sax::Token > & tokens, unsigned idx, const measurements::stealth_vector < MeasurementFrame > & frames ) {
	const CounterDataFrame & cdf = frames[idx].counter;

	tokens.emplace_back ( "CounterData", sax::Token::TokenType::START_ELEMENT );

	tokens.emplace_back ( "Counters", sax::Token::TokenType::START_ELEMENT );
	for ( const auto & elem : cdf.counters) {
		tokens.emplace_back ( "Counter", sax::Token::TokenType::START_ELEMENT );
		tokens.emplace_back ( "Name", sax::Token::TokenType::START_ELEMENT );
		tokens.emplace_back ( measurements::to_string ( elem.first ), sax::Token::TokenType::CHARACTER );
		tokens.emplace_back ( "Name", sax::Token::TokenType::END_ELEMENT );
		tokens.emplace_back ( "Value", sax::Token::TokenType::START_ELEMENT );
		tokens.emplace_back ( std::to_string ( elem.second ), sax::Token::TokenType::CHARACTER );
		tokens.emplace_back ( "Value", sax::Token::TokenType::END_ELEMENT );
		tokens.emplace_back ( "Counter", sax::Token::TokenType::END_ELEMENT );
	}
	tokens.emplace_back ( "Counters", sax::Token::TokenType::END_ELEMENT );

	tokens.emplace_back ( "RealCounters", sax::Token::TokenType::START_ELEMENT );
	for ( const auto & elem : cdf.real_counters) {
		tokens.emplace_back ( "Counter", sax::Token::TokenType::START_ELEMENT );
		tokens.emplace_back ( "Name", sax::Token::TokenType::START_ELEMENT );
		tokens.emplace_back ( measurements::to_string ( elem.first ), sax::Token::TokenType::CHARACTER );
		tokens.emplace_back ( "Name", sax::Token::TokenType::END_ELEMENT );
		tokens.emplace_back ( "Value", sax::Token::TokenType::START_ELEMENT );
		tokens.emplace_back ( std::to_string ( elem.second ), sax::Token::TokenType::CHARACTER );
		tokens.emplace_back ( "Value", sax::Token::TokenType::END_ELEMENT );
		tokens.emplace_back ( "Counter", sax::Token::TokenType::END_ELEMENT );
	}
	tokens.emplace_back ( "RealCounters", sax::Token::TokenType::END_ELEMENT );

	tokens.emplace_back ( "CounterData", sax::Token::TokenType::END_ELEMENT );
}

std::ostream & operator <<( std::ostream & os, const CounterDataFrame & cdf ) {

	os << "(";
	for ( auto it = cdf.counters.begin ( ); it != cdf.counters.end ( ); ++it ) {
		os << it->first << " : " << it->second;

		if ( it != cdf.counters.end ( ) ) os << ", ";
	}
	os << "), (";
	for ( auto it = cdf.real_counters.begin ( ); it != cdf.real_counters.end ( ); ++it ) {
		os << it->first << " : " << it->second;

		if ( it != cdf.real_counters.end ( ) ) os << ", ";
	}
	os << ")";

	return os;
}

}
