/*
 * Author: Radovan Cerveny
 */

#include "MeasurementEngine.hpp"

namespace measurements {

MeasurementEngine MeasurementEngine::INSTANCE;

MeasurementEngine::MeasurementEngine ( ) {
	reset_measurements ( );
}

void MeasurementEngine::push_measurement_frame ( measurements::stealth_string frame_name, measurements::Type frame_type ) {
	unsigned parent_idx = frame_idx_stack.back ( );

	frames.emplace_back ( std::move ( frame_name ), frame_type, parent_idx );

	unsigned current_idx = frames.size ( ) - 1;

	frames[parent_idx].sub_idxs.push_back ( current_idx );

	frame_idx_stack.push_back ( current_idx );

	TimeDataFrame::init ( current_idx, frames );
	MemoryDataFrame::init ( current_idx, frames );
	CounterDataFrame::init ( current_idx, frames );
}

void MeasurementEngine::pop_measurement_frame ( ) {
	unsigned current_idx = frame_idx_stack.back ( );

	frame_idx_stack.pop_back ( );

	TimeDataFrame::update ( current_idx, frames );
	MemoryDataFrame::update ( current_idx, frames );
	CounterDataFrame::update ( current_idx, frames );
}

void MeasurementEngine::reset_measurements ( ) {
	frames.clear ( );
	frame_idx_stack.clear ( );

	frames.emplace_back ( "Root", measurements::Type::ROOT, 0 );
	frame_idx_stack.push_back ( 0 );
}

MeasurementResults MeasurementEngine::get_results ( ) const {
	return MeasurementResults ( frames );
}

template < >
void MeasurementEngine::hint ( MemoryHint mh ) {
	if ( ( frame_idx_stack.size ( ) == 0 ) || ( frames[frame_idx_stack.back ( )].type == measurements::Type::ROOT ) ) return;

	MemoryDataFrame::hint ( frame_idx_stack.back ( ), frames, std::move ( mh ) );
}

template < >
void MeasurementEngine::hint ( CounterHint ch ) {
	if ( ( frame_idx_stack.size ( ) == 0 ) || ( frames[frame_idx_stack.back ( )].type == measurements::Type::ROOT ) ) return;

	CounterDataFrame::hint ( frame_idx_stack.back ( ), frames, std::move ( ch ) );
}

}
