/*
 * Author: Radovan Cerveny
 */

#include <sstream>
#include "MeasurementFrames.hpp"

namespace measurements {

MeasurementFrame::MeasurementFrame ( measurements::stealth_string name, measurements::Type type, unsigned parent_idx ) : name ( std::move ( name ) ), type ( type ), parent_idx ( parent_idx ), time ( ), memory ( ) {
}

void MeasurementFrame::xml_tokens ( std::deque < sax::Token > & tokens, unsigned idx, const measurements::stealth_vector < MeasurementFrame > & frames ) {
	const MeasurementFrame & frame = frames[idx];

	tokens.emplace_back ( "MeasurementFrame", sax::Token::TokenType::START_ELEMENT );
	tokens.emplace_back ( "Name", sax::Token::TokenType::START_ELEMENT );
	tokens.emplace_back ( measurements::to_string ( frame.name ), sax::Token::TokenType::CHARACTER );
	tokens.emplace_back ( "Name", sax::Token::TokenType::END_ELEMENT );
	tokens.emplace_back ( "Type", sax::Token::TokenType::START_ELEMENT );
	tokens.emplace_back ( measurements::to_string ( frame.type ), sax::Token::TokenType::CHARACTER );
	tokens.emplace_back ( "Type", sax::Token::TokenType::END_ELEMENT );

	if ( frame.type != measurements::Type::ROOT ) {
		TimeDataFrame::xml_tokens ( tokens, idx, frames );
		MemoryDataFrame::xml_tokens ( tokens, idx, frames );
		CounterDataFrame::xml_tokens ( tokens, idx, frames );
	}

	tokens.emplace_back ( "SubFrames", sax::Token::TokenType::START_ELEMENT );

	for ( unsigned sub_idx : frame.sub_idxs )
		xml_tokens ( tokens, sub_idx, frames );

	tokens.emplace_back ( "SubFrames", sax::Token::TokenType::END_ELEMENT );

	tokens.emplace_back ( "MeasurementFrame", sax::Token::TokenType::END_ELEMENT );
}

std::ostream & operator <<( std::ostream & os, const MeasurementFrame & frame ) {

	std::stringstream ss;

	ss << "(TIME: " << frame.time << "), (MEM: " << frame.memory << "), (COUNTER: " << frame.counter << ")";

	os << std::make_tuple ( frame.name, frame.type, frame.parent_idx, frame.sub_idxs, ss.str ( ) );
	return os;
}

}
