/*
 * Author: Radovan Cerveny
 */
#include "MeasurementResults.hpp"

namespace measurements {

MeasurementResults::MeasurementResults ( const measurements::stealth_vector < MeasurementFrame > & frames ) : frames ( frames ) {
}

std::deque < sax::Token > MeasurementResults::xml_tokens ( ) const {
	std::deque < sax::Token > tokens;

	tokens.emplace_back ( "MeasurementResults", sax::Token::TokenType::START_ELEMENT );

	MeasurementFrame::xml_tokens ( tokens, 0, frames );

	tokens.emplace_back ( "MeasurementResults", sax::Token::TokenType::END_ELEMENT );
	return tokens;
}

void MeasurementResults::print_as_list ( std::ostream & os ) const {
	print_as_list ( os, 0 );
}

void MeasurementResults::print_as_list ( std::ostream & os, unsigned idx ) const {
	const MeasurementFrame & frame = frames[idx];

	if ( frame.type != measurements::Type::ROOT ) {
		os << frame;

		if ( idx != frames.size ( ) - 1 )
			os << ", ";
	}

	for ( unsigned sub_idx : frame.sub_idxs )
		print_as_list ( os, sub_idx );
}

void MeasurementResults::print_as_tree ( std::ostream & os ) const {
	std::string prefix;

	print_as_tree ( os, 0, prefix, false );
}

void MeasurementResults::print_as_tree ( std::ostream & os, unsigned idx, std::string & prefix, bool last_in_parent ) const {
	const MeasurementFrame & frame = frames[idx];

	if ( frame.type == measurements::Type::ROOT ) {
		if ( frame.sub_idxs.size ( ) ) {
			prefix = "|-";

			last_in_parent = false;

			for ( unsigned sub_idx : frame.sub_idxs ) {

				prefix[prefix.size ( ) - 2] = '|';
				prefix[prefix.size ( ) - 1] = '-';

				if ( sub_idx == frame.sub_idxs.front ( ) )
					prefix[prefix.size ( ) - 2] = '+';

				if ( sub_idx == frame.sub_idxs.back ( ) ) {
					prefix[prefix.size ( ) - 2] = '\\';
					last_in_parent = true;
				}

				print_as_tree ( os, sub_idx, prefix, last_in_parent );
			}
		}
	} else {
		if ( frame.sub_idxs.size ( ) ) {
			prefix += '+';
			os << prefix << frame << std::endl;
			prefix += '-';

			prefix[prefix.size ( ) - 3] = ' ';

			if ( last_in_parent )
				prefix[prefix.size ( ) - 4] = ' ';
			else
				prefix[prefix.size ( ) - 4] = '|';

			last_in_parent = false;

			for ( unsigned sub_idx : frame.sub_idxs ) {

				prefix[prefix.size ( ) - 2] = '|';
				prefix[prefix.size ( ) - 1] = '-';

				if ( sub_idx == frame.sub_idxs.back ( ) ) {
					prefix[prefix.size ( ) - 2] = '\\';
					last_in_parent = true;
				}

				print_as_tree ( os, sub_idx, prefix, last_in_parent );
			}

			prefix.erase ( prefix.size ( ) - 2 );

		} else {
			os << prefix << frame << std::endl;
		}
	}
}

const int MeasurementXalloc::FORMAT = std::ios::xalloc ( );

std::ostream & operator <<( std::ostream & os, const MeasurementResults & mr ) {
	MeasurementFormat mf = static_cast < MeasurementFormat > ( os.iword ( MeasurementXalloc::FORMAT ) );

	switch ( mf ) {
	case MeasurementFormat::LIST:
		mr.print_as_list ( os );
		break;

	case MeasurementFormat::TREE:
		mr.print_as_tree ( os );
		break;

	default: /* MeasurementFormat::XML: */
		sax::SaxComposeInterface::printStream ( os, mr.xml_tokens ( ) );
		break;
	}

	return os;
}

}
