/*
 * Author: Radovan Cerveny
 */

#ifndef MEASUREMENT_RESULTS_HPP_
#define MEASUREMENT_RESULTS_HPP_

#include <ostream>
#include "sax/SaxComposeInterface.h"
#include "MeasurementTypes.hpp"
#include "MeasurementFrames.hpp"

namespace measurements {

enum class MeasurementFormat {
	LIST = 1, TREE = 2,	XML = 4
};

struct MeasurementXalloc {
	static const int FORMAT;
};

struct MeasurementResults {
	measurements::stealth_vector < MeasurementFrame > frames;

	MeasurementResults ( const measurements::stealth_vector < MeasurementFrame > & );

	void print_as_list ( std::ostream & ) const;
	void print_as_tree ( std::ostream & ) const;

	std::deque < sax::Token > xml_tokens ( ) const;

private:
	void print_as_list ( std::ostream &, unsigned ) const;
	void print_as_tree ( std::ostream &, unsigned, std::string &, bool ) const;
};

std::ostream & operator <<( std::ostream &, const MeasurementResults & );

template < typename _CharT, typename _Traits >
inline std::basic_ostream < _CharT, _Traits > & operator <<( std::basic_ostream < _CharT, _Traits > & x, MeasurementFormat f ) {
	x.iword ( MeasurementXalloc::FORMAT ) = static_cast < int > ( f );
	return x;
}

}

#endif /* MEASUREMENT_RESULTS_HPP_ */
