/*
 * Author: Radovan Cerveny
 */

#include <sstream>
#include "MeasurementFrames.hpp"

using namespace std::chrono;

namespace measurements {

void TimeDataFrame::init ( unsigned frame_idx, measurements::stealth_vector < MeasurementFrame > & frames ) {
	MeasurementFrame & current_frame = frames[frame_idx];

	current_frame.time.start = high_resolution_clock::now ( );
}

void TimeDataFrame::update ( unsigned frame_idx, measurements::stealth_vector < MeasurementFrame > & frames ) {
	MeasurementFrame & current_frame = frames[frame_idx];

	current_frame.time.duration = duration_cast < microseconds > ( high_resolution_clock::now ( ) - current_frame.time.start );
	current_frame.time.real_duration += current_frame.time.duration;

	MeasurementFrame & parent_frame = frames[current_frame.parent_idx];

	parent_frame.time.real_duration -= current_frame.time.duration;
}

void TimeDataFrame::xml_tokens ( std::deque < sax::Token > & tokens, unsigned idx, const measurements::stealth_vector < MeasurementFrame > & frames ) {
	const TimeDataFrame & frame = frames[idx].time;

	tokens.emplace_back ( "TimeData", sax::Token::TokenType::START_ELEMENT );
	tokens.emplace_back ( "Duration", sax::Token::TokenType::START_ELEMENT );
	tokens.emplace_back ( std::to_string ( frame.duration.count ( ) ), sax::Token::TokenType::CHARACTER );
	tokens.emplace_back ( "Duration", sax::Token::TokenType::END_ELEMENT );
	tokens.emplace_back ( "RealDuration", sax::Token::TokenType::START_ELEMENT );
	tokens.emplace_back ( std::to_string ( frame.real_duration.count ( ) ), sax::Token::TokenType::CHARACTER );
	tokens.emplace_back ( "RealDuration", sax::Token::TokenType::END_ELEMENT );
	tokens.emplace_back ( "TimeData", sax::Token::TokenType::END_ELEMENT );
}

std::ostream & operator <<( std::ostream & os, const TimeDataFrame & tdf ) {
	os << tdf.duration << ", " << tdf.real_duration;
	return os;
}

std::ostream & operator <<( std::ostream & os, const std::chrono::microseconds & ms ) {
	os << ms.count ( ) << "ms";
	return os;
}

}
