/*
 * TreeFromRawParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "TreeFromRawParser.h"
#include "tree/Tree.h"
#include "tree/TreeException.h"
#include "tree/TreeClasses.h"
#include "alphabet/LabeledSymbol.h"
#include "alphabet/BarSymbol.h"
#include "label/Label.h"

namespace tree {

Tree TreeFromRawParser::parseTree(std::deque<sax::Token>::iterator& input) const {
	return parseTree(input, std::set<FEATURES>({FEATURES::RANKED_TREE, FEATURES::PREFIX_RANKED_TREE, FEATURES::PREFIX_RANKED_BAR_TREE, FEATURES::UNRANKED_TREE}));
}

Tree TreeFromRawParser::parseTree(std::deque<sax::Token>::iterator& input, const std::set<FEATURES>& features) const {
	UnrankedNode* root = NULL;
	if (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		root = this->parseContent(input);
	} else if(isTokenType(input, sax::Token::TokenType::CHARACTER)) {
		root = this->parseContentLeaf(input);
	} else {
		throw exception::AlibException("Invalid token stream");
	}

	UnrankedTree tree(std::move(*root));
	delete root;
	if(features.count(FEATURES::UNRANKED_TREE)) return Tree{tree};

	if(features.count(FEATURES::PREFIX_RANKED_TREE)) return Tree{PrefixRankedTree{RankedTree{tree}}};

	if(features.count(FEATURES::PREFIX_RANKED_BAR_TREE)) return Tree{PrefixRankedBarTree{alphabet::Symbol{alphabet::BarSymbol{}}, RankedTree{tree}}};

	if(features.count(FEATURES::RANKED_TREE)) return Tree{RankedTree{tree}};

	throw exception::AlibException("Invalid input");
}

UnrankedNode* TreeFromRawParser::parseContent(std::deque<sax::Token>::iterator& input) const {
	UnrankedNode* node = new UnrankedNode(alphabet::symbolFrom(popTokenData(input, sax::Token::TokenType::START_ELEMENT)), std::vector<UnrankedNode*> {});
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		UnrankedNode* child = this->parseContent(input);
		node->pushBackChild(std::move(*child));
		delete child;
	}
	if(isTokenType(input, sax::Token::TokenType::CHARACTER)) {
		UnrankedNode* child = this->parseContentLeaf(input);
		node->pushBackChild(std::move(*child));
		delete child;
	}
	popTokenData(input, sax::Token::TokenType::END_ELEMENT);
	return node;
}

UnrankedNode* TreeFromRawParser::parseContentLeaf(std::deque<sax::Token>::iterator& input) const {
	return new UnrankedNode(alphabet::symbolFrom(popTokenData(input, sax::Token::TokenType::CHARACTER)), std::vector<UnrankedNode*> {});
}

} /* namespace tree */
