/*
 * tuple.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef __TUPLE_HPP_
#define __TUPLE_HPP_

namespace std {

template<int I, class Tuple>
struct operator_shift_left_impl;

template<int I, class Tuple>
struct operator_shift_left_impl {
	static void operator_shift_left(ostream& out, const Tuple& t) {
		operator_shift_left_impl<I - 1, Tuple>::operator_shift_left(out, t);
		out << ", " << get<I>(t);
	}
};

template<class Tuple>
struct operator_shift_left_impl<0, Tuple> {
	static void operator_shift_left(ostream& out, const Tuple& t) {
		out << get<0>(t);
	}
};

template< class... Ts>
std::ostream& operator<<(std::ostream& out, const std::tuple<Ts...>& tuple) {
	out << "(";
	operator_shift_left_impl<tuple_size<std::tuple<Ts...>>::value - 1, std::tuple<Ts...>>::operator_shift_left(out, tuple);
	out << ")";
	return out;
}



template<int I, typename Tuple>
struct compareTupleHelper;

template<int I, typename Tuple>
struct compareTupleHelper {
	static int compHelp(const Tuple& t1, const Tuple& t2) {
		compare<typename tuple_element<I, Tuple>::type> comp;
		int res = comp(std::get<I>(t1), std::get<I>(t2));
		if(res != 0) return res;
		return compareTupleHelper<I - 1, Tuple>::compHelp(t1, t2);
	}
};

template<class Tuple>
struct compareTupleHelper<0, Tuple> {
	static int compHelp(const Tuple& t1, const Tuple& t2) {
		compare<typename tuple_element<0, Tuple>::type> comp;
		return comp(std::get<0>(t1), std::get<0>(t2));
	}
};


template<typename ... Ts>
struct compare<tuple<Ts...>> {
	int operator()(const tuple<Ts...>& first, const tuple<Ts...>& second) const {
		return compareTupleHelper<tuple_size<std::tuple<Ts...>>::value - 1, std::tuple<Ts...>>::compHelp(first, second);
	}
};



template<int I, typename Tie>
struct compareTieHelper;

template<int I, typename Tie>
struct compareTieHelper {
	static int compHelp(const Tie& t1, const Tie& t2) {
		compare<typename std::remove_const<typename std::remove_reference<typename tuple_element<I, Tie>::type>::type>::type> comp;
		int res = comp(std::get<I>(t1), std::get<I>(t2));
		if(res != 0) return res;
		return compareTieHelper<I - 1, Tie>::compHelp(t1, t2);
	}
};

template<class Tie>
struct compareTieHelper<0, Tie> {
	static int compHelp(const Tie& t1, const Tie& t2) {
		compare<typename std::remove_const<typename std::remove_reference<typename tuple_element<0, Tie>::type>::type>::type> comp;
		return comp(std::get<0>(t1), std::get<0>(t2));
	}
};


template<typename ... Ts>
struct compare<tuple<Ts&...>> {
	int operator()(const tuple<Ts&...>& first, const tuple<Ts&...>& second) const {
		return compareTieHelper<tuple_size<std::tuple<Ts&...>>::value - 1, std::tuple<Ts&...>>::compHelp(first, second);
	}
};

} /* namespace std */

#endif /* __TUPLE_HPP_ */

