/*
 * PrimitiveToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "PrimitiveToStringComposer.h"
#include "primitive/Integer.h"
#include "primitive/String.h"
#include "primitive/Character.h"
#include "primitive/Unsigned.h"
#include "primitive/Bool.h"

namespace primitive {

void PrimitiveToStringComposer::compose(std::ostream& out, const String& primitive) {
	auto replace = [](std::string& str, const std::string& what, const std::string& with) {
		size_t index = 0;
		while((index = str.find(what, index)) != std::string::npos) {
			str.replace(index, what.length(), with);
			index += with.length();
		}
	};
	
	std::string tmp = primitive.getData();
	replace(tmp, "'", "\\'" );
	out << '\'' << tmp << '\'';
}

PrimitiveToStringComposer::RegistratorWrapper<void, String> PrimitiveToStringComposerString = PrimitiveToStringComposer::RegistratorWrapper<void, String>(PrimitiveToStringComposer::getInstance(), PrimitiveToStringComposer::compose);

void PrimitiveToStringComposer::compose(std::ostream& out, const Character& primitive) {
	out << primitive.getData();
}

PrimitiveToStringComposer::RegistratorWrapper<void, Character> PrimitiveToStringComposerCharacter = PrimitiveToStringComposer::RegistratorWrapper<void, Character>(PrimitiveToStringComposer::getInstance(), PrimitiveToStringComposer::compose);

void PrimitiveToStringComposer::compose(std::ostream& out, const Integer& primitive) {
	out << primitive.getData();
}

PrimitiveToStringComposer::RegistratorWrapper<void, Integer> PrimitiveToStringComposerInteger = PrimitiveToStringComposer::RegistratorWrapper<void, Integer>(PrimitiveToStringComposer::getInstance(), PrimitiveToStringComposer::compose);

void PrimitiveToStringComposer::compose(std::ostream& out, const Unsigned& primitive) {
	out << primitive.getData();
}

PrimitiveToStringComposer::RegistratorWrapper<void, Unsigned> PrimitiveToStringComposerUnsigned = PrimitiveToStringComposer::RegistratorWrapper<void, Unsigned>(PrimitiveToStringComposer::getInstance(), PrimitiveToStringComposer::compose);

void PrimitiveToStringComposer::compose(std::ostream& out, const Bool& primitive) {
	out << (primitive.getData() ? "true" : "false");
}

PrimitiveToStringComposer::RegistratorWrapper<void, Bool> PrimitiveToStringComposerBool = PrimitiveToStringComposer::RegistratorWrapper<void, Bool>(PrimitiveToStringComposer::getInstance(), PrimitiveToStringComposer::compose);

void PrimitiveToStringComposer::compose(std::ostream& out, const Primitive& primitive) {
	getInstance().dispatch(out, primitive.getData());
}

} /* namespace primitive */
