/*
 * TreeToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "TreeToStringComposer.h"
#include <tree/ranked/RankedTree.h>
#include <tree/ranked/RankedPattern.h>
#include <tree/unranked/UnrankedTree.h>
#include <tree/unranked/UnrankedPattern.h>

#include "../StringApi.hpp"

namespace tree {

void TreeToStringComposer::compose ( std::ostream & out, const RankedTree & tree ) {
	compose ( out, tree.getRoot ( ) );
}

TreeToStringComposer::RegistratorWrapper < void, RankedTree > StringToStringComposerRankedTree = TreeToStringComposer::RegistratorWrapper < void, RankedTree > ( TreeToStringComposer::getInstance ( ), TreeToStringComposer::compose );

void TreeToStringComposer::compose ( std::ostream & out, const RankedNode & node ) {
	alib::stringApi < alphabet::Symbol >::compose ( out, node.getSymbol ( ).getSymbol ( ) );

	out << std::utos ( node.getSymbol ( ).getRank ( ).getData ( ) );

	for ( const RankedNode * child : node.getChildren ( ) ) {
		out << " ";
		compose ( out, * child );
	}
}

void TreeToStringComposer::compose ( std::ostream & out, const RankedPattern & tree ) {
	compose ( out, tree.getSubtreeWildcard ( ), tree.getRoot ( ) );
}

TreeToStringComposer::RegistratorWrapper < void, RankedPattern > StringToStringComposerRankedPattern = TreeToStringComposer::RegistratorWrapper < void, RankedPattern > ( TreeToStringComposer::getInstance ( ), TreeToStringComposer::compose );

void TreeToStringComposer::compose ( std::ostream & out, const alphabet::RankedSymbol & subtreeWildcard, const RankedNode & node ) {
	if ( node.getSymbol ( ) == subtreeWildcard ) {
		out << "#S";
	} else {
		alib::stringApi < alphabet::Symbol >::compose ( out, node.getSymbol ( ).getSymbol ( ) );

		out << std::utos ( node.getSymbol ( ).getRank ( ).getData ( ) );

		for ( const RankedNode * child : node.getChildren ( ) ) {
			out << " ";
			compose ( out, subtreeWildcard, * child );
		}
	}
}

void TreeToStringComposer::compose ( std::ostream & out, const UnrankedTree & tree ) {
	compose ( out, tree.getRoot ( ) );
}

TreeToStringComposer::RegistratorWrapper < void, UnrankedTree > StringToStringComposerUnrankedTree = TreeToStringComposer::RegistratorWrapper < void, UnrankedTree > ( TreeToStringComposer::getInstance ( ), TreeToStringComposer::compose );

void TreeToStringComposer::compose ( std::ostream & out, const UnrankedNode & node ) {
	alib::stringApi < alphabet::Symbol >::compose ( out, node.getSymbol ( ) );

	for ( const UnrankedNode * child : node.getChildren ( ) ) {
		out << " ";
		compose ( out, * child );
	}

	out << " |";
}

void TreeToStringComposer::compose ( std::ostream & out, const UnrankedPattern & tree ) {
	compose ( out, tree.getSubtreeWildcard ( ), tree.getRoot ( ) );
}

TreeToStringComposer::RegistratorWrapper < void, UnrankedPattern > StringToStringComposerUnrankedPattern = TreeToStringComposer::RegistratorWrapper < void, UnrankedPattern > ( TreeToStringComposer::getInstance ( ), TreeToStringComposer::compose );

void TreeToStringComposer::compose ( std::ostream & out, const alphabet::Symbol & subtreeWildcard, const UnrankedNode & node ) {
	if ( node.getSymbol ( ) == subtreeWildcard ) {
		out << "#S |";
	} else {
		alib::stringApi < alphabet::Symbol >::compose ( out, node.getSymbol ( ) );

		for ( const UnrankedNode * child : node.getChildren ( ) ) {
			out << " ";
			compose ( out, subtreeWildcard, * child );
		}

		out << " |";
	}
}

void TreeToStringComposer::compose ( std::ostream & out, const Tree & tree ) {
	getInstance ( ).dispatch ( out, tree.getData ( ) );
}

} /* namespace tree */
