#include "GraphTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "factory/StringDataFactory.hpp"

#define CPPUNIT_ASSERT_EQUAL_INT(a, b) CPPUNIT_ASSERT_EQUAL(a, (int)b)

#define RUN_TEST(t) \
	std::cout << "ADJACENCY_LIST" << std::endl; \
	t(graph::REPRESENTATION::ADJACENCY_LIST); \
	std::cout << "ADJACENCY_MATRIX" << std::endl; \
	t(graph::REPRESENTATION::ADJACENCY_MATRIX)

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(GraphTest, "graph" );
CPPUNIT_TEST_SUITE_REGISTRATION(GraphTest);

void GraphTest::testStringParser()
{
	RUN_TEST(testStringParser_impl);
}

void GraphTest::testStringParser_impl(graph::REPRESENTATION representation)
{
	// Common
	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");

	// Directed
	graph::DirectedGraph dg(representation);
	dg.addNode(n1);
	dg.addNode(n2);
	dg.addNode(n3);
	dg.addEdge(graph::DirectedEdge(n1, n2));
	dg.addEdge(graph::DirectedEdge(n1, n3));

	graph::Graph dg1(dg);
	std::string tmp = alib::StringDataFactory::toString(dg1);
	graph::Graph dg2 = alib::StringDataFactory::fromString<graph::Graph>(tmp);
	std::string tmp2 = alib::StringDataFactory::toString(dg2);
	std::cout << tmp << std::endl;
	std::cout << tmp2 << std::endl;
	CPPUNIT_ASSERT(dg1 == dg2);

	dg.setEdgeValue(graph::DirectedEdge(n1, n2), 1);
	dg.setEdgeValue(graph::DirectedEdge(n1, n3), 2);

	graph::Graph dg1_2(dg);
	tmp = alib::StringDataFactory::toString(dg1_2);
	graph::Graph dg2_2 = alib::StringDataFactory::fromString<graph::Graph>(tmp);
	CPPUNIT_ASSERT(dg1_2 == dg2_2);

	// Undirected
	graph::UndirectedGraph ug(representation);
	ug.addNode(n1);
	ug.addNode(n2);
	ug.addNode(n3);
	ug.addEdge(graph::UndirectedEdge(n1, n2));
	ug.addEdge(graph::UndirectedEdge(n1, n3));

	graph::Graph ug1(ug);
	tmp = alib::StringDataFactory::toString(ug1);
	graph::Graph ug2 = alib::StringDataFactory::fromString<graph::Graph>(tmp);
	CPPUNIT_ASSERT(ug1 == ug2);

	ug.setEdgeValue(graph::UndirectedEdge(n1, n2), 1);
	ug.setEdgeValue(graph::UndirectedEdge(n1, n3), 2);

	graph::Graph ug1_2(ug);
	tmp = alib::StringDataFactory::toString(ug1_2);
	graph::Graph ug2_2 = alib::StringDataFactory::fromString<graph::Graph>(tmp);
	CPPUNIT_ASSERT(ug1_2 == ug2_2);
}

