/*
 * GrammarStat.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Jan Travnicek
 */

#include "GrammarStat.h"

#include <exception/AlibException.h>

#include <grammar/Regular/RightRG.h>
#include <grammar/Regular/RightLG.h>

#include <factory/XmlDataFactory.hpp>
#include <container/ObjectsSet.h>
#include <container/ObjectsMap.h>
#include <container/ObjectsVariant.h>
#include <container/ObjectsVector.h>
#include <iostream>

void GrammarStat::stat ( const grammar::Grammar & grammar, const GrammarSettings & settings ) {
	getInstance ( ).dispatch ( grammar.getData ( ), settings );
}

void GrammarStat::stat ( const grammar::RightRG & grammar, const GrammarSettings & settings ) {
	switch ( settings.nonterminals ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( grammar.getNonterminalAlphabet ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << grammar.getNonterminalAlphabet ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		std::cout << grammar.getNonterminalAlphabet ( ).size ( ) << std::endl;
		alib::XmlDataFactory::toStdout ( grammar.getNonterminalAlphabet ( ) );
		break;

	case PrintingOptions::NOOP:
		break;
	}

	switch ( settings.terminals ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( grammar.getTerminalAlphabet ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << grammar.getTerminalAlphabet ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		std::cout << grammar.getTerminalAlphabet ( ).size ( ) << std::endl;
		alib::XmlDataFactory::toStdout ( grammar.getTerminalAlphabet ( ) );
		break;

	case PrintingOptions::NOOP:
		break;
	}

	switch ( settings.initialSymbols ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( grammar.getInitialSymbol ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << 1 << std::endl;
		break;

	case PrintingOptions::BOTH:
		std::cout << 1 << std::endl;
		alib::XmlDataFactory::toStdout ( grammar.getInitialSymbol ( ) );
		break;

	case PrintingOptions::NOOP:
		break;
	}

	switch ( settings.rules ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( grammar.getRules ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << grammar.getRules ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		std::cout << grammar.getRules ( ).size ( ) << std::endl;
		alib::XmlDataFactory::toStdout ( grammar.getRules ( ) );
		break;

	case PrintingOptions::NOOP:
		break;
	}
}

auto GrammarStatRightRG = GrammarStat::RegistratorWrapper < void, grammar::RightRG > ( GrammarStat::getInstance ( ), GrammarStat::stat );

void GrammarStat::stat ( const grammar::RightLG & grammar, const GrammarSettings & settings ) {
	switch ( settings.nonterminals ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( grammar.getNonterminalAlphabet ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << grammar.getNonterminalAlphabet ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		std::cout << grammar.getNonterminalAlphabet ( ).size ( ) << std::endl;
		alib::XmlDataFactory::toStdout ( grammar.getNonterminalAlphabet ( ) );
		break;

	case PrintingOptions::NOOP:
		break;
	}

	switch ( settings.terminals ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( grammar.getTerminalAlphabet ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << grammar.getTerminalAlphabet ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		std::cout << grammar.getTerminalAlphabet ( ).size ( ) << std::endl;
		alib::XmlDataFactory::toStdout ( grammar.getTerminalAlphabet ( ) );
		break;

	case PrintingOptions::NOOP:
		break;
	}

	switch ( settings.initialSymbols ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( grammar.getInitialSymbol ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << 1 << std::endl;
		break;

	case PrintingOptions::BOTH:
		std::cout << 1 << std::endl;
		alib::XmlDataFactory::toStdout ( grammar.getInitialSymbol ( ) );
		break;

	case PrintingOptions::NOOP:
		break;
	}

	switch ( settings.rules ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( grammar.getRules ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << grammar.getRules ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		std::cout << grammar.getRules ( ).size ( ) << std::endl;
		alib::XmlDataFactory::toStdout ( grammar.getRules ( ) );
		break;

	case PrintingOptions::NOOP:
		break;
	}
}

auto GrammarStatRightLG = GrammarStat::RegistratorWrapper < void, grammar::RightLG > ( GrammarStat::getInstance ( ), GrammarStat::stat );
