/*
 * RegExpStat.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Jan Travnicek
 */

#include "RegExpStat.h"

#include <exception/AlibException.h>

#include <regexp/formal/FormalRegExp.h>
#include <regexp/unbounded/UnboundedRegExp.h>

#include <factory/XmlDataFactory.hpp>
#include <iostream>
#include <container/ObjectsSet.h>

void RegExpStat::stat ( const regexp::RegExp & regexp, const RegExpSettings & settings ) {
	getInstance ( ).dispatch ( regexp.getData ( ), settings );
}

unsigned RegExpStat::countNodes ( const regexp::FormalRegExpElement & ) {
	throw exception::AlibException ( "unimplemented" );
}

void RegExpStat::stat ( const regexp::FormalRegExp & regexp, const RegExpSettings & settings ) {
	switch ( settings.alphabet ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( regexp.getAlphabet ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << regexp.getAlphabet ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		std::cout << regexp.getAlphabet ( ).size ( ) << std::endl;
		alib::XmlDataFactory::toStdout ( regexp.getAlphabet ( ) );
		break;

	case PrintingOptions::NOOP:
		break;
	}

	switch ( settings.nodes ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( & regexp.getRegExp ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << countNodes ( regexp.getRegExp ( ) ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		alib::XmlDataFactory::toStdout ( & regexp.getRegExp ( ) );
		std::cout << countNodes ( regexp.getRegExp ( ) ) << std::endl;
		break;

	case PrintingOptions::NOOP:
		break;
	}
}

auto RegExpStatFormalRegExp = RegExpStat::RegistratorWrapper < void, regexp::FormalRegExp > ( RegExpStat::getInstance ( ), RegExpStat::stat );

unsigned RegExpStat::countNodes ( const regexp::UnboundedRegExpElement & ) {
	throw exception::AlibException ( "unimplemented" );
}

void RegExpStat::stat ( const regexp::UnboundedRegExp & regexp, const RegExpSettings & settings ) {
	switch ( settings.alphabet ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( regexp.getAlphabet ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << regexp.getAlphabet ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		std::cout << regexp.getAlphabet ( ).size ( ) << std::endl;
		alib::XmlDataFactory::toStdout ( regexp.getAlphabet ( ) );
		break;

	case PrintingOptions::NOOP:
		break;
	}

	switch ( settings.nodes ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( & regexp.getRegExp ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << countNodes ( regexp.getRegExp ( ) ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		alib::XmlDataFactory::toStdout ( & regexp.getRegExp ( ) );
		std::cout << countNodes ( regexp.getRegExp ( ) ) << std::endl;
		break;

	case PrintingOptions::NOOP:
		break;
	}
}

auto RegExpStatUnboundedRegExp = RegExpStat::RegistratorWrapper < void, regexp::UnboundedRegExp > ( RegExpStat::getInstance ( ), RegExpStat::stat );
