/*
 * StringStat.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Jan Travnicek
 */

#include "StringStat.h"

#include <exception/AlibException.h>

#include <string/LinearString.h>
#include <string/CyclicString.h>

#include <factory/XmlDataFactory.hpp>
#include <iostream>
#include <container/ObjectsSet.h>
#include <container/ObjectsVector.h>

void StringStat::stat ( const string::String & string, const StringSettings & settings ) {
	getInstance ( ).dispatch ( string.getData ( ), settings );
}

void StringStat::stat ( const string::LinearString & string, const StringSettings & settings ) {
	switch ( settings.alphabet ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( string.getAlphabet ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << string.getAlphabet ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		std::cout << string.getAlphabet ( ).size ( ) << std::endl;
		alib::XmlDataFactory::toStdout ( string.getAlphabet ( ) );
		break;

	case PrintingOptions::NOOP:
		break;
	}

	switch ( settings.content ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( string.getContent ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << string.getContent ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		alib::XmlDataFactory::toStdout ( string.getContent ( ) );
		std::cout << string.getContent ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::NOOP:
		break;
	}
}

auto StringStatFormalString = StringStat::RegistratorWrapper < void, string::LinearString > ( StringStat::getInstance ( ), StringStat::stat );

void StringStat::stat ( const string::CyclicString & string, const StringSettings & settings ) {
	switch ( settings.alphabet ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( string.getAlphabet ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << string.getAlphabet ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		std::cout << string.getAlphabet ( ).size ( ) << std::endl;
		alib::XmlDataFactory::toStdout ( string.getAlphabet ( ) );
		break;

	case PrintingOptions::NOOP:
		break;
	}

	switch ( settings.content ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( string.getContent ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << string.getContent ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		alib::XmlDataFactory::toStdout ( string.getContent ( ) );
		std::cout << string.getContent ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::NOOP:
		break;
	}
}

auto StringStatUnboundedString = StringStat::RegistratorWrapper < void, string::CyclicString > ( StringStat::getInstance ( ), StringStat::stat );
