/*
 * TreeStat.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Jan Travnicek
 */

#include "TreeStat.h"

#include <exception/AlibException.h>

#include <tree/ranked/RankedTree.h>
#include <tree/unranked/UnrankedTree.h>

#include <factory/XmlDataFactory.hpp>
#include <iostream>
#include <container/ObjectsSet.h>

void TreeStat::stat ( const tree::Tree & tree, const TreeSettings & settings ) {
	getInstance ( ).dispatch ( tree.getData ( ), settings );
}

unsigned TreeStat::countNodes ( const tree::RankedNode & node ) {
	unsigned res = 1;

	for ( const tree::RankedNode * child : node.getChildren ( ) )
		res += countNodes ( * child );

	return res;
}

void TreeStat::stat ( const tree::RankedTree & tree, const TreeSettings & settings ) {
	switch ( settings.alphabet ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( tree.getAlphabet ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << tree.getAlphabet ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		std::cout << tree.getAlphabet ( ).size ( ) << std::endl;
		alib::XmlDataFactory::toStdout ( tree.getAlphabet ( ) );
		break;

	case PrintingOptions::NOOP:
		break;
	}

	switch ( settings.nodes ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( & tree.getRoot ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << countNodes ( tree.getRoot ( ) ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		alib::XmlDataFactory::toStdout ( & tree.getRoot ( ) );
		std::cout << countNodes ( tree.getRoot ( ) ) << std::endl;
		break;

	case PrintingOptions::NOOP:
		break;
	}
}

auto TreeStatFormalTree = TreeStat::RegistratorWrapper < void, tree::RankedTree > ( TreeStat::getInstance ( ), TreeStat::stat );

unsigned TreeStat::countNodes ( const tree::UnrankedNode & node ) {
	unsigned res = 1;

	for ( const tree::UnrankedNode * child : node.getChildren ( ) )
		res += countNodes ( * child );

	return res;
}

void TreeStat::stat ( const tree::UnrankedTree & tree, const TreeSettings & settings ) {
	switch ( settings.alphabet ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( tree.getAlphabet ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << tree.getAlphabet ( ).size ( ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		std::cout << tree.getAlphabet ( ).size ( ) << std::endl;
		alib::XmlDataFactory::toStdout ( tree.getAlphabet ( ) );
		break;

	case PrintingOptions::NOOP:
		break;
	}

	switch ( settings.nodes ) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout ( & tree.getRoot ( ) );
		break;

	case PrintingOptions::SIZE:
		std::cout << countNodes ( tree.getRoot ( ) ) << std::endl;
		break;

	case PrintingOptions::BOTH:
		alib::XmlDataFactory::toStdout ( & tree.getRoot ( ) );
		std::cout << countNodes ( tree.getRoot ( ) ) << std::endl;
		break;

	case PrintingOptions::NOOP:
		break;
	}
}

auto TreeStatUnboundedTree = TreeStat::RegistratorWrapper < void, tree::UnrankedTree > ( TreeStat::getInstance ( ), TreeStat::stat );
