/*
 * acat.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Martin Zak
 */

#include <tclap/CmdLine.h>
#include <common/GlobalData.h>
#include <measure>
#include <string>
#include <exception/AlibException.h>
#include <factory/XmlDataFactory.hpp>
#include <sax/SaxParseInterface.h>
#include <sax/ParserException.h>
#include <tree/Tree.h>
#include <tree/ranked/RankedTree.h>
#include <tree/unranked/UnrankedTree.h>

int main(int argc, char** argv) {
	try {
		TCLAP::CmdLine cmd("Tree nice print binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Input to nice print",		false,	"-",		"file");
		cmd.add( input );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		measurements::start("Overal", measurements::Type::OVERALL);
		measurements::start("Input read", measurements::Type::AUXILIARY);

		std::deque<sax::Token> tokens;
		if(input.isSet()) {
			if(input.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(tokens);
			} else {
				sax::SaxParseInterface::parseFile(input.getValue(), tokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(tokens);
		}

		if (alib::XmlDataFactory::first<tree::RankedTree>(tokens)) {
			const tree::RankedTree & tree = alib::XmlDataFactory::fromTokens<tree::RankedTree>(tokens);

			measurements::end();
			measurements::start("Output write", measurements::Type::AUXILIARY);

			tree.nicePrint(std::cout);
		} else if (alib::XmlDataFactory::first<tree::UnrankedTree>(tokens)) {
			const tree::UnrankedTree & tree = alib::XmlDataFactory::fromTokens<tree::UnrankedTree>(tokens);

			measurements::end();
			measurements::start("Output write", measurements::Type::AUXILIARY);

			tree.nicePrint(std::cout);
		} else throw exception::AlibException("no tree on input");

		measurements::end();
		measurements::end();

		if(measure.getValue()) std::clog << measurements::results() << std::endl;

		return 0;
	} catch(const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
